@@ -378,6 +378,7 @@ BELLE_SIP_PARSE(header_contact)
 belle_sip_header_contact_t* belle_sip_header_contact_create (const belle_sip_header_address_t* contact) {
 	belle_sip_header_contact_t* header = belle_sip_header_contact_new();
 	_belle_sip_object_copy(BELLE_SIP_OBJECT(header),BELLE_SIP_OBJECT(contact));
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_CONTACT); /*restaure header name*/
 	return header;
 }
@@ -417,7 +418,7 @@ unsigned int belle_sip_header_contact_not_equals(const belle_sip_header_contact_
 void belle_sip_header_contact_set_automatic(belle_sip_header_contact_t *a, int enabled){
 	a->automatic=enabled;
 }
- 
+
 int belle_sip_header_contact_get_automatic(const belle_sip_header_contact_t *a){
 	return a->automatic;
 }
@@ -478,7 +479,7 @@ belle_sip_header_from_t* belle_sip_header_from_create(const belle_sip_header_add
 		belle_sip_uri_set_port(uri,0);
 		belle_sip_uri_headers_clean(uri);
 	}
-
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_FROM); /*restore header name*/
 	if (tag) belle_sip_header_from_set_tag(header,tag);
 	return header;
@@ -547,6 +548,7 @@ belle_sip_header_to_t* belle_sip_header_to_create(const belle_sip_header_address
 		belle_sip_uri_set_port(uri,0);
 		belle_sip_uri_headers_clean(uri);
 	}
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_TO); /*restaure header name*/
 	if (tag) belle_sip_header_to_set_tag(header,tag);
 	return header;
@@ -623,14 +625,14 @@ int belle_sip_header_user_agent_get_products_as_string(const belle_sip_header_us
 	size_t result = 0;
 	belle_sip_error_code error=BELLE_SIP_OK;
 	belle_sip_list_t* list = user_agent->products;
-	
+
 	for(;list!=NULL;list=list->next){
 		error=belle_sip_snprintf(value,value_size,&result,"%s ",(const char *)list->data);
 		if (error!=BELLE_SIP_OK) return -1;
 	}
-	if (result>0) value[result]='\0'; /*remove last space */
+	if (result>0) value[result-1]='\0'; /*remove last space */
 
-	return result-1;
+	return (int)result-1;
 }
 
 /**************************
@@ -898,6 +900,7 @@ BELLE_SIP_PARSE(header_route)
 belle_sip_header_route_t* belle_sip_header_route_create(const belle_sip_header_address_t* route) {
 	belle_sip_header_route_t* header= belle_sip_header_route_new();
 	_belle_sip_object_copy((belle_sip_object_t*)header,(belle_sip_object_t*)route);
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_ROUTE); /*restore header name*/
 	return header;
 }
@@ -961,7 +964,7 @@ BELLE_SIP_PARSE(header_service_route)
 */
 struct _belle_sip_header_content_length  {
 	belle_sip_header_t header;
-	unsigned int content_length;
+	size_t content_length;
 };
 
 static void belle_sip_header_content_length_destroy(belle_sip_header_content_length_t* content_length) {
@@ -975,15 +978,15 @@ static void belle_sip_header_content_length_clone(belle_sip_header_content_lengt
 belle_sip_error_code belle_sip_header_content_length_marshal(belle_sip_header_content_length_t* content_length, char* buff, size_t buff_size, size_t *offset) {
 	belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(content_length), buff, buff_size, offset);
 	if (error!=BELLE_SIP_OK) return error;
-	error=belle_sip_snprintf(buff,buff_size,offset,"%i",content_length->content_length);
+	error=belle_sip_snprintf(buff,buff_size,offset,FORMAT_SIZE_T,content_length->content_length);
 	if (error!=BELLE_SIP_OK) return error;
 	return error;
 }
 
 BELLE_SIP_NEW_HEADER(header_content_length,header,BELLE_SIP_CONTENT_LENGTH)
 BELLE_SIP_PARSE(header_content_length)
-GET_SET_INT(belle_sip_header_content_length,content_length,unsigned int)
-belle_sip_header_content_length_t* belle_sip_header_content_length_create (int content_length)  {
+GET_SET_INT(belle_sip_header_content_length,content_length,size_t)
+belle_sip_header_content_length_t* belle_sip_header_content_length_create (size_t content_length)  {
 	belle_sip_header_content_length_t* obj;
 	obj = belle_sip_header_content_length_new();
 	belle_sip_header_content_length_set_content_length(obj,content_length);
@@ -1535,6 +1538,7 @@ BELLE_SIP_PARSE(header_##name)\
 belle_sip_header_##name##_t* belle_sip_header_##name##_create(const belle_sip_header_address_t* address) { \
 	belle_sip_header_##name##_t* header= belle_sip_header_##name##_new();\
 	_belle_sip_object_copy((belle_sip_object_t*)header,(belle_sip_object_t*)address);\
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/\
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),header_name);  \
 	return header;\
 }
@@ -1668,7 +1672,7 @@ BELLESIP_EXPORT time_t belle_sip_header_date_get_time(belle_sip_header_date_t *o
 	char tmp2[16] ={0};
 	int i,j;
 	time_t seconds;
-#ifdef BELLE_SIP_WINDOWS_UNIVERSAL
+#if defined(BELLE_SIP_WINDOWS_UNIVERSAL) || defined(BELLE_SIP_MSC_VER_GREATER_19)
 	long adjust_timezone;
 #else
 	time_t adjust_timezone;
@@ -1702,7 +1706,7 @@ success:
 	adjust_timezone = 0;
 #else
 	seconds = mktime(&ret);
-#ifdef BELLE_SIP_WINDOWS_UNIVERSAL
+#if defined(BELLE_SIP_WINDOWS_UNIVERSAL) || defined(BELLE_SIP_MSC_VER_GREATER_19)
 	_get_timezone(&adjust_timezone);
 #else
 	adjust_timezone = timezone;
@@ -1772,6 +1776,7 @@ BELLE_SIP_PARSE(header_p_preferred_identity)
 belle_sip_header_p_preferred_identity_t* belle_sip_header_p_preferred_identity_create (const belle_sip_header_address_t* p_preferred_identity) {
 	belle_sip_header_p_preferred_identity_t* header = belle_sip_header_p_preferred_identity_new();
 	_belle_sip_object_copy(BELLE_SIP_OBJECT(header),BELLE_SIP_OBJECT(p_preferred_identity));
+	belle_sip_header_set_next(BELLE_SIP_HEADER(header),NULL); /*make sure only one header is kept*/
 	belle_sip_header_set_name(BELLE_SIP_HEADER(header),BELLE_SIP_P_PREFERRED_IDENTITY); /*restaure header name*/
 	return header;
 }
