@@ -28,6 +28,7 @@
 #include "belle-sip/belle-sip.h"
 
 #include "port.h"
+#include <bctoolbox/port.h>
 
 #ifdef HAVE_CONFIG_H
 
@@ -202,7 +203,7 @@ BELLE_SIP_DECLARE_VPTR(belle_http_request_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_response_t);
 BELLE_SIP_DECLARE_VPTR(belle_generic_uri_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_callbacks_t);
-BELLE_SIP_DECLARE_VPTR(belle_tls_verify_policy_t);
+BELLE_SIP_DECLARE_VPTR(belle_tls_crypto_config_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_header_authorization_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_event_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_supported_t);
@@ -222,8 +223,6 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_dual_resolver_context_t,belle_sip_resolver_context_t)
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
-typedef void (*belle_sip_source_remove_callback_t)(belle_sip_source_t *);
-
 struct belle_sip_source{
 	belle_sip_object_t base;
 	belle_sip_list_t node;
@@ -264,11 +263,6 @@ void belle_sip_source_set_notify(belle_sip_source_t *s, belle_sip_source_func_t
 #define belle_sip_new(type) (type*)belle_sip_malloc(sizeof(type))
 #define belle_sip_new0(type) (type*)belle_sip_malloc0(sizeof(type))
 
-#define belle_list_next(elem) ((elem)->next)
-belle_sip_list_t *belle_sip_list_new(void *data);
-belle_sip_list_t*  belle_sip_list_append_link(belle_sip_list_t* elem,belle_sip_list_t *new_elem);
-belle_sip_list_t *belle_sip_list_delete_custom(belle_sip_list_t *list, belle_sip_compare_func compare_func, const void *user_data);
-belle_sip_list_t*  _belle_sip_list_remove(belle_sip_list_t* first, void *data, int warn_if_not_found);
 
 #define belle_sip_list_next(elem) ((elem)->next)
 
@@ -285,7 +279,7 @@ struct belle_sip_dict {
 #define MAX(a,b)	((a)>(b) ? (a) : (b))
 
 
-BELLESIP_INTERNAL_EXPORT char * belle_sip_concat (const char *str, ...);
+#define belle_sip_concat bctbx_concat
 
 
 /*parameters accessors*/
@@ -523,13 +517,15 @@ struct belle_sip_stack{
 	int dscp;
 	char *dns_user_hosts_file; /* used to load additional hosts file for tests */
 	char *dns_resolv_conf; /*used to load custom resolv.conf, for tests*/
-	unsigned char dns_srv_enabled;
+	belle_sip_list_t *dns_servers; /*used when dns servers are supplied by app layer*/
 	/*http proxy stuff to be used by both http and sip provider*/
 	char *http_proxy_host;
 	int http_proxy_port;
-	char *http_proxy_username; /*for futur use*/
-	char *http_proxy_passwd; /*for futur use*/
+	char *http_proxy_username; /*for future use*/
+	char *http_proxy_passwd; /*for future use*/
 
+	unsigned char dns_srv_enabled;
+	unsigned char dns_search_enabled;
 };
 
 belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port);
@@ -555,6 +551,7 @@ struct belle_sip_provider{
 	unsigned char rport_enabled; /*0 if rport should not be set in via header*/
 	unsigned char nat_helper;
 	unsigned char unconditional_answer_enabled;
+	unsigned short unconditional_answer;
 };
 
 BELLESIP_INTERNAL_EXPORT belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
@@ -565,6 +562,7 @@ void belle_sip_provider_add_server_transaction(belle_sip_provider_t *prov, belle
 BELLESIP_INTERNAL_EXPORT belle_sip_server_transaction_t * belle_sip_provider_find_matching_server_transaction(belle_sip_provider_t *prov,belle_sip_request_t *req);
 void belle_sip_provider_remove_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
 void belle_sip_provider_set_transaction_terminated(belle_sip_provider_t *p, belle_sip_transaction_t *t);
+void *belle_sip_transaction_get_application_data_internal(const belle_sip_transaction_t *t);
 belle_sip_channel_t * belle_sip_provider_get_channel(belle_sip_provider_t *p, const belle_sip_hop_t *hop);
 void belle_sip_provider_add_dialog(belle_sip_provider_t *prov, belle_sip_dialog_t *dialog);
 void belle_sip_provider_remove_dialog(belle_sip_provider_t *prov, belle_sip_dialog_t *dialog);
@@ -584,6 +582,9 @@ typedef struct listener_ctx{
 #define BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(t,callback,event) \
 		BELLE_SIP_PROVIDER_INVOKE_LISTENERS((t)->is_internal?(t)->provider->internal_listeners:(t)->provider->listeners,callback,event)
 
+#define BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_DIALOG(d,callback,event) \
+		BELLE_SIP_PROVIDER_INVOKE_LISTENERS((d)->is_internal?(d)->provider->internal_listeners:(d)->provider->listeners,callback,event)
+
 #define BELLE_SIP_PROVIDER_INVOKE_LISTENERS(listeners,callback,event) \
 	BELLE_SIP_INVOKE_LISTENERS_ARG((listeners),belle_sip_listener_t,callback,(event))
 
@@ -671,6 +672,9 @@ static BELLESIP_INLINE void belle_sip_transaction_stop_timer(belle_sip_transacti
 	belle_sip_main_loop_remove_source(obj->provider->stack->ml,timer);
 }
 
+int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_client_transaction_t *trans
+																	  , belle_sip_request_t *notify);
+
 void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t);
 
 void belle_sip_transaction_set_dialog(belle_sip_transaction_t *t, belle_sip_dialog_t *dialog);
@@ -809,18 +813,21 @@ struct belle_sip_dialog{
 	belle_sip_transaction_t* last_transaction;
 	belle_sip_header_privacy_t* privacy;
 	belle_sip_list_t *queued_ct;/* queued client transactions*/
+	unsigned int remote_invite_cseq; /*needed because multiple trans can be handled whithin invite transaction (I.E UPDATE, PRACK,etc*/
 	unsigned char is_server;
 	unsigned char is_secure;
 	unsigned char terminate_on_bye;
 	unsigned char needs_ack;
-	unsigned int remote_invite_cseq; /*needed because multiple trans can be handled whithin invite transaction (I.E UPDATE, PRACK,etc*/
+	unsigned char is_expired;
 	unsigned char pending_trans_checking_enabled; /*use to disabled pending transaction check at request creation (testing)*/
+	unsigned char is_internal; /*Internal dialogs are those created by refreshers. */
 };
 
 belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t);
 belle_sip_dialog_t * belle_sip_provider_create_dialog_internal(belle_sip_provider_t *prov, belle_sip_transaction_t *t,unsigned int check_last_resp);
 int belle_sip_dialog_is_authorized_transaction(const belle_sip_dialog_t *dialog,const char* method) ;
 /*returns 1 if message belongs to the dialog, 0 otherwise */
+int belle_sip_dialog_is_null_dialog_with_matching_subscribe(belle_sip_dialog_t *obj, const char *call_id, const char *local_tag, belle_sip_request_t *notify);
 int _belle_sip_dialog_match(belle_sip_dialog_t *obj, const char *call_id, const char *local_tag, const char *remote_tag);
 int belle_sip_dialog_match(belle_sip_dialog_t *obj, belle_sip_message_t *msg, int as_uas);
 int belle_sip_dialog_update(belle_sip_dialog_t *obj,belle_sip_transaction_t* transaction, int as_uas);
@@ -887,6 +894,7 @@ belle_sdp_##object_type##_t* belle_sdp_##object_type##_parse (const char* value)
 struct belle_sip_dialog_terminated_event{
 	belle_sip_provider_t *source;
 	belle_sip_dialog_t *dialog;
+	int is_expired;
 };
 
 struct belle_sip_io_error_event{
@@ -948,7 +956,7 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
 
 /*
  * returns a char, even if entry is escaped*/
-int belle_sip_get_char (const char*a,int n,char*out);
+size_t belle_sip_get_char (const char*a,size_t n,char*out);
 /*return an escaped string*/
 BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buff) ;
 BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_userpasswd(const char* buff) ;
@@ -991,7 +999,7 @@ belle_sip_error_code belle_sip_headers_marshal(belle_sip_message_t *message, cha
  ******************************/
 typedef struct _belle_sip_header_extension belle_sip_header_extension_t;
 
-belle_sip_header_extension_t* belle_sip_header_extension_new();
+belle_sip_header_extension_t* belle_sip_header_extension_new(void);
 
 belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
 belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
@@ -1006,6 +1014,8 @@ void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extensio
 
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t,belle_sip_object_t)
+	void (*begin_transfer)(belle_sip_body_handler_t *obj);
+	void (*end_transfer)(belle_sip_body_handler_t *obj);
 	void (*chunk_recv)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size);
 	int (*chunk_send)(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t * size);
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
@@ -1028,6 +1038,8 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t,belle_sip_body_handler_t)
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
+void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, void *user_data, size_t transfered, size_t expected_total);
+
 /**
  * file manipulation
  */
