@@ -224,10 +224,15 @@ belle_sip_source_t * belle_sip_timeout_source_new(belle_sip_source_func_t func,
 	return belle_sip_socket_source_new(func,data,(belle_sip_socket_t)-1,0,timeout_value_ms);
 }
 
-unsigned long belle_sip_source_get_id(belle_sip_source_t *s){
+unsigned long belle_sip_source_get_id(const belle_sip_source_t *s){
 	return s->id;
 }
-
+void * belle_sip_source_get_user_data(const belle_sip_source_t *s) {
+	return s->data;
+}
+void belle_sip_source_set_user_data(belle_sip_source_t *s, void *user_data) {
+	s->data = user_data;
+}
 int belle_sip_source_set_events(belle_sip_source_t* source, int event_mask) {
 	source->events = event_mask;
 	return 0;
@@ -244,7 +249,7 @@ struct belle_sip_main_loop{
 	belle_sip_object_pool_t *pool;
 	int nsources;
 	int run;
-	int in_iterate;
+	int in_loop;
 };
 
 void belle_sip_main_loop_remove_source(belle_sip_main_loop_t *ml, belle_sip_source_t *source){
@@ -291,7 +296,7 @@ void belle_sip_main_loop_add_source(belle_sip_main_loop_t *ml, belle_sip_source_
 		source->expire_ms=belle_sip_time_ms()+source->timeout;
 	}
 	source->cancelled=FALSE;
-	ml->sources=belle_sip_list_append_link(ml->sources,&source->node);
+	ml->sources=belle_sip_list_prepend_link(ml->sources,&source->node);
 	ml->nsources++;
 }
 
@@ -325,11 +330,15 @@ void belle_sip_source_set_timeout(belle_sip_source_t *s, unsigned int value_ms){
 	}
 	s->timeout=value_ms;
 }
-
+void belle_sip_source_set_remove_cb(belle_sip_source_t *s, belle_sip_source_remove_callback_t func) {
+	s->on_remove=func;
+}
 unsigned int belle_sip_source_get_timeout(const belle_sip_source_t *s){
 	return s->timeout;
 }
-
+void belle_sip_source_cancel(belle_sip_source_t *s){
+	if (s) s->cancelled=TRUE;
+}
 
 static int match_source_id(const void *s, const void *pid){
 	if ( ((belle_sip_source_t*)s)->id==(unsigned long)(intptr_t)pid){
@@ -348,10 +357,10 @@ belle_sip_source_t *belle_sip_main_loop_find_source(belle_sip_main_loop_t *ml, u
 
 void belle_sip_main_loop_cancel_source(belle_sip_main_loop_t *ml, unsigned long id){
 	belle_sip_source_t *s=belle_sip_main_loop_find_source(ml,id);
-	if (s) s->cancelled=TRUE;
+	belle_sip_source_cancel(s);
 }
 
-void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
+static void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	size_t pfd_size = ml->nsources * sizeof(belle_sip_pollfd_t);
 	belle_sip_pollfd_t *pfd=(belle_sip_pollfd_t*)belle_sip_malloc0(pfd_size);
 	int i=0;
@@ -365,12 +374,6 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	int can_clean=belle_sip_object_pool_cleanable(ml->pool); /*iterate might not be called by the thread that created the main loop*/ 
 	belle_sip_object_pool_t *tmp_pool=NULL;
 	
-	if (ml->in_iterate){
-		belle_sip_warning("belle_sip_main_loop_iterate([%p]): reentrancy detected, doing nothing.",ml);
-		return;
-	}
-	ml->in_iterate=TRUE;
-	
 	if (!can_clean){
 		/*Push a temporary pool for the time of the iterate loop*/
 		tmp_pool=belle_sip_object_pool_push();
@@ -439,12 +442,14 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 		s=(belle_sip_source_t*)elem->data;
 		next=elem->next;
 		if (!s->cancelled){
-			char *objdesc=belle_sip_object_to_string((belle_sip_object_t*)s);
-			if (s->timeout>0) {
+			
+			if (s->timeout > 0 && belle_sip_log_level_enabled(BELLE_SIP_LOG_DEBUG)) {
 				/*to avoid too many traces*/ 
-				belle_sip_debug("source %s notified revents=%u, timeout=%i",objdesc,revents,s->timeout); 
+				char *objdesc=belle_sip_object_to_string((belle_sip_object_t*)s);
+				belle_sip_debug("source %s notified revents=%u, timeout=%i",objdesc,revents,s->timeout);
+				belle_sip_free(objdesc);
 			}
-			belle_sip_free(objdesc);
+			
 			ret=s->notify(s->data,s->revents);
 			if (ret==BELLE_SIP_STOP || s->oneshot){
 				/*this source needs to be removed*/
@@ -468,16 +473,19 @@ void belle_sip_main_loop_iterate(belle_sip_main_loop_t *ml){
 	else if (tmp_pool) belle_sip_object_unref(tmp_pool);
 end:
 	belle_sip_free(pfd);
-	
-	ml->in_iterate=FALSE;
 }
 
 void belle_sip_main_loop_run(belle_sip_main_loop_t *ml){
-	ml->run=1;
+	if (ml->in_loop){
+		belle_sip_warning("belle_sip_main_loop_run(): reentrancy detected, doing nothing");
+		return;
+	}
+	ml->run = TRUE;
+	ml->in_loop = TRUE;
 	while(ml->run){
 		belle_sip_main_loop_iterate(ml);
-		if (ml->in_iterate) break;
 	}
+	ml->in_loop = FALSE;
 }
 
 int belle_sip_main_loop_quit(belle_sip_main_loop_t *ml){
