@@ -53,7 +53,10 @@ static void add_new_object(belle_sip_object_t *obj){
 
 static void remove_free_object(belle_sip_object_t *obj){
 	if (belle_sip_leak_detector_enabled && !belle_sip_leak_detector_inhibited){
-		all_objects=_belle_sip_list_remove(all_objects,obj,FALSE); /*it may fail if the leak detector was inhibitted at the time the object was created*/
+		belle_sip_list_t* it;
+		it=belle_sip_list_find(all_objects,obj);
+		if (it)
+			all_objects = belle_sip_list_delete_link(all_objects,it); /*it may fail if the leak detector was inhibitted at the time the object was created*/
 	}
 }
 
@@ -66,7 +69,7 @@ void belle_sip_object_enable_leak_detector(int enable){
 }
 
 int belle_sip_object_get_object_count(void){
-	return belle_sip_list_size(all_objects);
+	return (int)belle_sip_list_size(all_objects);
 }
 
 void belle_sip_object_flush_active_objects(void){
@@ -304,7 +307,7 @@ static void belle_sip_object_data_destroy(void* data)
 int belle_sip_object_data_set( belle_sip_object_t *obj, const char* name, void* data, belle_sip_data_destroy destroy_func )
 {
 	int ret = 0;
-	struct _belle_sip_list*  list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
+	belle_sip_list_t*  list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
 	struct belle_sip_object_data* entry = (list_entry)? list_entry->data : NULL;
 
 	if( entry == NULL){
@@ -330,7 +333,7 @@ int belle_sip_object_data_set( belle_sip_object_t *obj, const char* name, void*
 
 void* belle_sip_object_data_get( belle_sip_object_t *obj, const char* name )
 {
-	struct _belle_sip_list*  list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
+	belle_sip_list_t  *list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
 	struct belle_sip_object_data* entry = (list_entry)? list_entry->data : NULL;
 
 	return entry? entry->data : NULL;
@@ -338,7 +341,7 @@ void* belle_sip_object_data_get( belle_sip_object_t *obj, const char* name )
 
 int belle_sip_object_data_remove( belle_sip_object_t *obj, const char* name)
 {
-	struct _belle_sip_list*  list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
+	belle_sip_list_t  *list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
 	struct belle_sip_object_data* entry = (list_entry)? list_entry->data : NULL;
 	if( entry ){
 		belle_sip_free(entry->name);
@@ -357,7 +360,7 @@ int belle_sip_object_data_exists( const belle_sip_object_t *obj, const char* nam
 
 void* belle_sip_object_data_grab( belle_sip_object_t* obj, const char* name)
 {
-	struct _belle_sip_list*  list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
+	belle_sip_list_t  *list_entry = belle_sip_list_find_custom(obj->data_store, belle_sip_object_data_find, name);
 	struct belle_sip_object_data* entry = (list_entry)? list_entry->data : NULL;
 	void* data =NULL;
 
@@ -385,7 +388,7 @@ void belle_sip_object_data_clone( const belle_sip_object_t* src, belle_sip_objec
 
 void belle_sip_object_data_merge( const belle_sip_object_t* src, belle_sip_object_t* dst, belle_sip_data_clone clone_func)
 {
-	struct _belle_sip_list* list = src->data_store;
+	belle_sip_list_t *list = src->data_store;
 	struct belle_sip_object_data* it = NULL;
 	void* cloned_data = NULL;
 
@@ -488,18 +491,19 @@ static belle_sip_error_code checked_marshal(belle_sip_object_vptr_t *vptr, belle
 		if (p[i]=='\0') break;
 	}
 	written=i-initial_offset;
-	if (error==BELLE_SIP_BUFFER_OVERFLOW){
+	if (error==BELLE_SIP_OK){
+		if (written!=(*offset-initial_offset) && written!=(buff_size-initial_offset-1)){ /*this is because snprintf won't allow you to write a non null character at the end of the buffer*/
+			belle_sip_fatal("Object of type %s marshalled %i bytes but said it marshalled %i bytes !",
+				vptr->type_name,(int)written,(int)(*offset-initial_offset));
+		}
+		memcpy(buff+initial_offset,p+initial_offset,*offset-initial_offset);
+	}else if (error==BELLE_SIP_BUFFER_OVERFLOW){
 		belle_sip_error("Object of type %s commited a buffer overflow by marshalling %i bytes",
 			vptr->type_name,(int)(*offset-initial_offset));
-	} else if (error!=BELLE_SIP_OK){
+	}else{
 		belle_sip_error("Object of type %s produced an error during marshalling: %i",
 			vptr->type_name,error);
-	}
-	if (written!=(*offset-initial_offset) && written!=(buff_size-initial_offset-1)){ /*this is because snprintf won't allow you to write a non null character at the end of the buffer*/
-		belle_sip_fatal("Object of type %s marshalled %i bytes but said it marshalled %i bytes !",
-			vptr->type_name,(int)written,(int)(*offset-initial_offset));
-	}
-	memcpy(buff+initial_offset,p+initial_offset,*offset-initial_offset);
+	} 
 	belle_sip_free(p);
 	return error;
 }
@@ -548,13 +552,13 @@ char* belle_sip_object_to_string(void* _obj) {
 	}else{
 		char buff[BELLE_SIP_MAX_TO_STRING_SIZE];
 		size_t offset=0;
-		belle_sip_error_code error = belle_sip_object_marshal(obj,buff,sizeof(buff),&offset);
+		belle_sip_error_code error = belle_sip_object_marshal(obj,buff,sizeof(buff)-1,&offset);
 		if (error==BELLE_SIP_BUFFER_OVERFLOW){
 			belle_sip_message("belle_sip_object_to_string(): temporary buffer is too short while doing to_string() for %s, retrying", obj->vptr->type_name);
-			return belle_sip_object_to_alloc_string(obj,get_hint_size(2*offset));
+			return belle_sip_object_to_alloc_string(obj,get_hint_size(2*(int)offset));
 		}
 		buff[offset]='\0';
-		obj->vptr->tostring_bufsize_hint=get_hint_size(2*offset);
+		obj->vptr->tostring_bufsize_hint=get_hint_size(2*(int)offset);
 		return belle_sip_strdup(buff);
 	}
 }
@@ -719,8 +723,8 @@ static void cleanup_pool_stack(void *data){
 		 * Instead, we mark them as detached, so that when the thread that will attempt to destroy them will do it,
 		 * we'll accept (since anyway these object pool are no longer needed.
 		 */
-		belle_sip_warning("There were still [%i] object pools for thread [%lu] while the thread exited. ",
-				 belle_sip_list_size(*pool_stack),belle_sip_thread_self_id());
+		belle_sip_warning("There were still [%u] object pools for thread [%lu] while the thread exited. ",
+			(unsigned int)belle_sip_list_size(*pool_stack),belle_sip_thread_self_id());
 		belle_sip_list_free_with_data(*pool_stack,(void (*)(void*)) belle_sip_object_pool_detach_from_thread);
 	}
 	*pool_stack=NULL;
