@@ -64,7 +64,7 @@ static void belle_sip_dns_srv_destroy(belle_sip_dns_srv_t *obj){
 		obj->a_resolver=NULL;
 	}
 	if (obj->a_results){
-		belle_sip_freeaddrinfo(obj->a_results);
+		bctbx_freeaddrinfo(obj->a_results);
 		obj->a_results=NULL;
 	}
 }
@@ -168,8 +168,22 @@ void belle_sip_resolver_context_init(belle_sip_resolver_context_t *obj, belle_si
 	belle_sip_init_sockets(); /* Need to be called for DNS resolution to work on Windows platform. */
 }
 
+static int dns_resconf_nameservers_from_list(struct dns_resolv_conf *resconf, const belle_sip_list_t *l) {
+	int max_servers = sizeof(resconf->nameserver)/sizeof(struct sockaddr_storage);
+	int i;
+	const belle_sip_list_t *elem;
+
+	for (i = 0, elem = l; i < max_servers && elem != NULL; elem = elem->next) {
+		int error = dns_resconf_pton(&resconf->nameserver[i], (const char *) elem->data);
+		if (error == 0) ++i;
+	}
+
+	return i > 0 ? 0 : -1;
+}
+
 static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx) {
 	const char *path;
+	const belle_sip_list_t *servers;
 	int error;
 
 	if (ctx->resconf)
@@ -180,8 +194,13 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 		return NULL;
 	}
 	
-	path=belle_sip_stack_get_dns_resolv_conf_file(ctx->base.stack);
-	if (!path){
+	path = belle_sip_stack_get_dns_resolv_conf_file(ctx->base.stack);
+	servers = ctx->base.stack->dns_servers;
+	
+	if (servers){
+		belle_sip_message("%s using application supplied dns server list.", __FUNCTION__);
+		error = dns_resconf_nameservers_from_list(ctx->resconf, servers);
+	}else if (!path){
 #if defined(USE_FIXED_NAMESERVERS)
 		error = dns_resconf_load_fixed_nameservers(ctx->resconf);
 		if (error) {
@@ -329,7 +348,7 @@ static belle_sip_dns_srv_t *srv_elect_one(belle_sip_list_t *srv_list){
 	rand_number=belle_sip_random() % sum; /*random number choosen in the range of the sum of weights*/
 	for(elem=srv_list;elem!=NULL;elem=elem->next){
 		srv=(belle_sip_dns_srv_t*)elem->data;
-		if (rand_number>=srv->cumulative_weight)
+		if (rand_number<=srv->cumulative_weight)
 			return srv;
 	}
 	return (belle_sip_dns_srv_t*)srv_list->data;
@@ -417,7 +436,7 @@ static void append_dns_result(belle_sip_simple_resolver_context_t *ctx, struct a
 		return;
 	}
 	if (ctx->flags & AI_V4MAPPED) family=AF_INET6;
-	*ai_list = ai_list_append(*ai_list, belle_sip_ip_address_to_addrinfo(family, host, ctx->port));
+	*ai_list = ai_list_append(*ai_list, bctbx_ip_address_to_addrinfo(family, SOCK_STREAM, host, ctx->port));
 	belle_sip_message("%s resolved to %s", ctx->name, host);
 }
 
@@ -428,6 +447,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 	int error;
 	unsigned char simulated_timeout=0;
 	int timeout=belle_sip_stack_get_dns_timeout(ctx->base.stack);
+	unsigned char search_enabled = belle_sip_stack_dns_search_enabled(ctx->base.stack);
 
 	/*Setting timeout to 0 can be used to simulate DNS timeout*/
 	if ((revents!=0) && timeout==0){
@@ -440,6 +460,7 @@ static int resolver_process_data(belle_sip_simple_resolver_context_t *ctx, unsig
 		notify_results(ctx);
 		return BELLE_SIP_STOP;
 	}
+	dns_res_enable_search(ctx->R, search_enabled);
 	/*belle_sip_message("resolver_process_data(): revents=%i",revents);*/
 	error = dns_res_check(ctx->R);
 	if (!error) {
@@ -533,7 +554,7 @@ static void * _resolver_getaddrinfo_thread(void *ptr) {
 		belle_sip_error("getaddrinfo DNS resolution of %s failed: %s", ctx->name, gai_strerror(err));
 	} else if (!ctx->getaddrinfo_cancelled) {
 		do {
-			append_dns_result(ctx, &ctx->getaddrinfo_ai_list, res->ai_addr, res->ai_addrlen);
+			append_dns_result(ctx, &ctx->getaddrinfo_ai_list, res->ai_addr, (socklen_t)res->ai_addrlen);
 			res = res->ai_next;
 		} while (res != NULL);
 	}
@@ -618,7 +639,6 @@ static int resolver_process_data_delayed(belle_sip_simple_resolver_context_t *ct
 }
 
 static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
-	struct dns_hints *(*hints)() = &dns_hints_local;
 	struct dns_options opts;
 	int error;
 	struct dns_resolv_conf *conf;
@@ -637,7 +657,7 @@ static int _resolver_start_query(belle_sip_simple_resolver_context_t *ctx) {
 
 	memset(&opts, 0, sizeof opts);
 
-	if (!(ctx->R = dns_res_open(ctx->resconf, ctx->hosts, dns_hints_mortal(hints(ctx->resconf, &error)), cache(ctx), &opts, &error))) {
+	if (!(ctx->R = dns_res_open(ctx->resconf, ctx->hosts, dns_hints_mortal(dns_hints_local(ctx->resconf, &error)), cache(ctx), &opts, &error))) {
 		belle_sip_error("%s dns_res_open error [%s]: %s", __FUNCTION__, ctx->name, dns_strerror(error));
 		return -1;
 	}
@@ -670,41 +690,6 @@ static belle_sip_simple_resolver_context_t * resolver_start_query(belle_sip_simp
 }
 
 
-int belle_sip_addrinfo_to_ip(const struct addrinfo *ai, char *ip, size_t ip_size, int *port){
-	char serv[16];
-	int err=getnameinfo(ai->ai_addr,ai->ai_addrlen,ip,ip_size,serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
-	if (err!=0){
-		belle_sip_error("getnameinfo() error: %s",gai_strerror(err));
-		strncpy(ip,"<bug!!>",ip_size);
-	}
-	if (port) *port=atoi(serv);
-	return 0;
-}
-
-struct addrinfo * belle_sip_ip_address_to_addrinfo(int family, const char *ipaddress, int port){
-	struct addrinfo *res=NULL;
-	struct addrinfo hints={0};
-	char serv[10];
-	int err;
-
-	snprintf(serv,sizeof(serv),"%i",port);
-	hints.ai_family=family;
-	hints.ai_flags=AI_NUMERICSERV|AI_NUMERICHOST;
-	hints.ai_socktype=SOCK_STREAM; //not used but it's needed to specify it because otherwise getaddrinfo returns one struct addrinfo per socktype.
-	
-	if (family==AF_INET6 && strchr(ipaddress,':')==NULL) {
-		hints.ai_flags|=AI_V4MAPPED;
-	}
-	err=belle_sip_getaddrinfo(ipaddress,serv,&hints,&res);
-
-	if (err!=0){
-		if (err!=EAI_NONAME)
-			belle_sip_error("belle_sip_ip_address_to_addrinfo(): getaddrinfo() error: %s",gai_strerror(err));
-		return NULL;
-	}
-	return res;
-}
-
 static void belle_sip_combined_resolver_context_destroy(belle_sip_combined_resolver_context_t *obj){
 	if (obj->name != NULL) {
 		belle_sip_free(obj->name);
@@ -721,7 +706,7 @@ static void belle_sip_combined_resolver_context_destroy(belle_sip_combined_resol
 }
 
 static void belle_sip_simple_resolver_context_destroy(belle_sip_simple_resolver_context_t *ctx){
-	/* Do not free elements of ctx->ai_list with belle_sip_freeaddrinfo(). Let the caller do it, otherwise
+	/* Do not free elements of ctx->ai_list with bctbx_freeaddrinfo(). Let the caller do it, otherwise
 	   it will not be able to use them after the resolver has been destroyed. */
 #ifdef USE_GETADDRINFO_FALLBACK
 	if (ctx->getaddrinfo_thread != 0) {
@@ -737,12 +722,12 @@ static void belle_sip_simple_resolver_context_destroy(belle_sip_simple_resolver_
 
 #endif
 	if (ctx->ai_list != NULL) {
-		belle_sip_freeaddrinfo(ctx->ai_list);
+		bctbx_freeaddrinfo(ctx->ai_list);
 		ctx->ai_list = NULL;
 	}
 #ifdef USE_GETADDRINFO_FALLBACK
 	if (ctx->getaddrinfo_ai_list != NULL) {
-		belle_sip_freeaddrinfo(ctx->getaddrinfo_ai_list);
+		bctbx_freeaddrinfo(ctx->getaddrinfo_ai_list);
 		ctx->getaddrinfo_ai_list = NULL;
 	}
 #endif
@@ -774,11 +759,11 @@ static void belle_sip_dual_resolver_context_destroy(belle_sip_dual_resolver_cont
 		obj->aaaa_ctx=NULL;
 	}
 	if (obj->a_results){
-		belle_sip_freeaddrinfo(obj->a_results);
+		bctbx_freeaddrinfo(obj->a_results);
 		obj->a_results=NULL;
 	}
 	if (obj->aaaa_results){
-		belle_sip_freeaddrinfo(obj->aaaa_results);
+		bctbx_freeaddrinfo(obj->aaaa_results);
 		obj->aaaa_results=NULL;
 	}
 	if (obj->name){
@@ -872,19 +857,16 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_combined_resolver_context_t)
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
-static char * srv_prefix_from_transport(const char *transport) {
-	char *prefix = "";
-
+static char * srv_prefix_from_service_and_transport(const char *service, const char *transport) {
+	if (service == NULL) service = "sip";
 	if (strcasecmp(transport, "udp") == 0) {
-		prefix = "_sip._udp.";
+		return belle_sip_strdup_printf("_%s._udp.", service);
 	} else if (strcasecmp(transport, "tcp") == 0) {
-		prefix = "_sip._tcp.";
+		return belle_sip_strdup_printf("_%s._tcp.", service);
 	} else if (strcasecmp(transport, "tls") == 0) {
-		prefix = "_sips._tcp.";
-	} else {
-		prefix = "_sip._udp.";
+		return belle_sip_strdup_printf("_%ss._tcp.", service);
 	}
-	return prefix;
+	return belle_sip_strdup_printf("_%s._udp.", service);
 }
 
 static void combined_notify_results(belle_sip_combined_resolver_context_t *obj){
@@ -975,8 +957,8 @@ static void process_srv_results(void *data, const char *name, belle_sip_list_t *
 /**
  * Perform combined SRV + A / AAAA resolution.
 **/
-belle_sip_resolver_context_t * belle_sip_stack_resolve(belle_sip_stack_t *stack, const char *transport, const char *name, int port, int family, belle_sip_resolver_callback_t cb, void *data) {
-	struct addrinfo *res = belle_sip_ip_address_to_addrinfo(family, name, port);
+belle_sip_resolver_context_t * belle_sip_stack_resolve(belle_sip_stack_t *stack, const char *service, const char *transport, const char *name, int port, int family, belle_sip_resolver_callback_t cb, void *data) {
+	struct addrinfo *res = bctbx_ip_address_to_addrinfo(family, SOCK_STREAM, name, port);
 	if (res == NULL) {
 		/* First perform asynchronous DNS SRV query */
 		belle_sip_combined_resolver_context_t *ctx = belle_sip_object_new(belle_sip_combined_resolver_context_t);
@@ -990,7 +972,7 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve(belle_sip_stack_t *stack,
 		ctx->family = family;
 		/*take a ref for the entire duration of the DNS procedure, it will be released when it is finished*/
 		belle_sip_object_ref(ctx);
-		ctx->srv_ctx=belle_sip_stack_resolve_srv(stack,transport,name,process_srv_results,ctx);
+		ctx->srv_ctx=belle_sip_stack_resolve_srv(stack,service,transport,name,process_srv_results,ctx);
 		if (ctx->srv_ctx) belle_sip_object_ref(ctx->srv_ctx);
 		if (ctx->base.done){
 			belle_sip_object_unref(ctx);
@@ -1073,7 +1055,7 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_dual(belle_sip_sta
 }
 
 belle_sip_resolver_context_t * belle_sip_stack_resolve_a(belle_sip_stack_t *stack, const char *name, int port, int family, belle_sip_resolver_callback_t cb , void *data) {
-	struct addrinfo *res = belle_sip_ip_address_to_addrinfo(family, name, port);
+	struct addrinfo *res = bctbx_ip_address_to_addrinfo(family, SOCK_STREAM, name, port);
 	if (res == NULL) {
 		switch(family){
 			case AF_UNSPEC:
@@ -1094,13 +1076,15 @@ belle_sip_resolver_context_t * belle_sip_stack_resolve_a(belle_sip_stack_t *stac
 	return NULL;
 }
 
-belle_sip_resolver_context_t * belle_sip_stack_resolve_srv(belle_sip_stack_t *stack, const char *transport, const char *name, belle_sip_resolver_srv_callback_t cb, void *data) {
+belle_sip_resolver_context_t * belle_sip_stack_resolve_srv(belle_sip_stack_t *stack, const char *service, const char *transport, const char *name, belle_sip_resolver_srv_callback_t cb, void *data) {
 	belle_sip_simple_resolver_context_t *ctx = belle_sip_object_new(belle_sip_simple_resolver_context_t);
+	char *srv_prefix = srv_prefix_from_service_and_transport(service, transport);
 	belle_sip_resolver_context_init((belle_sip_resolver_context_t*)ctx,stack);
 	ctx->srv_cb_data = data;
 	ctx->srv_cb = cb;
-	ctx->name = belle_sip_concat(srv_prefix_from_transport(transport), name, NULL);
+	ctx->name = belle_sip_concat(srv_prefix, name, NULL);
 	ctx->type = DNS_T_SRV;
+	belle_sip_free(srv_prefix);
 	return (belle_sip_resolver_context_t*)resolver_start_query(ctx);
 }
 
@@ -1114,11 +1098,13 @@ It works on all platform except for windows using ipv6 sockets. TODO: find a wor
 */
 void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen, struct sockaddr *src, socklen_t *srclen, int local_port){
 	int af_type=dest->sa_family;
-	int sock=socket(af_type,SOCK_DGRAM,IPPROTO_UDP);
+	int sock=(int)socket(af_type,SOCK_DGRAM,IPPROTO_UDP);
 	
 	belle_sip_message("belle_sip_get_src_addr_for(): af_inet6=%i",af_type==AF_INET6);
 	if (sock==(belle_sip_socket_t)-1){
-		belle_sip_fatal("Could not create socket: %s",belle_sip_get_socket_error_string());
+		if (af_type == AF_INET){
+			belle_sip_fatal("Could not create socket: %s",belle_sip_get_socket_error_string());
+		}
 		goto fail;
 	}
 	
@@ -1149,43 +1135,14 @@ void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen,
 	return;
 fail:
 	{
-		struct addrinfo *res = belle_sip_ip_address_to_addrinfo(af_type, af_type == AF_INET ? "127.0.0.1" : "::1", local_port);
+		struct addrinfo *res = bctbx_ip_address_to_addrinfo(af_type, SOCK_STREAM, af_type == AF_INET ? "127.0.0.1" : "::1", local_port);
 		if (res != NULL) {
 			memcpy(src,res->ai_addr,MIN((size_t)*srclen,res->ai_addrlen));
-			*srclen=res->ai_addrlen;
-			belle_sip_freeaddrinfo(res);
+			*srclen=(socklen_t)res->ai_addrlen;
+			bctbx_freeaddrinfo(res);
 		} else {
-			belle_sip_fatal("belle_sip_get_src_addr_for(): belle_sip_ip_address_to_addrinfo() failed");
+			if (af_type == AF_INET) belle_sip_fatal("belle_sip_get_src_addr_for(): belle_sip_ip_address_to_addrinfo() failed");
 		}
 	}
 	if (sock!=(belle_sip_socket_t)-1) close_socket(sock);
 }
-
-#ifndef IN6_GET_ADDR_V4MAPPED
-#define IN6_GET_ADDR_V4MAPPED(sin6_addr)	*(unsigned int*)((unsigned char*)(sin6_addr)+12)
-#endif
-
-
-void belle_sip_address_remove_v4_mapping(const struct sockaddr *v6, struct sockaddr *result, socklen_t *result_len){
-	if (v6->sa_family==AF_INET6){
-		struct sockaddr_in6 *in6=(struct sockaddr_in6*)v6;
-		
-		if (IN6_IS_ADDR_V4MAPPED(&in6->sin6_addr)){
-			struct sockaddr_in *in=(struct sockaddr_in*)result;
-			result->sa_family=AF_INET;
-			in->sin_addr.s_addr = IN6_GET_ADDR_V4MAPPED(&in6->sin6_addr);
-			in->sin_port=in6->sin6_port;
-			*result_len=sizeof(struct sockaddr_in);
-		}else{
-			if (v6!=result) memcpy(result,v6,sizeof(struct sockaddr_in6));
-			*result_len=sizeof(struct sockaddr_in6);
-		}
-		
-	}else{
-		*result_len=sizeof(struct sockaddr_in);
-		if (v6!=result) memcpy(result,v6,sizeof(struct sockaddr_in));
-	}
-}
-
-
-
