@@ -17,6 +17,9 @@
 */
 
 #include "belle_sip_internal.h"
+#ifdef HAVE_ZLIB
+#include "zlib.h"
+#endif
 
 /*
  * Body handler base class implementation
@@ -32,12 +35,6 @@ struct belle_sip_body_handler{
 	void *user_data;
 };
 
-void belle_sip_body_handler_add_header(belle_sip_body_handler_t *obj, belle_sip_header_t *header) {
-	if (header != NULL) {
-		obj->headers=belle_sip_list_append(obj->headers,belle_sip_object_ref(header));
-	}
-}
-
 static void belle_sip_body_handler_clone(belle_sip_body_handler_t *obj, const belle_sip_body_handler_t *orig){
 	obj->progress_cb=orig->progress_cb;
 	obj->user_data=orig->user_data;
@@ -54,6 +51,20 @@ static void belle_sip_body_handler_destroy(belle_sip_body_handler_t *obj){
 	belle_sip_free(obj->headerStringBuffer);
 }
 
+static belle_sip_error_code belle_sip_body_handler_marshal(belle_sip_body_handler_t *obj, char *buff, size_t buff_size, size_t *offset) {
+	int ret;
+	size_t len;
+	if (*offset == 0) belle_sip_body_handler_begin_transfer(obj);
+	do {
+		len = buff_size - *offset;
+		ret = belle_sip_body_handler_send_chunk(obj, NULL, (uint8_t*)buff + *offset, &len);
+		*offset += len;
+	} while ((ret == BELLE_SIP_CONTINUE) && (len > 0));
+	if ((ret == BELLE_SIP_CONTINUE) || (ret == BELLE_SIP_BUFFER_OVERFLOW)) return BELLE_SIP_BUFFER_OVERFLOW;
+	if (ret == BELLE_SIP_STOP) belle_sip_body_handler_end_transfer(obj);
+	return BELLE_SIP_OK;
+}
+
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_body_handler_t);
 
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t)
@@ -61,8 +72,10 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t)
 		BELLE_SIP_VPTR_INIT(belle_sip_body_handler_t,belle_sip_object_t,TRUE),
 		(belle_sip_object_destroy_t) belle_sip_body_handler_destroy,
 		(belle_sip_object_clone_t) belle_sip_body_handler_clone,
-		NULL,/*no marshal*/
+		(belle_sip_object_marshal_t) belle_sip_body_handler_marshal,
 	},
+	NULL, /* begin_transfer */
+	NULL, /* end_transfer */
 	NULL, /*chunk_recv*/
 	NULL /*chunk_send*/
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
@@ -74,6 +87,24 @@ void belle_sip_body_handler_init(belle_sip_body_handler_t *obj, belle_sip_body_h
 	obj->headerStringBuffer = NULL; /* header string buffer is set when adding a body handler to a multipart body handler */
 }
 
+void belle_sip_body_handler_add_header(belle_sip_body_handler_t *obj, belle_sip_header_t *header) {
+	if (header != NULL) {
+		obj->headers=belle_sip_list_append(obj->headers,belle_sip_object_ref(header));
+	}
+}
+
+void belle_sip_body_handler_remove_header_from_ptr(belle_sip_body_handler_t *obj, belle_sip_header_t* header) {
+	belle_sip_list_t* it = belle_sip_list_find(obj->headers, header);
+	if (it) {
+		belle_sip_object_unref(header);
+		obj->headers = belle_sip_list_delete_link(obj->headers, it);
+	}
+}
+
+const belle_sip_list_t* belle_sip_body_handler_get_headers(const belle_sip_body_handler_t *obj) {
+	return obj->headers;
+}
+
 size_t belle_sip_body_handler_get_size(const belle_sip_body_handler_t *obj){
 	return obj->expected_size;
 }
@@ -92,6 +123,10 @@ static void update_progress(belle_sip_body_handler_t *obj, belle_sip_message_t *
 }
 
 void belle_sip_body_handler_begin_transfer(belle_sip_body_handler_t *obj){
+	BELLE_SIP_OBJECT_VPTR_TYPE(belle_sip_body_handler_t) *vptr = BELLE_SIP_OBJECT_VPTR(obj, belle_sip_body_handler_t);
+	if (vptr->begin_transfer != NULL) {
+		vptr->begin_transfer(obj);
+	}
 	obj->transfered_size=0;
 }
 
@@ -103,11 +138,13 @@ void belle_sip_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_
 
 int belle_sip_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, uint8_t *buf, size_t *size){
 	int ret;
+	size_t to_send = *size;
 	if (obj->expected_size!=0){
-		*size=MIN(*size,obj->expected_size-obj->transfered_size);
+		to_send=MIN(*size,obj->expected_size-obj->transfered_size);
 	}
-	ret=BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_send(obj,msg,obj->transfered_size,buf,size);
-	obj->transfered_size+=*size;
+	ret=BELLE_SIP_OBJECT_VPTR(obj,belle_sip_body_handler_t)->chunk_send(obj,msg,obj->transfered_size,buf,&to_send);
+	obj->transfered_size+=to_send;
+	*size=to_send;
 	update_progress(obj,msg);
 	if (obj->expected_size!=0){
 		if (obj->transfered_size==obj->expected_size)
@@ -121,6 +158,10 @@ int belle_sip_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_m
 }
 
 void belle_sip_body_handler_end_transfer(belle_sip_body_handler_t *obj){
+	BELLE_SIP_OBJECT_VPTR_TYPE(belle_sip_body_handler_t) *vptr = BELLE_SIP_OBJECT_VPTR(obj, belle_sip_body_handler_t);
+	if (vptr->end_transfer != NULL) {
+		vptr->end_transfer(obj);
+	}
 	if (obj->expected_size==0)
 		obj->expected_size=obj->transfered_size;
 }
@@ -132,6 +173,7 @@ void belle_sip_body_handler_end_transfer(belle_sip_body_handler_t *obj){
 struct belle_sip_memory_body_handler{
 	belle_sip_body_handler_t base;
 	uint8_t *buffer;
+	uint8_t encoding_applied;
 };
 
 static void belle_sip_memory_body_handler_destroy(belle_sip_memory_body_handler_t *obj){
@@ -144,6 +186,7 @@ static void belle_sip_memory_body_handler_clone(belle_sip_memory_body_handler_t
 		memcpy(obj->buffer,orig->buffer,orig->base.expected_size);
 		obj->buffer[orig->base.expected_size]='\0';
 	}
+	obj->encoding_applied = orig->encoding_applied;
 }
 
 static void belle_sip_memory_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size){
@@ -156,6 +199,7 @@ static void belle_sip_memory_body_handler_recv_chunk(belle_sip_body_handler_t *b
 static int belle_sip_memory_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t *size){
 	belle_sip_memory_body_handler_t *obj=(belle_sip_memory_body_handler_t*)base;
 	size_t to_send=MIN(*size,obj->base.expected_size-offset);
+	if (obj->buffer == NULL) return BELLE_SIP_STOP;
 	memcpy(buf,obj->buffer+offset,to_send);
 	*size=to_send;
 	return (obj->base.expected_size-offset==*size) ? BELLE_SIP_STOP : BELLE_SIP_CONTINUE;
@@ -170,15 +214,138 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_memory_body_handler_t)
 			(belle_sip_object_clone_t)belle_sip_memory_body_handler_clone,
 			NULL
 		},
+		NULL,
+		NULL,
 		belle_sip_memory_body_handler_recv_chunk,
 		belle_sip_memory_body_handler_send_chunk
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
-const void *belle_sip_memory_body_handler_get_buffer(const belle_sip_memory_body_handler_t *obj){
+void *belle_sip_memory_body_handler_get_buffer(const belle_sip_memory_body_handler_t *obj){
 	return obj->buffer;
 }
 
+void belle_sip_memory_body_handler_set_buffer(belle_sip_memory_body_handler_t *obj, void *buffer) {
+	if (obj->buffer != NULL) belle_sip_free(obj->buffer);
+	obj->buffer = (uint8_t *)buffer;
+}
+
+#define BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE 2048
+
+void belle_sip_memory_body_handler_apply_encoding(belle_sip_memory_body_handler_t *obj, const char *encoding) {
+	if ((obj->buffer == NULL) || (obj->encoding_applied == TRUE)) return;
+
+#ifdef HAVE_ZLIB
+	if (strcmp(encoding, "deflate") == 0) {
+		z_stream strm;
+		size_t initial_size = belle_sip_body_handler_get_size(BELLE_SIP_BODY_HANDLER(obj));
+		size_t final_size;
+		unsigned int avail_out = BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE;
+		unsigned int outbuf_size = avail_out;
+		unsigned char *outbuf = belle_sip_malloc(outbuf_size);
+		unsigned char *outbuf_ptr = outbuf;
+		int ret;
+
+		strm.zalloc = Z_NULL;
+		strm.zfree = Z_NULL;
+		strm.opaque = Z_NULL;
+		ret = deflateInit(&strm, Z_DEFAULT_COMPRESSION);
+		if (ret != Z_OK) return;
+		strm.avail_in = (uInt)initial_size;
+		strm.next_in = obj->buffer;
+		do {
+			if (avail_out < BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE) {
+				unsigned int cursize = (unsigned int)(outbuf_ptr - outbuf);
+				outbuf_size *= 2;
+				outbuf = belle_sip_realloc(outbuf, outbuf_size);
+				outbuf_ptr = outbuf + cursize;
+			}
+			strm.avail_out = avail_out;
+			strm.next_out = outbuf_ptr;
+			deflate(&strm, Z_FINISH);
+			outbuf_ptr += avail_out - strm.avail_out;
+			avail_out = outbuf_size - (unsigned int)(outbuf_ptr - outbuf);
+		} while (strm.avail_out == 0);
+		deflateEnd(&strm);
+		final_size = outbuf_ptr - outbuf;
+		belle_sip_message("Body has been compressed: %u->%u:\n%s", (unsigned int)initial_size, (unsigned int)final_size, obj->buffer);
+		belle_sip_free(obj->buffer);
+		obj->buffer = outbuf;
+		belle_sip_body_handler_set_size(BELLE_SIP_BODY_HANDLER(obj), final_size);
+		obj->encoding_applied = TRUE;
+	} else
+#endif
+	{
+		belle_sip_warning("%s: unknown encoding '%s'", __FUNCTION__, encoding);
+	}
+}
+
+int belle_sip_memory_body_handler_unapply_encoding(belle_sip_memory_body_handler_t *obj, const char *encoding) {
+	if (obj->buffer == NULL) return -1;
+
+#ifdef HAVE_ZLIB
+	if (strcmp(encoding, "deflate") == 0) {
+		z_stream strm;
+		size_t initial_size = belle_sip_body_handler_get_size(BELLE_SIP_BODY_HANDLER(obj));
+		size_t final_size;
+		unsigned int avail_out = BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE;
+		unsigned int outbuf_size = avail_out;
+		unsigned char *outbuf = belle_sip_malloc(outbuf_size);
+		unsigned char *outbuf_ptr = outbuf;
+		int ret;
+
+		strm.zalloc = Z_NULL;
+		strm.zfree = Z_NULL;
+		strm.opaque = Z_NULL;
+		strm.avail_in = 0;
+		strm.next_in = Z_NULL;
+		ret = inflateInit(&strm);
+		if (ret != Z_OK) return -1;
+		strm.avail_in = (uInt)initial_size;
+		strm.next_in = obj->buffer;
+		do {
+			if (avail_out < BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE) {
+				unsigned int cursize = (unsigned int)(outbuf_ptr - outbuf);
+				outbuf_size *= 2;
+				outbuf = belle_sip_realloc(outbuf, outbuf_size);
+				outbuf_ptr = outbuf + cursize;
+			}
+			strm.avail_out = avail_out;
+			strm.next_out = outbuf_ptr;
+			ret = inflate(&strm, Z_NO_FLUSH);
+			switch (ret) {
+				case Z_NEED_DICT:
+					ret = Z_DATA_ERROR;
+				case Z_DATA_ERROR:
+				case Z_MEM_ERROR:
+					inflateEnd(&strm);
+					belle_sip_free(outbuf);
+					return -1;
+			}
+			outbuf_ptr += avail_out - strm.avail_out;
+			avail_out = outbuf_size - (unsigned int)(outbuf_ptr - outbuf);
+		} while (ret != Z_STREAM_END);
+		inflateEnd(&strm);
+		final_size = outbuf_ptr - outbuf;
+		if (final_size == outbuf_size) {
+			/* If not enough space get it to be able to put the final NULL character. */
+			outbuf = belle_sip_realloc(outbuf, outbuf_size + 1);
+			outbuf_ptr = outbuf + final_size;
+		}
+		*outbuf_ptr = '\0';
+		belle_sip_message("Body has been uncompressed: %u->%u:\n%s", (unsigned int)initial_size, (unsigned int)final_size, outbuf);
+		belle_sip_free(obj->buffer);
+		obj->buffer = outbuf;
+		belle_sip_body_handler_set_size(BELLE_SIP_BODY_HANDLER(obj), final_size);
+		return 0;
+	} else
+#endif
+	{
+		belle_sip_warning("%s: unknown encoding '%s'", __FUNCTION__, encoding);
+		return -1;
+	}
+}
+
 belle_sip_memory_body_handler_t *belle_sip_memory_body_handler_new(belle_sip_body_handler_progress_callback_t cb, void *user_data){
 	belle_sip_memory_body_handler_t *obj=belle_sip_object_new(belle_sip_memory_body_handler_t);
 	belle_sip_body_handler_init((belle_sip_body_handler_t*)obj,cb,user_data);
@@ -243,6 +410,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_user_body_handler_t)
 			(belle_sip_object_clone_t)belle_sip_user_body_handler_clone,
 			NULL
 		},
+		NULL,
+		NULL,
 		belle_sip_user_body_handler_recv_chunk,
 		belle_sip_user_body_handler_send_chunk
 	}
@@ -282,7 +451,7 @@ static void belle_sip_file_body_handler_clone(belle_sip_file_body_handler_t *obj
 
 static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, const uint8_t *buf, size_t size) {
 	FILE *f;
-	int ret;
+	size_t ret;
 	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
 	if (obj->filepath == NULL) return;
 	f = fopen(obj->filepath, "ab");
@@ -297,23 +466,24 @@ static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *bas
 
 static int belle_sip_file_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, size_t offset, uint8_t *buf, size_t *size) {
 	FILE *f;
-	int ret;
+	int int_ret;
+	size_t size_t_ret;
 	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
 	size_t to_send = MIN(*size, obj->base.expected_size - offset);
 	if (obj->filepath == NULL) return BELLE_SIP_STOP;
 	f = fopen(obj->filepath, "rb");
 	if (f == NULL) return BELLE_SIP_STOP;
-	ret = fseek(f, offset, SEEK_SET);
-	if (ret < 0) {
+	int_ret = fseek(f, (long)offset, SEEK_SET);
+	if (int_ret < 0) {
 		fclose(f);
 		return BELLE_SIP_STOP;
 	}
-	ret = fread(buf, 1, to_send, f);
-	if (ret < 0) {
+	size_t_ret = fread(buf, 1, to_send, f);
+	if (size_t_ret < 0) {
 		fclose(f);
 		return BELLE_SIP_STOP;
 	}
-	*size = ret;
+	*size = size_t_ret;
 	fclose(f);
 	return (((obj->base.expected_size - offset) == *size) || (*size == 0)) ? BELLE_SIP_STOP : BELLE_SIP_CONTINUE;
 }
@@ -327,6 +497,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_file_body_handler_t)
 			(belle_sip_object_clone_t)belle_sip_file_body_handler_clone,
 			NULL
 		},
+		NULL,
+		NULL,
 		belle_sip_file_body_handler_recv_chunk,
 		belle_sip_file_body_handler_send_chunk
 	}
@@ -349,40 +521,69 @@ belle_sip_file_body_handler_t *belle_sip_file_body_handler_new(const char *filep
  * TODO
 **/
 
-#define MULTIPART_SEPARATOR "--" BELLESIP_MULTIPART_BOUNDARY "\r\n"
-#define MULTIPART_END "\r\n--" BELLESIP_MULTIPART_BOUNDARY "--\r\n"
 struct belle_sip_multipart_body_handler{
 	belle_sip_body_handler_t base;
 	belle_sip_list_t *parts;
+	belle_sip_list_t *transfer_current_part;
+	char *boundary;
+	uint8_t *buffer;
+	unsigned int related;
 };
+GET_SET_BOOL(belle_sip_multipart_body_handler,related,is);
 
 static void belle_sip_multipart_body_handler_destroy(belle_sip_multipart_body_handler_t *obj){
 	belle_sip_list_free_with_data(obj->parts,belle_sip_object_unref);
+	if (obj->buffer != NULL) belle_sip_free(obj->buffer);
+	if (obj->boundary != NULL) belle_sip_free(obj->boundary);
 }
 
 static void belle_sip_multipart_body_handler_clone(belle_sip_multipart_body_handler_t *obj){
 	obj->parts=belle_sip_list_copy_with_data(obj->parts,(void *(*)(void*))belle_sip_object_clone_and_ref);
 }
 
+static void belle_sip_multipart_body_handler_begin_transfer(belle_sip_body_handler_t *obj) {
+	const belle_sip_list_t *it;
+	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
+	for (it = obj_multipart->parts; it != NULL; it = it->next) {
+		belle_sip_body_handler_t *bh = BELLE_SIP_BODY_HANDLER(it->data);
+		belle_sip_body_handler_begin_transfer(bh);
+	}
+	obj_multipart->transfer_current_part = obj_multipart->parts;
+}
+
+static void belle_sip_multipart_body_handler_end_transfer(belle_sip_body_handler_t *obj) {
+	const belle_sip_list_t *it;
+	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
+	for (it = obj_multipart->parts; it != NULL; it = it->next) {
+		belle_sip_body_handler_t *bh = BELLE_SIP_BODY_HANDLER(it->data);
+		belle_sip_body_handler_end_transfer(bh);
+	}
+	obj_multipart->transfer_current_part = NULL;
+}
+
 static void belle_sip_multipart_body_handler_recv_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset,
 							const uint8_t *buffer, size_t size){
-	/*TODO*/
+	/* Store the whole buffer, the parts will be split when belle_sip_multipart_body_handler_progress_cb() is called with transfered size equal to expected size. */
+	belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
+	obj_multipart->buffer = belle_sip_realloc(obj_multipart->buffer,offset + size + 1);
+	memcpy(obj_multipart->buffer + offset, buffer, size);
+	obj_multipart->buffer[offset + size] = '\0';
 }
 
 static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, size_t offset,
 							uint8_t *buffer, size_t *size){
-
 	belle_sip_multipart_body_handler_t *obj_multipart=(belle_sip_multipart_body_handler_t*)obj;
 
-	if (obj_multipart->parts->data) { /* we have a part, get its content from handler */
+	if (obj_multipart->transfer_current_part->data) { /* we have a part, get its content from handler */
 		int retval = BELLE_SIP_STOP;
 		size_t offsetSize = 0; /* used to store size of data added by this function and not given by the body handler of current part */
-		belle_sip_body_handler_t *current_part = (belle_sip_body_handler_t *)obj_multipart->parts->data;
-		*size -= strlen(MULTIPART_END); /* just in case it will be the end of the message, ask for less characters than possible in order to be able to add the multipart message termination */
+		size_t boundary_len = strlen(obj_multipart->boundary);
+		belle_sip_body_handler_t *current_part = (belle_sip_body_handler_t *)obj_multipart->transfer_current_part->data;
+		*size -= strlen(obj_multipart->boundary) + 8; /* just in case it will be the end of the message, ask for less characters than possible in order to be able to add the multipart message termination. 8 is for "\r\n--" and "--\r\n" */
 
 		if (current_part->transfered_size == 0) { /* Nothing transfered yet on this part, include a separator and the header if exists */
 			size_t headersSize = 0;
-			offsetSize = strlen(MULTIPART_SEPARATOR);
+			offsetSize = strlen(obj_multipart->boundary) + 4; /* 4 is for "--" and "\r\n" */
 
 			if (current_part->headerStringBuffer != NULL) {
 				headersSize = strlen(current_part->headerStringBuffer);
@@ -394,7 +595,10 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 			}
 
 			/* insert separator */
-			memcpy(buffer, MULTIPART_SEPARATOR, offsetSize);
+			memcpy(buffer, "--", 2);
+			memcpy(buffer + 2, obj_multipart->boundary, boundary_len);
+			memcpy(buffer + 2 + boundary_len, "\r\n", 2);
+			offsetSize = boundary_len + 4;
 			/* insert part header */
 			if (headersSize!=0) {
 				memcpy(buffer+offsetSize, current_part->headerStringBuffer, headersSize);
@@ -412,12 +616,15 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 		if (retval == BELLE_SIP_CONTINUE) {
 			return BELLE_SIP_CONTINUE; /* there is still data to be sent, continue */
 		} else { /* this part has reach the end, pass to next one if there is one */
-			if (obj_multipart->parts->next!=NULL) { /* there is an other part to be sent */
-				obj_multipart->parts = belle_sip_list_next(obj_multipart->parts);
+			if (obj_multipart->transfer_current_part->next!=NULL) { /* there is an other part to be sent */
+				obj_multipart->transfer_current_part = belle_sip_list_next(obj_multipart->transfer_current_part);
 				return BELLE_SIP_CONTINUE;
 			} else { /* there is nothing else, close the message and return STOP */
-				memcpy(buffer+*size, MULTIPART_END, strlen(MULTIPART_END));
-				*size+=strlen(MULTIPART_END);
+				size_t boundary_len = strlen(obj_multipart->boundary);
+				memcpy(buffer + *size, "\r\n--", 4);
+				memcpy(buffer + *size + 4, obj_multipart->boundary, boundary_len);
+				memcpy(buffer + *size + 4 + boundary_len, "--\r\n", 4);
+				*size+=boundary_len + 8;
 				return BELLE_SIP_STOP;
 			}
 		}
@@ -434,23 +641,49 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t)
 			(belle_sip_object_clone_t)belle_sip_multipart_body_handler_clone,
 			NULL
 		},
+		belle_sip_multipart_body_handler_begin_transfer,
+		belle_sip_multipart_body_handler_end_transfer,
 		belle_sip_multipart_body_handler_recv_chunk,
 		belle_sip_multipart_body_handler_send_chunk
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
+static void belle_sip_multipart_body_handler_set_boundary(belle_sip_multipart_body_handler_t *obj, const char *boundary) {
+	if (obj->boundary != NULL) {
+		belle_sip_free(obj->boundary);
+	}
+	if (boundary != NULL) {
+		obj->boundary = belle_sip_strdup(boundary);
+	} else {
+		obj->boundary = belle_sip_strdup(BELLESIP_MULTIPART_BOUNDARY);
+	}
+}
+
 belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new(belle_sip_body_handler_progress_callback_t progress_cb, void *data,
-									 belle_sip_body_handler_t *first_part){
+									 belle_sip_body_handler_t *first_part, const char *boundary){
 	belle_sip_multipart_body_handler_t *obj=belle_sip_object_new(belle_sip_multipart_body_handler_t);
 	belle_sip_body_handler_init((belle_sip_body_handler_t*)obj,progress_cb,data);
-	obj->base.expected_size = strlen(MULTIPART_END); /* body's length will be part length(including boundary) + multipart end */
+	belle_sip_multipart_body_handler_set_boundary(obj, boundary);
+	obj->base.expected_size = strlen(obj->boundary) + 8; /* body's length will be part length(including boundary) + multipart end. 8 is for "\r\n--" and "--\r\n" */
 	if (first_part) belle_sip_multipart_body_handler_add_part(obj,first_part);
 	return obj;
 }
 
+belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new_from_buffer(void *buffer, size_t bufsize, const char *boundary) {
+	belle_sip_multipart_body_handler_t *obj_multipart = belle_sip_object_new(belle_sip_multipart_body_handler_t);
+	belle_sip_body_handler_t *obj = (belle_sip_body_handler_t *)obj_multipart;
+	belle_sip_body_handler_init((belle_sip_body_handler_t *)obj, belle_sip_multipart_body_handler_progress_cb, NULL);
+	belle_sip_multipart_body_handler_set_boundary(obj_multipart, boundary);
+	obj_multipart->base.expected_size = bufsize;
+	belle_sip_body_handler_begin_transfer(obj);
+	belle_sip_body_handler_recv_chunk(obj, NULL, (uint8_t *)buffer, bufsize);
+	belle_sip_body_handler_end_transfer(obj);
+	return obj_multipart;
+}
+
 #define DEFAULT_HEADER_STRING_SIZE 512
 void belle_sip_multipart_body_handler_add_part(belle_sip_multipart_body_handler_t *obj, belle_sip_body_handler_t *part){
-	obj->base.expected_size+=part->expected_size+strlen(MULTIPART_SEPARATOR); /* add the separator length to the body length as each part start with a separator */
+	obj->base.expected_size+=part->expected_size+strlen(obj->boundary) + 4; /* add the separator length to the body length as each part start with a separator. 4 is for "--" and "\r\n" */
 	if (part->headers != NULL) { /* there is a declared header for this part, add its length to the expected total length */
 		size_t headerStringBufferSize = DEFAULT_HEADER_STRING_SIZE;
 		size_t offset = 0;
@@ -477,5 +710,61 @@ void belle_sip_multipart_body_handler_add_part(belle_sip_multipart_body_handler_
 	}
 	obj->parts=belle_sip_list_append(obj->parts,belle_sip_object_ref(part));
 }
+const belle_sip_list_t* belle_sip_multipart_body_handler_get_parts(const belle_sip_multipart_body_handler_t *obj) {
+	return obj->parts;
+}
 
-
+void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, void *user_data, size_t transfered, size_t expected_total) {
+	if (transfered == expected_total) {
+		/* The full multipart body has been received, we can now parse it and split the different parts,
+		 * creating a belle_sip_memory_body_handler for each part and adding them to the belle_sip_multipart_body_handler
+		 * parts list. */
+		belle_sip_multipart_body_handler_t *obj_multipart = (belle_sip_multipart_body_handler_t *)obj;
+		belle_sip_memory_body_handler_t *memorypart;
+		belle_sip_header_t *header;
+		uint8_t *end_part_cursor;
+		uint8_t *end_headers_cursor;
+		uint8_t *end_header_cursor;
+		uint8_t *cursor = obj_multipart->buffer;
+		char *boundary = belle_sip_strdup_printf("--%s", obj_multipart->boundary);
+		if (strncmp((char *)cursor, boundary, strlen(boundary))) {
+			belle_sip_warning("belle_sip_multipart_body_handler [%p]: body not starting by specified boundary '%s'", obj_multipart, obj_multipart->boundary);
+			belle_sip_free(boundary);
+			return;
+		}
+		cursor += strlen(boundary);
+		do {
+			if (strncmp((char *)cursor, "\r\n", 2)) {
+				belle_sip_warning("belle_sip_multipart_body_handler [%p]: no new-line after boundary", obj_multipart);
+				return;
+			}
+			cursor += 2;
+			end_part_cursor = (uint8_t *)strstr((char *)cursor, boundary);
+			if (end_part_cursor == NULL) {
+				belle_sip_warning("belle_sip_multipart_body_handler [%p]: cannot find next boundary", obj_multipart);
+				return;
+			} else {
+				*end_part_cursor = 0;
+				end_headers_cursor = (uint8_t *)strstr((char *)cursor, "\r\n\r\n");
+				if (end_headers_cursor == NULL) {
+					memorypart = belle_sip_memory_body_handler_new_copy_from_buffer(cursor, strlen((char *)cursor), NULL, NULL);
+				} else {
+					uint8_t *begin_body_cursor = end_headers_cursor + 4;
+					memorypart = belle_sip_memory_body_handler_new_copy_from_buffer(begin_body_cursor, strlen((char *)begin_body_cursor), NULL, NULL);
+					do {
+						end_header_cursor = (uint8_t *)strstr((char *)cursor, "\r\n");
+						*end_header_cursor = 0;
+						header = belle_sip_header_parse((char *)cursor);
+						if (header != NULL) {
+							belle_sip_body_handler_add_header(BELLE_SIP_BODY_HANDLER(memorypart), header);
+						}
+						cursor = end_header_cursor + 2;
+					} while (end_header_cursor != end_headers_cursor);
+				}
+				belle_sip_multipart_body_handler_add_part(obj_multipart, BELLE_SIP_BODY_HANDLER(memorypart));
+				cursor = end_part_cursor + strlen(boundary);
+			}
+		} while (strcmp((char *)cursor, "--\r\n"));
+		belle_sip_free(boundary);
+	}
+}
