@@ -69,8 +69,37 @@ static belle_sip_list_t * for_each_weak_unref_free(belle_sip_list_t *l, belle_si
 	return NULL;
 }
 
+static void belle_sip_channel_input_stream_rewind(belle_sip_channel_input_stream_t* input_stream){
+	int remaining;
+
+	remaining=(int)(input_stream->write_ptr-input_stream->read_ptr);
+	if (remaining>0){
+		/* copy remaning bytes at top of buffer*/
+		memmove(input_stream->buff,input_stream->read_ptr,remaining);
+		input_stream->read_ptr=input_stream->buff;
+		input_stream->write_ptr=input_stream->buff+remaining;
+		*input_stream->write_ptr='\0';
+	}else{
+		input_stream->read_ptr=input_stream->write_ptr=input_stream->buff;
+	}
+}
+
+static void belle_sip_channel_input_stream_reset(belle_sip_channel_input_stream_t* input_stream) {
+	belle_sip_channel_input_stream_rewind(input_stream);
+	input_stream->state=WAITING_MESSAGE_START;
+	if (input_stream->msg != NULL) belle_sip_object_unref(input_stream->msg);
+	input_stream->msg=NULL;
+	input_stream->chuncked_mode=FALSE;
+	input_stream->content_length=-1;
+}
+
+static size_t belle_sip_channel_input_stream_get_buff_length(belle_sip_channel_input_stream_t* input_stream) {
+	return sizeof(input_stream->buff) - (input_stream->write_ptr-input_stream->buff);
+}
+
 static void belle_sip_channel_destroy(belle_sip_channel_t *obj){
-	if (obj->peer_list) belle_sip_freeaddrinfo(obj->peer_list);
+	belle_sip_channel_input_stream_reset(&obj->input_stream);
+	if (obj->peer_list) bctbx_freeaddrinfo(obj->peer_list);
 	if (obj->peer_cname) belle_sip_free(obj->peer_cname);
 	belle_sip_free(obj->peer_name);
 	if (obj->local_ip) belle_sip_free(obj->local_ip);
@@ -82,6 +111,7 @@ static void belle_sip_channel_destroy(belle_sip_channel_t *obj){
 	}
 	if (obj->public_ip) belle_sip_free(obj->public_ip);
 	if (obj->outgoing_messages) belle_sip_list_free_with_data(obj->outgoing_messages,belle_sip_object_unref);
+	if (obj->incoming_messages) belle_sip_list_free_with_data(obj->incoming_messages,belle_sip_object_unref);
 	free_ewouldblock_buffer(obj);
 	if (obj->cur_out_message){
 		belle_sip_object_unref(obj->cur_out_message);
@@ -117,7 +147,7 @@ static void fix_incoming_via(belle_sip_request_t *msg, const struct addrinfo* or
 		belle_sip_warning("cannot fix via for message [%p], probably a test",msg);
 		return;
 	}
-	belle_sip_address_remove_v4_mapping(origin->ai_addr, (struct sockaddr*)&saddr, &slen);
+	bctbx_sockaddr_remove_v4_mapping(origin->ai_addr, (struct sockaddr*)&saddr, &slen);
 	err=getnameinfo((struct sockaddr*)&saddr,slen,received,sizeof(received),
 	                rport,sizeof(rport),NI_NUMERICHOST|NI_NUMERICSERV);
 	if (err!=0){
@@ -178,7 +208,7 @@ static int get_message_start_pos(char *buff, size_t bufflen) {
 	char method[17]={0};
 	char saved_char1;
 	char sip_version[10]={0};
-	int saved_char1_index;
+	size_t saved_char1_index;
 
 	for(i=0; i<(int)bufflen-12;i++) { /*9=strlen( SIP/2.0\r\n)*/
 		switch (buff[i]) { /*to avoid this character to be ignored by scanf*/
@@ -206,33 +236,6 @@ static int get_message_start_pos(char *buff, size_t bufflen) {
 	return -1;
 }
 
-static void belle_sip_channel_input_stream_rewind(belle_sip_channel_input_stream_t* input_stream){
-	int remaining;
-
-	remaining=input_stream->write_ptr-input_stream->read_ptr;
-	if (remaining>0){
-		/* copy remaning bytes at top of buffer*/
-		memmove(input_stream->buff,input_stream->read_ptr,remaining);
-		input_stream->read_ptr=input_stream->buff;
-		input_stream->write_ptr=input_stream->buff+remaining;
-		*input_stream->write_ptr='\0';
-	}else{
-		input_stream->read_ptr=input_stream->write_ptr=input_stream->buff;
-	}
-}
-
-static void belle_sip_channel_input_stream_reset(belle_sip_channel_input_stream_t* input_stream) {
-	belle_sip_channel_input_stream_rewind(input_stream);
-	input_stream->state=WAITING_MESSAGE_START;
-	input_stream->msg=NULL;
-	input_stream->chuncked_mode=FALSE;
-	input_stream->content_length=-1;
-}
-
-static size_t belle_sip_channel_input_stream_get_buff_length(belle_sip_channel_input_stream_t* input_stream) {
-	return sizeof(input_stream->buff) - (input_stream->write_ptr-input_stream->buff);
-}
-
 char *belle_sip_channel_get_public_ip_port(belle_sip_channel_t *obj){
 	if (obj->public_ip){
 		return belle_sip_strdup_printf("%s:%d", obj->public_ip, obj->public_port);
@@ -290,12 +293,47 @@ static void belle_sip_channel_learn_public_ip_port(belle_sip_channel_t *obj, bel
 	obj->learnt_ip_port=TRUE;
 }
 
+static void uncompress_body_if_required(belle_sip_message_t *msg) {
+	belle_sip_body_handler_t *bh = belle_sip_message_get_body_handler(msg);
+	belle_sip_memory_body_handler_t *mbh = NULL;
+	belle_sip_header_t *ceh = NULL;
+	size_t body_len = 0;
+
+	if (bh != NULL) {
+		body_len = belle_sip_message_get_body_size(msg);
+		ceh = belle_sip_message_get_header(msg, "Content-Encoding");
+	}
+	if ((body_len > 0) && (ceh != NULL)) {
+		const char *content_encoding = belle_sip_header_get_unparsed_value(ceh);
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(bh, belle_sip_memory_body_handler_t)) {
+			mbh = BELLE_SIP_MEMORY_BODY_HANDLER(bh);
+			if (belle_sip_memory_body_handler_unapply_encoding(mbh, content_encoding) == 0) {
+				belle_sip_header_content_type_t *content_type = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+				belle_sip_message_remove_header_from_ptr(msg, ceh);
+				if (content_type
+					&& (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
+					const char *unparsed_value = belle_sip_header_get_unparsed_value(BELLE_SIP_HEADER(content_type));
+					const char *boundary = strstr(unparsed_value, ";boundary=");
+					if (boundary != NULL) boundary += 10;
+					if (boundary[0] == '\0') boundary = NULL;
+					bh = (belle_sip_body_handler_t *)belle_sip_multipart_body_handler_new_from_buffer(
+						belle_sip_memory_body_handler_get_buffer(mbh), belle_sip_body_handler_get_size((belle_sip_body_handler_t *)mbh), boundary);
+					belle_sip_message_set_body_handler(msg, bh);
+				}
+			}
+		} else {
+			belle_sip_warning("message [%p] has Content-Encoding [%s] that cannot be unapplied", msg, content_encoding);
+		}
+	}
+}
+
 static void belle_sip_channel_message_ready(belle_sip_channel_t *obj){
 	belle_sip_message_t *msg=obj->input_stream.msg;
 	belle_sip_body_handler_t *bh=belle_sip_message_get_body_handler(msg);
 	if (bh) belle_sip_body_handler_end_transfer(bh);
 	if (belle_sip_message_is_response(msg)) belle_sip_channel_learn_public_ip_port(obj,BELLE_SIP_RESPONSE(msg));
-	obj->incoming_messages=belle_sip_list_append(obj->incoming_messages,msg);
+	uncompress_body_if_required(msg);
+	obj->incoming_messages=belle_sip_list_append(obj->incoming_messages,belle_sip_object_ref(msg));
 	obj->stop_logging_buffer=0;
 	belle_sip_channel_input_stream_reset(&obj->input_stream);
 }
@@ -340,7 +378,21 @@ static int check_body(belle_sip_channel_t *obj){
 		BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_message_headers,obj,msg);
 		/*check if the listener has setup a body handler, otherwise create a default one*/
 		if ((bh=belle_sip_message_get_body_handler(msg))==NULL){
-			belle_sip_message_set_body_handler(msg,(bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new(NULL,NULL)));
+			belle_sip_header_t *content_encoding = belle_sip_message_get_header(msg, "Content-Encoding");
+			belle_sip_header_content_type_t *content_type = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+			if (content_encoding != NULL) {
+				belle_sip_message_set_body_handler(msg, (bh = (belle_sip_body_handler_t *)belle_sip_memory_body_handler_new(NULL, NULL)));
+			} else if (content_type
+				&& (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
+				const char *unparsed_value = belle_sip_header_get_unparsed_value(BELLE_SIP_HEADER(content_type));
+				const char *boundary = strstr(unparsed_value, ";boundary=");
+				if (boundary != NULL) boundary += 10;
+				if (boundary[0] == '\0') boundary = NULL;
+				belle_sip_message_set_body_handler(msg, (bh = (belle_sip_body_handler_t *)belle_sip_multipart_body_handler_new(belle_sip_multipart_body_handler_progress_cb, NULL, NULL, boundary)));
+				belle_sip_body_handler_set_size(bh, obj->input_stream.content_length);
+			} else {
+				belle_sip_message_set_body_handler(msg,(bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new(NULL,NULL)));
+			}
 		}
 		belle_sip_body_handler_begin_transfer(bh);
 	}
@@ -348,7 +400,7 @@ static int check_body(belle_sip_channel_t *obj){
 }
 
 static int acquire_body_simple(belle_sip_channel_t *obj, int end_of_stream){
-	int content_length=obj->input_stream.content_length;
+	size_t content_length=obj->input_stream.content_length;
 	size_t to_read=obj->input_stream.write_ptr-obj->input_stream.read_ptr;
 	belle_sip_message_t *msg=obj->input_stream.msg;
 	belle_sip_body_handler_t *bh=belle_sip_message_get_body_handler(msg);
@@ -367,7 +419,7 @@ static int acquire_body_simple(belle_sip_channel_t *obj, int end_of_stream){
 	if (end_of_stream ||  belle_sip_body_handler_get_transfered_size(bh)>=content_length){
 		/*great body completed*/
 		belle_sip_message("channel [%p] read [%i] bytes of body from [%s:%i]"	,obj
-			,content_length
+			,(int)content_length
 			,obj->peer_name
 			,obj->peer_port);
 		belle_sip_channel_message_ready(obj);
@@ -414,7 +466,7 @@ static int acquire_chuncked_body(belle_sip_channel_t *obj){
 				return BELLE_SIP_STOP;
 			}
 		}
-		readsize=MIN(st->write_ptr-st->read_ptr,st->chunk_size-st->chunk_read_size);
+		readsize=MIN((int)(st->write_ptr-st->read_ptr),st->chunk_size-st->chunk_read_size);
 		if (readsize>0){
 			feed_body(obj,readsize);
 			st->chunk_read_size+=readsize;
@@ -462,7 +514,7 @@ void belle_sip_channel_parse_stream(belle_sip_channel_t *obj, int end_of_stream)
 	size_t read_size=0;
 	int num;
 
-	while ((num=(obj->input_stream.write_ptr-obj->input_stream.read_ptr))>0){
+	while ((num=(int)(obj->input_stream.write_ptr-obj->input_stream.read_ptr))>0){
 
 		if (obj->input_stream.state == WAITING_MESSAGE_START) {
 			int i;
@@ -502,7 +554,7 @@ void belle_sip_channel_parse_stream(belle_sip_channel_t *obj, int end_of_stream)
 				char tmp;
 				/*end of message found*/
 				end_of_message+=4;/*add \r\n\r\n*/
-				bytes_to_parse=end_of_message-obj->input_stream.read_ptr;
+				bytes_to_parse=(int)(end_of_message-obj->input_stream.read_ptr);
 				tmp=*end_of_message;
 				*end_of_message='\0';/*this is in order for the following log to print the message only to its end.*/
 				/*belle_sip_message("channel [%p] read message of [%i] bytes:\n%.40s...",obj, bytes_to_parse, obj->input_stream.read_ptr);*/
@@ -543,7 +595,21 @@ void belle_sip_channel_parse_stream(belle_sip_channel_t *obj, int end_of_stream)
 
 static void belle_sip_channel_process_stream(belle_sip_channel_t *obj, int eos){
 	belle_sip_channel_parse_stream(obj,eos);
-	if (obj->incoming_messages) notify_incoming_messages(obj);
+	if (obj->incoming_messages) {
+		if (obj->simulated_recv_return == 1500) {
+			belle_sip_list_t *elem;
+			for(elem=obj->incoming_messages;elem!=NULL;elem=elem->next){
+				belle_sip_message_t *msg=(belle_sip_message_t*)elem->data;
+				char* dump = belle_sip_message_to_string(msg);
+				belle_sip_message("Silently discarding incoming message [%.50s...] on channel [%p]",dump, obj);
+				belle_sip_free(dump);
+			}
+			belle_sip_list_free_with_data(obj->incoming_messages,belle_sip_object_unref);
+			obj->incoming_messages=NULL;
+		} else {
+			notify_incoming_messages(obj);
+		}
+	}
 }
 
 static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
@@ -567,7 +633,7 @@ static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
 		/*first null terminate the read buff*/
 		*obj->input_stream.write_ptr='\0';
 		if (num>20 || obj->input_stream.state != WAITING_MESSAGE_START ) /*to avoid tracing server based keep alives*/ {
-			char *logbuf = make_logbuf(obj, BELLE_SIP_LOG_MESSAGE,begin,num);
+			char *logbuf = make_logbuf(obj, BELLE_SIP_LOG_MESSAGE ,begin,num);
 			if (logbuf) {
 				belle_sip_message("channel [%p]: received [%i] new bytes from [%s://%s:%i]:\n%s",
 						obj,
@@ -580,6 +646,9 @@ static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
 			}
 		}
 		belle_sip_channel_process_stream(obj,FALSE);
+		if (obj->input_stream.state == WAITING_MESSAGE_START){
+			channel_end_recv_background_task(obj);
+		}/*if still in message acquisition state, keep the backgroud task*/
 	} else if (num == 0) {
 		/*before closing the channel, check if there was a pending message to receive, whose body acquisition is to be finished.*/
 		belle_sip_channel_process_stream(obj,TRUE);
@@ -593,8 +662,6 @@ static int belle_sip_channel_process_read_data(belle_sip_channel_t *obj){
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 		ret=BELLE_SIP_STOP;
 	}
-	if (obj->input_stream.state == WAITING_MESSAGE_START)
-		channel_end_recv_background_task(obj);
 	return ret;
 }
 
@@ -669,8 +736,8 @@ void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,c
 	obj->simulated_recv_return=1;/*not set*/
 	if (peername){
 		/*check if we are given a real dns name or just an ip address*/
-		struct addrinfo *ai=belle_sip_ip_address_to_addrinfo(AF_UNSPEC,peername,peer_port);
-		if (ai) belle_sip_freeaddrinfo(ai);
+		struct addrinfo *ai=bctbx_ip_address_to_addrinfo(AF_UNSPEC,SOCK_STREAM,peername,peer_port);
+		if (ai) bctbx_freeaddrinfo(ai);
 		else obj->has_name=TRUE;
 	}
 	belle_sip_channel_input_stream_reset(&obj->input_stream);
@@ -687,9 +754,9 @@ void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_
 	ai.ai_family=peer_addr->sa_family;
 	ai.ai_addr=(struct sockaddr*)peer_addr;
 	ai.ai_addrlen=addrlen;
-	belle_sip_addrinfo_to_ip(&ai,remoteip,sizeof(remoteip),&peer_port);
+	bctbx_addrinfo_to_ip_address(&ai,remoteip,sizeof(remoteip),&peer_port);
 	belle_sip_channel_init(obj,stack,bindip,localport,NULL,remoteip,peer_port);
-	obj->peer_list=obj->current_peer=belle_sip_ip_address_to_addrinfo(ai.ai_family, obj->peer_name,obj->peer_port);
+	obj->peer_list=obj->current_peer=bctbx_ip_address_to_addrinfo(ai.ai_family, ai.ai_socktype, obj->peer_name,obj->peer_port);
 	obj->ai_family=ai.ai_family;
 }
 
@@ -724,7 +791,7 @@ int belle_sip_channel_matches(const belle_sip_channel_t *obj, const belle_sip_ho
 		return 1;
 	}
 	if (addr && obj->current_peer)
-		return addr->ai_addrlen==obj->current_peer->ai_addrlen && memcmp(addr->ai_addr,obj->current_peer->ai_addr,addr->ai_addrlen)==0;
+		return bctbx_sockaddr_equals(addr->ai_addr,obj->current_peer->ai_addr);
 	return 0;
 }
 
@@ -789,6 +856,7 @@ int belle_sip_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 void belle_sip_channel_close(belle_sip_channel_t *obj){
 	if (BELLE_SIP_OBJECT_VPTR(obj,belle_sip_channel_t)->close)
 		BELLE_SIP_OBJECT_VPTR(obj,belle_sip_channel_t)->close(obj); /*udp channel doesn't have close function*/
+	/*removing the source (our base class) will decrement the ref count, this why this code needs to be protected by ref/unref.*/
 	belle_sip_main_loop_remove_source(obj->stack->ml,(belle_sip_source_t*)obj);
 	belle_sip_source_uninit((belle_sip_source_t*)obj);
 }
@@ -838,10 +906,25 @@ static void channel_end_recv_background_task(belle_sip_channel_t *obj){
 }
 
 static void channel_invoke_state_listener(belle_sip_channel_t *obj){
-	if (obj->state==BELLE_SIP_CHANNEL_DISCONNECTED || obj->state==BELLE_SIP_CHANNEL_ERROR){
-		belle_sip_channel_close(obj);
+	int close = FALSE;
+	switch(obj->state){
+		case BELLE_SIP_CHANNEL_DISCONNECTED:
+		case BELLE_SIP_CHANNEL_ERROR:
+		/*the background tasks must be released "after" notifying the app of the disconnected or error state
+		 By "after" it is means not before the main loop iteration that will notify the app.
+		 This is the reason why these calls are done here rather than in the channel_set_state() function.*/
+		channel_end_send_background_task(obj);
+		channel_end_recv_background_task(obj);
+		close = TRUE;
+		break;
+		default:
+		break;
 	}
+	/*Channel listeners may drop the last reference of the channel, so protect by ref/unref until we finish.*/
+	belle_sip_object_ref(obj);
 	BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_state_changed,obj,obj->state);
+	if (close) belle_sip_channel_close(obj);
+	belle_sip_object_unref(obj);
 }
 
 static void channel_invoke_state_listener_defered(belle_sip_channel_t *obj){
@@ -869,7 +952,6 @@ static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 	}/*else the channel was previously working good with the current ip address but now fails, so let's notify the error*/
 
 	obj->state=BELLE_SIP_CHANNEL_ERROR;
-	channel_end_send_background_task(obj);
 	/*Because error notification will in practice trigger the destruction of possible transactions and this channel,
 	* it is safer to invoke the listener outside the current call stack.
 	* Indeed the channel encounters network errors while being called for transmiting by a transaction.
@@ -900,9 +982,6 @@ void channel_set_state(belle_sip_channel_t *obj, belle_sip_channel_state_t state
 		belle_sip_channel_handle_error(obj);
 	}else{
 		obj->state=state;
-		if (state==BELLE_SIP_CHANNEL_DISCONNECTED){
-			channel_end_send_background_task(obj);
-		}
 		channel_invoke_state_listener(obj);
 	}
 }
@@ -954,7 +1033,7 @@ static size_t find_non_printable(const char *buffer, size_t size){
 static char *make_logbuf(belle_sip_channel_t *obj, belle_sip_log_level level, const char *buffer, size_t size){
 	char *logbuf;
 	char truncate_msg[128]={0};
-	int limit=7000; /*big message when many ice candidates*/
+	size_t limit=7000; /*big message when many ice candidates*/
 
 	if (!belle_sip_log_level_enabled(level)){
 		return NULL;
@@ -962,7 +1041,7 @@ static char *make_logbuf(belle_sip_channel_t *obj, belle_sip_log_level level, co
 	if (obj->stop_logging_buffer == 1) {
 		return NULL;
 	}
-	limit=find_non_printable(buffer,MIN((int)size,limit));
+	limit=find_non_printable(buffer,MIN(size,limit));
 	if (limit != size) {
 		belle_sip_message("channel [%p]: found binary data in buffer, will stop logging it now.", obj);
 		obj->stop_logging_buffer = 1;
@@ -970,7 +1049,7 @@ static char *make_logbuf(belle_sip_channel_t *obj, belle_sip_log_level level, co
 			snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (binary data)");
 		} else {
 			size=limit;
-			snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (first %i bytes shown)",limit);
+			snprintf(truncate_msg,sizeof(truncate_msg)-1,"... (first %u bytes shown)",(unsigned int)limit);
 		}
 	}
 
@@ -997,7 +1076,7 @@ static int send_buffer(belle_sip_channel_t *obj, const char *buffer, size_t size
 		/*for testing purpose only */
 		ret=obj->stack->send_error;
 	} else {
-		ret=size; /*to silently discard message*/
+		ret=(int)size; /*to silently discard message*/
 	}
 
 	if (ret<0){
@@ -1042,29 +1121,56 @@ static int send_buffer(belle_sip_channel_t *obj, const char *buffer, size_t size
 
 static void check_content_length(belle_sip_message_t *msg, size_t body_len){
 	belle_sip_header_content_length_t *ctlen=belle_sip_message_get_header_by_type(msg,belle_sip_header_content_length_t);
-	unsigned int value=ctlen ? belle_sip_header_content_length_get_content_length(ctlen) : 0;
+	size_t value=ctlen ? belle_sip_header_content_length_get_content_length(ctlen) : 0;
 	if (body_len){
 		if (ctlen==NULL){
-			belle_sip_message("message [%p] has body of size [%i] but no Content-Length, adding it.",msg,(int)body_len);
+			belle_sip_message("message [%p] has body of size ["FORMAT_SIZE_T"] but no Content-Length, adding it.",msg,body_len);
 			belle_sip_message_add_header(msg,
-				(belle_sip_header_t*)belle_sip_header_content_length_create((int)body_len)
+				(belle_sip_header_t*)belle_sip_header_content_length_create(body_len)
 			);
 		}else{
-			if (value!=(unsigned int)body_len){
-				belle_sip_warning("message [%p] has Content-Length [%u] and body size [%i] which are inconsistent, fixing it.",
-					msg, value, (int)body_len);
-				belle_sip_header_content_length_set_content_length(ctlen,(int)value);
+			if (value!=body_len){
+				belle_sip_warning("message [%p] has Content-Length ["FORMAT_SIZE_T"] and body size ["FORMAT_SIZE_T"] which are inconsistent, fixing it.",
+					msg, value, body_len);
+				belle_sip_header_content_length_set_content_length(ctlen,body_len);
 			}
 		}
 	}else{
 		/*no body, or undetermined size body*/
 		if (ctlen && value!=0){
-			belle_sip_error("message [%p] has Content-Length [%u], but without body or body with undetermined size. Fix your app.",
+			belle_sip_error("message [%p] has Content-Length ["FORMAT_SIZE_T"], but without body or body with undetermined size. Fix your app.",
 				msg,value);
 		}
 	}
 }
 
+static void compress_body_if_required(belle_sip_message_t *msg) {
+	belle_sip_body_handler_t *bh = belle_sip_message_get_body_handler(msg);
+	belle_sip_memory_body_handler_t *mbh = NULL;
+	belle_sip_header_t *ceh = NULL;
+	size_t body_len = 0;
+
+	if (bh != NULL) {
+		body_len = belle_sip_message_get_body_size(msg);
+		ceh = belle_sip_message_get_header(msg, "Content-Encoding");
+	}
+	if ((body_len > 0) && (ceh != NULL)) {
+		const char *content_encoding = belle_sip_header_get_unparsed_value(ceh);
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(bh, belle_sip_multipart_body_handler_t)) {
+			char *marshalled_content = belle_sip_object_to_string(BELLE_SIP_OBJECT(bh));
+			mbh = belle_sip_memory_body_handler_new_from_buffer(marshalled_content, strlen(marshalled_content), NULL, NULL);
+			bh = BELLE_SIP_BODY_HANDLER(mbh);
+			belle_sip_message_set_body_handler(msg, bh);
+		}
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(bh, belle_sip_memory_body_handler_t)) {
+			mbh = BELLE_SIP_MEMORY_BODY_HANDLER(bh);
+			belle_sip_memory_body_handler_apply_encoding(mbh, content_encoding);
+		} else {
+			belle_sip_warning("message [%p] has Content-Encoding [%s] that cannot be applied", msg, content_encoding);
+		}
+	}
+}
+
 static void _send_message(belle_sip_channel_t *obj){
 	char buffer[belle_sip_send_network_buffer_size];
 	size_t len=0;
@@ -1175,6 +1281,7 @@ static void _send_message(belle_sip_channel_t *obj){
 static void send_message(belle_sip_channel_t *obj, belle_sip_message_t *msg){
 	obj->cur_out_message=(belle_sip_message_t*)belle_sip_object_ref(msg);
 	obj->out_state=OUTPUT_STREAM_SENDING_HEADERS;
+	compress_body_if_required(obj->cur_out_message);
 	_send_message(obj);
 }
 
@@ -1239,7 +1346,7 @@ void belle_sip_channel_set_ready(belle_sip_channel_t *obj, const struct sockaddr
 		socklen_t slen2=sizeof(saddr);
 		int err;
 
-		belle_sip_address_remove_v4_mapping(addr,(struct sockaddr*) &saddr,&slen2);
+		bctbx_sockaddr_remove_v4_mapping(addr,(struct sockaddr*) &saddr,&slen2);
 
 		err=getnameinfo((struct sockaddr*)&saddr,slen2,name,sizeof(name),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 		if (err!=0){
@@ -1274,7 +1381,7 @@ void belle_sip_channel_resolve(belle_sip_channel_t *obj){
 	belle_sip_message("channel [%p]: starting resolution of %s", obj, obj->peer_name);
 	channel_set_state(obj,BELLE_SIP_CHANNEL_RES_IN_PROGRESS);
 	if (belle_sip_stack_dns_srv_enabled(obj->stack) && obj->lp!=NULL)
-		obj->resolver_ctx=belle_sip_stack_resolve(obj->stack, belle_sip_channel_get_transport_name_lower_case(obj), obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
+		obj->resolver_ctx=belle_sip_stack_resolve(obj->stack, "sip", belle_sip_channel_get_transport_name_lower_case(obj), obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
 	else
 		obj->resolver_ctx=belle_sip_stack_resolve_a(obj->stack, obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
 	if (obj->resolver_ctx){
@@ -1288,7 +1395,7 @@ void belle_sip_channel_connect(belle_sip_channel_t *obj){
 	int port=obj->peer_port;
 
 	channel_set_state(obj,BELLE_SIP_CHANNEL_CONNECTING);
-	belle_sip_addrinfo_to_ip(obj->current_peer,ip,sizeof(ip),&port);
+	bctbx_addrinfo_to_ip_address(obj->current_peer,ip,sizeof(ip),&port);
 	/* update peer_port as it may have been overriden by SRV resolution*/
 	if (port!=obj->peer_port){
 		/*the SRV resolution provided a port number that must be used*/
@@ -1367,20 +1474,10 @@ belle_sip_channel_t *belle_sip_channel_find_from_list_with_addrinfo(belle_sip_li
 
 /* search a matching channel from a list according to supplied hop. The ai_family tells which address family is supported by the list of channels*/
 belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int ai_family, const belle_sip_hop_t *hop){
-	struct addrinfo *res=NULL;
-	struct addrinfo hints={0};
-	char portstr[20];
-	belle_sip_channel_t *chan;
-
-	hints.ai_family=ai_family;
-	hints.ai_flags=AI_NUMERICHOST|AI_NUMERICSERV;
-	hints.ai_socktype=SOCK_STREAM; // needed on some platforms that return an error otherwise (QNX)
-	if (ai_family==AF_INET6) hints.ai_flags|=AI_V4MAPPED|AI_ALL;
-	snprintf(portstr,sizeof(portstr),"%i",hop->port);
-	belle_sip_getaddrinfo(hop->host,portstr,&hints,&res);
-
+	belle_sip_channel_t *chan=NULL;
+	struct addrinfo *res = bctbx_ip_address_to_addrinfo(ai_family,SOCK_STREAM/*needed on some platforms that return an error otherwise (QNX)*/,hop->host,hop->port);
 	chan=belle_sip_channel_find_from_list_with_addrinfo(l,hop,res);
-	if (res) belle_sip_freeaddrinfo(res);
+	if (res) bctbx_freeaddrinfo(res);
 	return chan;
 }
 
