@@ -76,7 +76,7 @@ typedef struct belle_sip_channel_input_stream{
 	char* read_ptr;
 	char* write_ptr;
 	belle_sip_message_t *msg;
-	int content_length;
+	size_t content_length;
 	int chuncked_mode;
 	int chunk_size;
 	int chunk_read_size;
@@ -112,7 +112,7 @@ struct belle_sip_channel{
 	belle_sip_list_t* incoming_messages;
 	belle_sip_source_t *inactivity_timer;
 	uint64_t last_recv_time;
-	int simulated_recv_return; /* used to simulate network error. 0= no data (disconnected) >0= do nothing -1= network error*/
+	int simulated_recv_return; /* used to simulate network error. 0= no data (disconnected) >0= do nothing -1= network error, 1500 special number to silently discard incoming buffer*/
 	unsigned long bg_task_id;
 	unsigned long recv_bg_task_id;
 	unsigned char force_close; /* when channel is intentionnaly disconnected, in order to prevent looping notifications*/
@@ -221,10 +221,14 @@ belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int a
 
 #define BELLE_SIP_TLS_CHANNEL(obj)		BELLE_SIP_CAST(obj,belle_sip_tls_channel_t)
 
-struct belle_tls_verify_policy{
+struct belle_tls_crypto_config{
 	belle_sip_object_t base;
-	char *root_ca;
-	int exception_flags;
+	char *root_ca; /**< path to the trusted certificate chain used when verifiying peer certificate */
+	int exception_flags; /**< override some exception raised during certificate verification, can be:
+						   BELLE_TLS_VERIFY_NONE do not override any exception
+						   BELLE_TLS_VERIFY_CN_MISMATCH ignore Common Name mismatch exception
+						   BELLE_TLS_VERIFY_ANY_REASON(ignore any exception */
+	void *ssl_config; /**< externally provided ssl configuration context, will be casted and given to the underlying crypto library, use only if you really know what you're doing */
 };
 
 #endif
