@@ -47,6 +47,11 @@ static void belle_sip_dialog_uninit(belle_sip_dialog_t *obj){
 		belle_sip_object_unref(obj->last_transaction);
 	if(obj->privacy)
 		belle_sip_object_unref(obj->privacy);
+	if (obj->expiration_timer){ /*In some situations, dialog deleted might not be called*/
+		belle_sip_main_loop_remove_source(obj->provider->stack->ml, obj->expiration_timer);
+		belle_sip_object_unref(obj->expiration_timer);
+		obj->expiration_timer = NULL;
+	}
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_dialog_t);
@@ -92,22 +97,18 @@ static void check_route_set(belle_sip_list_t *rs){
 	}
 }
 
-static int belle_sip_dialog_init_as_uas(belle_sip_dialog_t *obj, belle_sip_request_t *req, belle_sip_response_t *resp){
+static int belle_sip_dialog_init_as_uas(belle_sip_dialog_t *obj, belle_sip_request_t *req){
 	const belle_sip_list_t *elem;
 	belle_sip_header_contact_t *ct=belle_sip_message_get_header_by_type(req,belle_sip_header_contact_t);
 	belle_sip_header_cseq_t *cseq=belle_sip_message_get_header_by_type(req,belle_sip_header_cseq_t);
-	belle_sip_header_to_t *to=belle_sip_message_get_header_by_type(resp,belle_sip_header_to_t);
 	belle_sip_header_via_t *via=belle_sip_message_get_header_by_type(req,belle_sip_header_via_t);
 	belle_sip_uri_t *requri=belle_sip_request_get_uri(req);
+	belle_sip_header_to_t *to=belle_sip_message_get_header_by_type(req,belle_sip_header_to_t);
 
 	if (!ct){
 		belle_sip_error("No contact in request.");
 		return -1;
 	}
-	if (!to){
-		belle_sip_error("No to in response.");
-		return -1;
-	}
 	if (!cseq){
 		belle_sip_error("No cseq in request.");
 		return -1;
@@ -116,6 +117,11 @@ static int belle_sip_dialog_init_as_uas(belle_sip_dialog_t *obj, belle_sip_reque
 		belle_sip_error("No via in request.");
 		return -1;
 	}
+	if (!to){
+		belle_sip_error("No to in request.");
+		return -1;
+	}
+
 	if (strcasecmp(belle_sip_header_via_get_protocol(via),"TLS")==0
 	    && belle_sip_uri_is_secure(requri)){
 		obj->is_secure=TRUE;
@@ -137,10 +143,10 @@ static int belle_sip_dialog_init_as_uas(belle_sip_dialog_t *obj, belle_sip_reque
 	if (strcmp(belle_sip_request_get_method(req),"INVITE")==0){
 		obj->remote_invite_cseq = belle_sip_header_cseq_get_seq_number(cseq);
 	}
-	/*call id already set */
+
 	/*remote party already set */
 	obj->local_party=(belle_sip_header_address_t*)belle_sip_object_ref(to);
-
+	
 	return 0;
 }
 
@@ -150,16 +156,14 @@ static void set_last_out_invite(belle_sip_dialog_t *obj, belle_sip_request_t *re
 	obj->last_out_invite=(belle_sip_request_t*)belle_sip_object_ref(req);
 }
 
-static int belle_sip_dialog_init_as_uac(belle_sip_dialog_t *obj, belle_sip_request_t *req, belle_sip_response_t *resp){
-	const belle_sip_list_t *elem;
-	belle_sip_header_contact_t *ct=belle_sip_message_get_header_by_type(resp,belle_sip_header_contact_t);
+static int belle_sip_dialog_init_as_uac(belle_sip_dialog_t *obj, belle_sip_request_t *req){
 	belle_sip_header_cseq_t *cseq=belle_sip_message_get_header_by_type(req,belle_sip_header_cseq_t);
-	belle_sip_header_to_t *to=belle_sip_message_get_header_by_type(resp,belle_sip_header_to_t);
+	belle_sip_header_to_t *to=belle_sip_message_get_header_by_type(req,belle_sip_header_to_t);
 	belle_sip_header_via_t *via=belle_sip_message_get_header_by_type(req,belle_sip_header_via_t);
 	belle_sip_uri_t *requri=belle_sip_request_get_uri(req);
 
 	if (!to){
-		belle_sip_error("No to in response.");
+		belle_sip_error("No to in request.");
 		return -1;
 	}
 	if (!cseq){
@@ -171,27 +175,12 @@ static int belle_sip_dialog_init_as_uac(belle_sip_dialog_t *obj, belle_sip_reque
 		return -1;
 	}
 
-	if (strcasecmp(belle_sip_header_via_get_protocol(via),"TLS")==0
+	if (belle_sip_header_via_get_protocol(via) /*might be null at dialog creation (uac case)*/
+		&& strcasecmp(belle_sip_header_via_get_protocol(via),"TLS")==0
 	    && belle_sip_uri_is_secure(requri)){
 		obj->is_secure=TRUE;
 	}
-	/**12.1.2
-	 *  The route set MUST be set to the list of URIs in the Record-Route
-   	 *header field from the response, taken in reverse order and preserving
-   	 *all URI parameters.  If no Record-Route header field is present in
-   	 *the response, the route set MUST be set to the empty set.
-   	 **/
-	obj->route_set=belle_sip_list_free_with_data(obj->route_set,belle_sip_object_unref);
-	for(elem=belle_sip_message_get_headers((belle_sip_message_t*)resp,BELLE_SIP_RECORD_ROUTE);elem!=NULL;elem=elem->next){
-		obj->route_set=belle_sip_list_prepend(obj->route_set,belle_sip_object_ref(belle_sip_header_route_create(
-		                                     (belle_sip_header_address_t*)elem->data)));
-	}
-
-	check_route_set(obj->route_set);
-	/*contact might be provided later*/
-	if (ct)
-		obj->remote_target=(belle_sip_header_address_t*)belle_sip_object_ref(ct);
-
+	
 	obj->local_cseq=belle_sip_header_cseq_get_seq_number(cseq);
 	/*call id is already set */
 	/*local_tag is already set*/
@@ -200,11 +189,13 @@ static int belle_sip_dialog_init_as_uac(belle_sip_dialog_t *obj, belle_sip_reque
 	if (strcmp(belle_sip_request_get_method(req),"INVITE")==0){
 		set_last_out_invite(obj,req);
 	}
+	
 	return 0;
 }
 
 static int belle_sip_dialog_expired(belle_sip_dialog_t *dialog){
 	belle_sip_message("Dialog [%p] expired", dialog);
+	dialog->is_expired = TRUE;
 	belle_sip_dialog_delete(dialog);
 	return BELLE_SIP_STOP;
 }
@@ -276,25 +267,45 @@ int belle_sip_dialog_establish_full(belle_sip_dialog_t *obj, belle_sip_request_t
 
 int belle_sip_dialog_establish(belle_sip_dialog_t *obj, belle_sip_request_t *req, belle_sip_response_t *resp){
 	belle_sip_header_to_t *to=belle_sip_message_get_header_by_type(resp,belle_sip_header_to_t);
-	belle_sip_header_call_id_t *call_id=belle_sip_message_get_header_by_type(req,belle_sip_header_call_id_t);
-	int err;
+	belle_sip_header_via_t *via=belle_sip_message_get_header_by_type(req,belle_sip_header_via_t);
+	belle_sip_uri_t *requri=belle_sip_request_get_uri(req);
 
 	if (obj->state != BELLE_SIP_DIALOG_NULL) {
 		belle_sip_error("Dialog [%p] already established.",obj);
 		return -1;
 	}
-	if (!call_id){
-		belle_sip_error("No call-id in response.");
+	if (!to){
+		belle_sip_error("No to in response.");
 		return -1;
 	}
-	obj->call_id=(belle_sip_header_call_id_t*)belle_sip_object_ref(call_id);
-
-	if (obj->is_server)
-		err= belle_sip_dialog_init_as_uas(obj,req,resp);
-	else
-		err= belle_sip_dialog_init_as_uac(obj,req,resp);
 	
-	if (err) return err;
+	if (!obj->is_server) {
+		belle_sip_header_contact_t *ct=belle_sip_message_get_header_by_type(resp,belle_sip_header_contact_t);
+		const belle_sip_list_t* elem;
+		/*contact might be provided later*/
+		if (ct)
+			obj->remote_target=(belle_sip_header_address_t*)belle_sip_object_ref(ct);
+
+		/**12.1.2
+		 *  The route set MUST be set to the list of URIs in the Record-Route
+		 *header field from the response, taken in reverse order and preserving
+		 *all URI parameters.  If no Record-Route header field is present in
+		 *the response, the route set MUST be set to the empty set.
+		 **/
+		obj->route_set=belle_sip_list_free_with_data(obj->route_set,belle_sip_object_unref);
+		for(elem=belle_sip_message_get_headers((belle_sip_message_t*)resp,BELLE_SIP_RECORD_ROUTE);elem!=NULL;elem=elem->next){
+			obj->route_set=belle_sip_list_prepend(obj->route_set,belle_sip_object_ref(belle_sip_header_route_create(
+																													(belle_sip_header_address_t*)elem->data)));
+		}
+		
+		check_route_set(obj->route_set);
+		/*via might be unknown at dialog creation*/
+		if (strcasecmp(belle_sip_header_via_get_protocol(via),"TLS")==0
+			&& belle_sip_uri_is_secure(requri)){
+			obj->is_secure=TRUE;
+		}
+
+	}
 
 	set_to_tag(obj,to);
 
@@ -338,6 +349,10 @@ static int dialog_on_200Ok_end(belle_sip_dialog_t *dialog){
 
 static void belle_sip_dialog_init_200Ok_retrans(belle_sip_dialog_t *obj, belle_sip_response_t *resp){
 	const belle_sip_timer_config_t *cfg=belle_sip_stack_get_timer_config(obj->provider->stack);
+	if (obj->timer_200Ok || obj->timer_200Ok_end) {
+		belle_sip_error("dialog [%p] already has a 200ok retransmition timer ! skipping",obj);
+		return;
+	}
 	obj->timer_200Ok=belle_sip_timeout_source_new((belle_sip_source_func_t)dialog_on_200Ok_timer,obj,cfg->T1);
 	belle_sip_object_set_name((belle_sip_object_t*)obj->timer_200Ok,"dialog_200Ok_timer");
 	belle_sip_main_loop_add_source(obj->provider->stack->ml,obj->timer_200Ok);
@@ -376,6 +391,7 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 	belle_sip_request_t *req=belle_sip_transaction_get_request(transaction);
 	belle_sip_response_t *resp=belle_sip_transaction_get_response(transaction);
 	int code=0;
+	int ret = 0;
 	int is_invite = strcmp(belle_sip_request_get_method(req),"INVITE")==0;
 	int is_subscribe = strcmp(belle_sip_request_get_method(req),"SUBSCRIBE")==0;
 
@@ -400,8 +416,9 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 	if (as_uas) {
 		belle_sip_header_cseq_t* cseq=belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(req),belle_sip_header_cseq_t);
 		obj->remote_cseq=belle_sip_header_cseq_get_seq_number(cseq);
-		if (is_invite)
+		if (is_invite && code>=200 && code<300)
 			obj->remote_invite_cseq = belle_sip_header_cseq_get_seq_number(cseq);
+		/*else ACK is handled by transaction, not dialog*/
 	}
 
 	
@@ -427,7 +444,11 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 					break;
 			}
 			if (code>=200 && code<300 && (is_invite || is_subscribe)){
-				belle_sip_dialog_establish_full(obj,req,resp);
+				if (belle_sip_dialog_establish_full(obj,req,resp) != 0){
+					/*the dialog found this response invalid so notify the transaction layer by returning -1*/
+					ret = -1;
+					goto end;
+				}
 			}
 			if (is_subscribe){
 				if (belle_sip_dialog_schedule_expiration(obj, (belle_sip_message_t*)req) == BELLE_SIP_STOP
@@ -443,30 +464,42 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 			}
 			break;
 		case BELLE_SIP_DIALOG_CONFIRMED:
+			if (code==481 && (is_invite || is_subscribe)) {
+				/*Dialog is terminated in such case*/
+				delete_dialog=TRUE;
+				break;
+			}
+			/*refreshing target is also true in case of subscribe*/
+			if ((is_invite || is_subscribe) && (code>=200 && code<300)) {
+				/*refresh the remote_target*/
+				belle_sip_header_contact_t *ct;
+				if (as_uas){
+					ct=belle_sip_message_get_header_by_type(req,belle_sip_header_contact_t);
+					
+				}else{
+					if (is_invite)
+						set_last_out_invite(obj,req);
+					ct=belle_sip_message_get_header_by_type(resp,belle_sip_header_contact_t);
+				}
+				if (ct){
+					belle_sip_object_unref(obj->remote_target);
+					obj->remote_target=(belle_sip_header_address_t*)belle_sip_object_ref(ct);
+				}
+				
+			}
 			if (is_invite){
 				if (code>=200 && code<300){
-					/*refresh the remote_target*/
-					belle_sip_header_contact_t *ct;
-					if (as_uas){
-						ct=belle_sip_message_get_header_by_type(req,belle_sip_header_contact_t);
-
-					}else{
-						set_last_out_invite(obj,req);
-						ct=belle_sip_message_get_header_by_type(resp,belle_sip_header_contact_t);
-					}
-					if (ct){
-						belle_sip_object_unref(obj->remote_target);
-						obj->remote_target=(belle_sip_header_address_t*)belle_sip_object_ref(ct);
-					}
 					/*handle possible retransmission of 200Ok */
 					if (!as_uas && (is_retransmition=(belle_sip_dialog_handle_200Ok(obj,resp)==0))) {
 						return is_retransmition;
 					} else {
+						if (as_uas)
+							belle_sip_dialog_init_200Ok_retrans(obj,resp);
 						obj->needs_ack=TRUE; /*REINVITE case, ack needed by both uas and uac*/
 					}
 				}else if (code>=300){
 					/*final response, ack will be automatically sent by transaction layer*/
-					obj->needs_ack=FALSE;
+					/* do not need to do anything because  not set yet or set by previous invite transaction obj->needs_ack=FALSE;*/
 				}
 			} else if (strcmp(belle_sip_request_get_method(req),"BYE")==0){
 				/*15.1.1 UAC Behavior
@@ -499,11 +532,13 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 			/*ignore*/
 		break;
 	}
+	
+end:
 	if (delete_dialog) belle_sip_dialog_delete(obj);
 	else {
 		belle_sip_dialog_process_queue(obj);
 	}
-	return 0;
+	return ret;
 }
 
 belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
@@ -512,6 +547,7 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 	const char *from_tag;
 	belle_sip_header_to_t *to;
 	const char *to_tag=NULL;
+	belle_sip_header_call_id_t *call_id=belle_sip_message_get_header_by_type(t->request,belle_sip_header_call_id_t);
 
 	from=belle_sip_message_get_header_by_type(t->request,belle_sip_header_from_t);
 	if (from==NULL){
@@ -523,6 +559,22 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 		belle_sip_error("belle_sip_dialog_new(): no from tag!");
 		return NULL;
 	}
+	
+	to = belle_sip_message_get_header_by_type(t->request,belle_sip_header_to_t);
+	if (to == NULL){
+		belle_sip_error("belle_sip_dialog_new(): no to in request!");
+		return NULL;
+	}
+	to_tag = belle_sip_header_to_get_tag(to);
+	if (to_tag){
+		belle_sip_error("belle_sip_dialog_new(): there is a to tag in the request. This is not allowed"
+			" to create a dialog on such a transaction.");
+		return NULL;
+	}
+	if (!call_id){
+		belle_sip_error("No call-id in response.");
+		return NULL;
+	}
 
 	if (t->last_response) {
 		to=belle_sip_message_get_header_by_type(t->last_response,belle_sip_header_to_t);
@@ -530,18 +582,23 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 			belle_sip_error("belle_sip_dialog_new(): no to!");
 			return NULL;
 		}
-		to_tag=belle_sip_header_to_get_tag(to);
+		to_tag = belle_sip_header_to_get_tag(to);
 	}
 	obj=belle_sip_object_new(belle_sip_dialog_t);
 	obj->terminate_on_bye=1;
 	obj->provider=t->provider;
 	obj->pending_trans_checking_enabled=1;
+	obj->call_id=(belle_sip_header_call_id_t*)belle_sip_object_ref(call_id);
+	
+	belle_sip_object_ref(t);
+	obj->last_transaction=t;
 	
 	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_server_transaction_t)){
 		obj->remote_tag=belle_sip_strdup(from_tag);
-		obj->local_tag=to_tag?belle_sip_strdup(to_tag):NULL; /*might be null at dialog creation*/
+		obj->local_tag=belle_sip_strdup(BELLE_SIP_SERVER_TRANSACTION(t)->to_tag);
 		obj->remote_party=(belle_sip_header_address_t*)belle_sip_object_ref(from);
 		obj->is_server=TRUE;
+		belle_sip_dialog_init_as_uas(obj,belle_sip_transaction_get_request(t));
 	}else{
 		const belle_sip_list_t *predefined_routes=NULL;
 		obj->local_tag=belle_sip_strdup(from_tag);
@@ -552,7 +609,10 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 			predefined_routes!=NULL;predefined_routes=predefined_routes->next){
 			obj->route_set=belle_sip_list_append(obj->route_set,belle_sip_object_ref(predefined_routes->data));
 		}
+		belle_sip_dialog_init_as_uac(obj,belle_sip_transaction_get_request(t));
 	}
+	
+
 	belle_sip_message("New %s dialog [%p] , local tag [%s], remote tag [%s]"
 			,obj->is_server?"server":"client"
 			,obj
@@ -607,18 +667,28 @@ belle_sip_request_t *belle_sip_dialog_create_ack(belle_sip_dialog_t *obj, unsign
 
 static belle_sip_request_t *create_request(belle_sip_dialog_t *obj, const char *method, int full){
 	belle_sip_request_t *req;
+	char* from_tag=NULL, *to_tag=NULL;
 	
 	if (!obj->remote_target){
 		belle_sip_error("dialog [%p]: no remote_target set, unable to create request.", obj);
 		return NULL;
 	}
 	
+	if (!belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(obj->local_party), "tag")) {
+		/*special case for dialog created by server transaction*/
+		from_tag = obj->local_tag;
+	}
+	if (!belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(obj->remote_party), "tag")) {
+		/*special case for dialog created by server transaction*/
+		to_tag = obj->remote_tag;
+	}
+
 	req=belle_sip_request_create(belle_sip_header_address_get_uri(obj->remote_target),
 	                                                method,
 	                                                obj->call_id,
 	                                                belle_sip_header_cseq_create(obj->local_cseq,method),
-	                                                belle_sip_header_from_create(obj->local_party,NULL),
-	                                                belle_sip_header_to_create(obj->remote_party,NULL),
+	                                                belle_sip_header_from_create(obj->local_party,from_tag),
+	                                                belle_sip_header_to_create(obj->remote_party,to_tag),
 	                                                belle_sip_header_via_new(),
 	                                                0);
 	
@@ -634,9 +704,22 @@ static belle_sip_request_t *create_request(belle_sip_dialog_t *obj, const char *
 	return req;
 }
 
+static int dialog_can_create_request(belle_sip_dialog_t *obj, const char *method){
+	if (obj->state != BELLE_SIP_DIALOG_CONFIRMED && obj->state != BELLE_SIP_DIALOG_EARLY
+		&& (strcmp("NOTIFY", method) != 0
+			|| !obj->is_server
+			|| !obj->last_transaction
+			|| strcmp("SUBSCRIBE", belle_sip_transaction_get_method(obj->last_transaction)) !=0)) {
+		belle_sip_error("belle_sip_dialog_create_request(): cannot create [%s] request from dialog [%p] in state [%s]",method,obj,belle_sip_dialog_state_to_string(obj->state));
+		return FALSE;
+	}
+	return TRUE;
+}
+
 belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t *obj, const char *method){
 	belle_sip_request_t *req;
 	
+	if (!dialog_can_create_request(obj, method)) return NULL;
 	if (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0){
 		/*we don't allow requests that can update the dialog's state to be sent asynchronously*/
 		belle_sip_error("belle_sip_dialog_create_queued_request([%p]): [%s] requests are forbidden using this method.",obj,method);
@@ -656,18 +739,15 @@ static void belle_sip_dialog_update_local_cseq(belle_sip_dialog_t *obj, const ch
 
 belle_sip_request_t *belle_sip_dialog_create_request(belle_sip_dialog_t *obj, const char *method){
 	belle_sip_request_t *req;
-
-	if (obj->state != BELLE_SIP_DIALOG_CONFIRMED && obj->state != BELLE_SIP_DIALOG_EARLY) {
-		belle_sip_error("belle_sip_dialog_create_request(): cannot create [%s] request from dialog [%p] in state [%s]",method,obj,belle_sip_dialog_state_to_string(obj->state));
-		return NULL;
-	}
+	
+	if (!dialog_can_create_request(obj, method)) return NULL;
 	/*don't prevent to send a BYE in any case */
 	if (	obj->pending_trans_checking_enabled
 			&& strcmp(method,"BYE")!=0
 			&& obj->last_transaction
 			&& belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(obj->last_transaction))){
 
-		if (obj->state != BELLE_SIP_DIALOG_EARLY && strcmp(method,"UPDATE")!=0) {
+		if (obj->state != BELLE_SIP_DIALOG_EARLY && strcmp(method,"UPDATE")!=0 && strcmp(method,"NOTIFY")!=0) {
 			belle_sip_error("belle_sip_dialog_create_request(): cannot create [%s] request from dialog [%p] while pending [%s] transaction in state [%s]",method,obj,belle_sip_transaction_get_method(obj->last_transaction), belle_sip_transaction_state_to_string(belle_sip_transaction_get_state(obj->last_transaction)));
 			return NULL;
 		} /*else UPDATE transaction can be send in // */
@@ -737,18 +817,18 @@ belle_sip_request_t *belle_sip_dialog_create_queued_request_from(belle_sip_dialo
 }
 
 void belle_sip_dialog_delete(belle_sip_dialog_t *obj){
-	int dropped_transactions;
+	size_t dropped_transactions;
 	
 	if (obj->expiration_timer){
 		belle_sip_main_loop_remove_source(obj->provider->stack->ml, obj->expiration_timer);
 		belle_sip_object_unref(obj->expiration_timer);
 		obj->expiration_timer = NULL;
 	}
-	belle_sip_message("dialog [%p] deleted.",obj);
+	belle_sip_message("Dialog [%p] deleted (is_expired=%i)",obj, obj->is_expired);
 	belle_sip_dialog_stop_200Ok_retrans(obj); /*if any*/
 	set_state(obj,BELLE_SIP_DIALOG_TERMINATED);
 	dropped_transactions=belle_sip_list_size(obj->queued_ct);
-	if (dropped_transactions>0) belle_sip_warning("dialog [%p]: leaves %i queued transaction aborted.",obj,dropped_transactions);
+	if (dropped_transactions>0) belle_sip_warning("dialog [%p]: leaves %u queued transaction aborted.",obj,(unsigned int)dropped_transactions);
 	belle_sip_list_for_each(obj->queued_ct,(void(*)(void*))belle_sip_transaction_terminate);
 	obj->queued_ct=belle_sip_list_free_with_data(obj->queued_ct,belle_sip_object_unref);
 	belle_sip_provider_remove_dialog(obj->provider,obj);
@@ -854,7 +934,7 @@ int belle_sip_dialog_match(belle_sip_dialog_t *obj, belle_sip_message_t *msg, in
 
 int _belle_sip_dialog_match(belle_sip_dialog_t *obj, const char *call_id, const char *local_tag, const char *remote_tag){
 	const char *dcid;
-	if (obj->state==BELLE_SIP_DIALOG_NULL) belle_sip_fatal("_belle_sip_dialog_match() must not be used for dialog in null state.");
+	/*Dialog created by notify matching subscription are still in NULL state if (obj->state==BELLE_SIP_DIALOG_NULL) belle_sip_fatal("_belle_sip_dialog_match() must not be used for dialog in null state.");*/
 	dcid=belle_sip_header_call_id_get_call_id(obj->call_id);
 	return strcmp(dcid,call_id)==0
 		&& strcmp(obj->local_tag,local_tag)==0 
@@ -915,7 +995,7 @@ belle_sip_transaction_t* belle_sip_dialog_get_last_transaction(const belle_sip_d
 }
 
 int belle_sip_dialog_request_pending(const belle_sip_dialog_t *dialog){
-	return dialog->last_transaction ? belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(dialog->last_transaction)) : FALSE;
+	return dialog->needs_ack || (dialog->last_transaction ? belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(dialog->last_transaction)) : FALSE);
 }
 
 /* for notify exception
@@ -933,9 +1013,6 @@ If an initial SUBSCRIBE is sent on a pre-existing dialog, a matching
    subscription associated with that dialog.
    */
 
-
-
-
 int belle_sip_dialog_is_authorized_transaction(const belle_sip_dialog_t *dialog,const char* method) {
 	if (belle_sip_dialog_request_pending(dialog)){
 		const char* last_transaction_request;
