@@ -1,7 +1,7 @@
 /* ==========================================================================
  * dns.h - Recursive, Reentrant DNS Resolver.
  * --------------------------------------------------------------------------
- * Copyright (c) 2009, 2010, 2012  William Ahern
+ * Copyright (c) 2009, 2010, 2012-2015  William Ahern
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
@@ -51,11 +51,13 @@
 #endif
 
 
-typedef unsigned char DNSBool;
-#ifdef _MSC_VER
-#define DNS_INLINE __inline
-#else
-#define DNS_INLINE inline
+/*
+ * V I S I B I L I T Y
+ *
+ * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+#ifndef DNS_PUBLIC
+#define DNS_PUBLIC
 #endif
 
 
@@ -75,16 +77,16 @@ typedef unsigned char DNSBool;
 
 #define DNS_VENDOR "william@25thandClement.com"
 
-#define DNS_V_REL  0x20121023
-#define DNS_V_ABI  0x20120806
-#define DNS_V_API  0x20120806
+#define DNS_V_REL  0x20160608
+#define DNS_V_ABI  0x20160608
+#define DNS_V_API  0x20160608
 
 
-const char *dns_vendor(void);
+DNS_PUBLIC const char *dns_vendor(void);
 
-int dns_v_rel(void);
-int dns_v_abi(void);
-int dns_v_api(void);
+DNS_PUBLIC int dns_v_rel(void);
+DNS_PUBLIC int dns_v_abi(void);
+DNS_PUBLIC int dns_v_api(void);
 
 
 /*
@@ -115,12 +117,20 @@ enum dns_errno {
 	DNS_ESECTION,
 	DNS_EUNKNOWN,
 	DNS_EADDRESS,
+	DNS_ENOQUERY,
+	DNS_ENOANSWER,
+	DNS_EFETCHED,
+	DNS_ESERVICE, /* EAI_SERVICE */
+	DNS_ENONAME,  /* EAI_NONAME */
+	DNS_EFAIL,    /* EAI_FAIL */
 	DNS_ELAST,
 }; /* dns_errno */
 
-const char *dns_strerror(dns_error_t);
+DNS_PUBLIC const char *dns_strerror(dns_error_t);
 
-extern int dns_debug;
+DNS_PUBLIC int *dns_debug_p(void);
+
+#define dns_debug (*dns_debug_p()) /* was extern int dns_debug before 20160523 API */
 
 
 /*
@@ -155,7 +165,7 @@ extern int dns_debug;
 
 /* GCC parses the _Pragma operator less elegantly than clang. */
 #define dns_quietinit(...) \
-	({ DNS_PRAGMA_PUSH DNS_PRAGMA_QUIET __VA_ARGS__; DNS_PRAGMA_POP })
+	__extension__ ({ DNS_PRAGMA_PUSH DNS_PRAGMA_QUIET __VA_ARGS__; DNS_PRAGMA_POP })
 #else
 #define DNS_PRAGMA_PUSH
 #define DNS_PRAGMA_QUIET
@@ -163,6 +173,12 @@ extern int dns_debug;
 #define dns_quietinit(...) __VA_ARGS__
 #endif
 
+#if defined __GNUC__
+#define DNS_PRAGMA_EXTENSION __extension__
+#else
+#define DNS_PRAGMA_EXTENSION
+#endif
+
 
 /*
  * E V E N T S  I N T E R F A C E S
@@ -239,6 +255,7 @@ enum dns_type {
 	DNS_T_OPT	= 41,
 	DNS_T_SSHFP	= 44,
 	DNS_T_SPF	= 99,
+	DNS_T_AXFR      = 252,
 
 	DNS_T_ALL	= 255
 }; /* enum dns_type */
@@ -265,6 +282,9 @@ enum dns_rcode {
 	DNS_RC_NXRRSET	= 8,
 	DNS_RC_NOTAUTH	= 9,
 	DNS_RC_NOTZONE	= 10,
+
+	/* EDNS(0) extended RCODEs */
+	DNS_RC_BADVERS = 16,
 }; /* dns_rcode */
 
 
@@ -275,41 +295,35 @@ enum dns_rcode {
  */
 #define DNS_STRMAXLEN 47 /* "QUESTION|ANSWER|AUTHORITY|ADDITIONAL" */
 
-const char *dns_strsection(enum dns_section, void *, size_t);
-#ifdef HAVE_C99
+DNS_PUBLIC const char *dns_strsection(enum dns_section, void *, size_t);
 #define dns_strsection3(a, b, c) \
 				dns_strsection((a), (b), (c))
 #define dns_strsection1(a)	dns_strsection((a), (char [DNS_STRMAXLEN + 1]){ 0 }, DNS_STRMAXLEN + 1)
 #define dns_strsection(...)	DNS_PP_CALL(DNS_PP_XPASTE(dns_strsection, DNS_PP_NARG(__VA_ARGS__)), __VA_ARGS__)
-#endif
 
-enum dns_section dns_isection(const char *);
+DNS_PUBLIC enum dns_section dns_isection(const char *);
 
-const char *dns_strclass(enum dns_class, void *, size_t);
-#ifdef HAVE_C99
+DNS_PUBLIC const char *dns_strclass(enum dns_class, void *, size_t);
 #define dns_strclass3(a, b, c)	dns_strclass((a), (b), (c))
 #define dns_strclass1(a)	dns_strclass((a), (char [DNS_STRMAXLEN + 1]){ 0 }, DNS_STRMAXLEN + 1)
 #define dns_strclass(...)	DNS_PP_CALL(DNS_PP_XPASTE(dns_strclass, DNS_PP_NARG(__VA_ARGS__)), __VA_ARGS__)
-#endif
 
-enum dns_class dns_iclass(const char *);
+DNS_PUBLIC enum dns_class dns_iclass(const char *);
 
-const char *dns_strtype(enum dns_type, void *, size_t);
-#ifdef HAVE_C99
+DNS_PUBLIC const char *dns_strtype(enum dns_type, void *, size_t);
 #define dns_strtype3(a, b, c)	dns_strtype((a), (b), (c))
 #define dns_strtype1(a)		dns_strtype((a), (char [DNS_STRMAXLEN + 1]){ 0 }, DNS_STRMAXLEN + 1)
 #define dns_strtype(...)	DNS_PP_CALL(DNS_PP_XPASTE(dns_strtype, DNS_PP_NARG(__VA_ARGS__)), __VA_ARGS__)
-#endif
 
-enum dns_type dns_itype(const char *);
+DNS_PUBLIC enum dns_type dns_itype(const char *);
 
-const char *dns_stropcode(enum dns_opcode);
+DNS_PUBLIC const char *dns_stropcode(enum dns_opcode);
 
-enum dns_opcode dns_iopcode(const char *);
+DNS_PUBLIC enum dns_opcode dns_iopcode(const char *);
 
-const char *dns_strrcode(enum dns_rcode);
+DNS_PUBLIC const char *dns_strrcode(enum dns_rcode);
 
-enum dns_rcode dns_ircode(const char *);
+DNS_PUBLIC enum dns_rcode dns_ircode(const char *);
 
 
 /*
@@ -319,13 +333,19 @@ enum dns_rcode dns_ircode(const char *);
 
 typedef unsigned long dns_atomic_t;
 
+typedef unsigned long dns_refcount_t; /* must be same value type as dns_atomic_t */
+
 
 /*
  * C R Y P T O  I N T E R F A C E S
  *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-extern unsigned (*dns_random)(void);
+typedef unsigned dns_random_f(void);
+
+DNS_PUBLIC dns_random_f **dns_random_p(void);
+
+#define dns_random (*dns_random_p()) /* was extern unsigned (*dns_random)(void) before 20160523 API */
 
 
 /*
@@ -375,22 +395,20 @@ struct dns_header {
 #define DNS_P_DICTSIZE	16
 #endif
 
-#ifndef HAVE_C99
-struct dns_s_memo {
-	unsigned short base, end;
-};
-#endif
-
 struct dns_packet {
 	unsigned short dict[DNS_P_DICTSIZE];
 
-#ifdef HAVE_C99
-	struct dns_s_memo {
-		unsigned short base, end;
-	} qd, an, ns, ar;
-#else
-	struct dns_s_memo qd, an, ns, ar;
-#endif
+	struct dns_p_memo {
+		struct dns_s_memo {
+			unsigned short base, end;
+		} qd, an, ns, ar;
+
+		struct {
+			unsigned short p;
+			unsigned short maxudp;
+			unsigned ttl;
+		} opt;
+	} memo;
 
 	struct { struct dns_packet *cqe_next, *cqe_prev; } cqe;
 
@@ -398,7 +416,7 @@ struct dns_packet {
 
 	int:16; /* tcp padding */
 
-	union {
+	DNS_PRAGMA_EXTENSION union {
 		struct dns_header header;
 		unsigned char data[1];
 	};
@@ -412,30 +430,30 @@ struct dns_packet {
 #define dns_p_new(n)		(dns_p_init((struct dns_packet *)&(union { unsigned char b[dns_p_calcsize((n))]; struct dns_packet p; }){ { 0 } }, dns_p_calcsize((n))))
 
 /** takes size of entire packet structure as allocated */
-struct dns_packet *dns_p_init(struct dns_packet *, size_t);
+DNS_PUBLIC struct dns_packet *dns_p_init(struct dns_packet *, size_t);
 
 /** takes size of maximum desired payload */
-struct dns_packet *dns_p_make(size_t, int *);
+DNS_PUBLIC struct dns_packet *dns_p_make(size_t, int *);
 
-int dns_p_grow(struct dns_packet **);
+DNS_PUBLIC int dns_p_grow(struct dns_packet **);
 
-struct dns_packet *dns_p_copy(struct dns_packet *, const struct dns_packet *);
+DNS_PUBLIC struct dns_packet *dns_p_copy(struct dns_packet *, const struct dns_packet *);
 
 #define dns_p_opcode(P)		(dns_header(P)->opcode)
 
-#define dns_p_rcode(P)		(dns_header(P)->rcode)
+DNS_PUBLIC enum dns_rcode dns_p_rcode(struct dns_packet *);
 
-unsigned dns_p_count(struct dns_packet *, enum dns_section);
+DNS_PUBLIC unsigned dns_p_count(struct dns_packet *, enum dns_section);
 
-int dns_p_push(struct dns_packet *, enum dns_section, const void *, size_t, enum dns_type, enum dns_class, unsigned, const void *);
+DNS_PUBLIC int dns_p_push(struct dns_packet *, enum dns_section, const void *, size_t, enum dns_type, enum dns_class, unsigned, const void *);
 
-void dns_p_dictadd(struct dns_packet *, unsigned short);
+DNS_PUBLIC void dns_p_dictadd(struct dns_packet *, unsigned short);
 
-struct dns_packet *dns_p_merge(struct dns_packet *, enum dns_section, struct dns_packet *, enum dns_section, int *);
+DNS_PUBLIC struct dns_packet *dns_p_merge(struct dns_packet *, enum dns_section, struct dns_packet *, enum dns_section, int *);
 
-void dns_p_dump(struct dns_packet *, FILE *);
+DNS_PUBLIC void dns_p_dump(struct dns_packet *, FILE *);
 
-int dns_p_study(struct dns_packet *);
+DNS_PUBLIC int dns_p_study(struct dns_packet *);
 
 
 /*
@@ -455,21 +473,21 @@ int dns_p_study(struct dns_packet *);
 #define dns_d_new1(a)		dns_d_new3((a), strlen((a)), DNS_D_ANCHOR)
 #define dns_d_new(...)		DNS_PP_CALL(DNS_PP_XPASTE(dns_d_new, DNS_PP_NARG(__VA_ARGS__)), __VA_ARGS__)
 
-char *dns_d_init(void *, size_t, const void *, size_t, int);
+DNS_PUBLIC char *dns_d_init(void *, size_t, const void *, size_t, int);
 
-size_t dns_d_anchor(void *, size_t, const void *, size_t);
+DNS_PUBLIC size_t dns_d_anchor(void *, size_t, const void *, size_t);
 
-size_t dns_d_cleave(void *, size_t, const void *, size_t);
+DNS_PUBLIC size_t dns_d_cleave(void *, size_t, const void *, size_t);
 
-size_t dns_d_comp(void *, size_t, const void *, size_t, struct dns_packet *, int *);
+DNS_PUBLIC size_t dns_d_comp(void *, size_t, const void *, size_t, struct dns_packet *, int *);
 
-size_t dns_d_expand(void *, size_t, unsigned short, struct dns_packet *, int *);
+DNS_PUBLIC size_t dns_d_expand(void *, size_t, unsigned short, struct dns_packet *, int *);
 
-unsigned short dns_d_skip(unsigned short, struct dns_packet *);
+DNS_PUBLIC unsigned short dns_d_skip(unsigned short, struct dns_packet *);
 
-int dns_d_push(struct dns_packet *, const void *, size_t);
+DNS_PUBLIC int dns_d_push(struct dns_packet *, const void *, size_t);
 
-size_t dns_d_cname(void *, size_t, const void *, size_t, struct dns_packet *, int *error);
+DNS_PUBLIC size_t dns_d_cname(void *, size_t, const void *, size_t, struct dns_packet *, int *error);
 
 
 /*
@@ -496,15 +514,15 @@ struct dns_rr {
 }; /* struct dns_rr */
 
 
-int dns_rr_copy(struct dns_packet *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_rr_copy(struct dns_packet *, struct dns_rr *, struct dns_packet *);
 
-int dns_rr_parse(struct dns_rr *, unsigned short, struct dns_packet *);
+DNS_PUBLIC int dns_rr_parse(struct dns_rr *, unsigned short, struct dns_packet *);
 
-unsigned short dns_rr_skip(unsigned short, struct dns_packet *);
+DNS_PUBLIC unsigned short dns_rr_skip(unsigned short, struct dns_packet *);
 
-int dns_rr_cmp(struct dns_rr *, struct dns_packet *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_rr_cmp(struct dns_rr *, struct dns_packet *, struct dns_rr *, struct dns_packet *);
 
-size_t dns_rr_print(void *, size_t, struct dns_rr *, struct dns_packet *, int *);
+DNS_PUBLIC size_t dns_rr_print(void *, size_t, struct dns_rr *, struct dns_packet *, int *);
 
 
 #define dns_rr_i_new(P, ...) \
@@ -519,7 +537,7 @@ struct dns_rr_i {
 
 	int follow;
 
-	int (*sort)();
+	int (*sort)(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
 	unsigned args[2];
 
 	struct {
@@ -531,25 +549,23 @@ struct dns_rr_i {
 	} state, saved;
 }; /* struct dns_rr_i */
 
-int dns_rr_i_packet(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
+DNS_PUBLIC int dns_rr_i_packet(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
 
-int dns_rr_i_order(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
+DNS_PUBLIC int dns_rr_i_order(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
 
-int dns_rr_i_shuffle(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
+DNS_PUBLIC int dns_rr_i_shuffle(struct dns_rr *, struct dns_rr *, struct dns_rr_i *, struct dns_packet *);
 
-struct dns_rr_i *dns_rr_i_init(struct dns_rr_i *, struct dns_packet *);
+DNS_PUBLIC struct dns_rr_i *dns_rr_i_init(struct dns_rr_i *, struct dns_packet *);
 
 #define dns_rr_i_save(i)	((i)->saved = (i)->state)
 #define dns_rr_i_rewind(i)	((i)->state = (i)->saved)
 #define dns_rr_i_count(i)	((i)->state.count)
 
-unsigned dns_rr_grep(struct dns_rr *, unsigned, struct dns_rr_i *, struct dns_packet *, int *);
+DNS_PUBLIC unsigned dns_rr_grep(struct dns_rr *, unsigned, struct dns_rr_i *, struct dns_packet *, int *);
 
-#define dns_rr_foreach_(rr, P, ...)	\
+#define dns_rr_foreach(rr, P, ...)	\
 	for (struct dns_rr_i DNS_PP_XPASTE(i, __LINE__) = *dns_rr_i_new((P), __VA_ARGS__); dns_rr_grep((rr), 1, &DNS_PP_XPASTE(i, __LINE__), (P), &(int){ 0 }); )
 
-#define dns_rr_foreach(...)	dns_rr_foreach_(__VA_ARGS__)
-
 
 /*
  * A  R E S O U R C E  R E C O R D
@@ -559,13 +575,15 @@ struct dns_a {
 	struct in_addr addr;
 }; /* struct dns_a */
 
-int dns_a_parse(struct dns_a *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_a_parse(struct dns_a *, struct dns_rr *, struct dns_packet *);
 
-int dns_a_push(struct dns_packet *, struct dns_a *);
+DNS_PUBLIC int dns_a_push(struct dns_packet *, struct dns_a *);
 
-int dns_a_cmp(const struct dns_a *, const struct dns_a *);
+DNS_PUBLIC int dns_a_cmp(const struct dns_a *, const struct dns_a *);
 
-size_t dns_a_print(void *, size_t, struct dns_a *);
+DNS_PUBLIC size_t dns_a_print(void *, size_t, struct dns_a *);
+
+DNS_PUBLIC size_t dns_a_arpa(void *, size_t, const struct dns_a *);
 
 
 /*
@@ -576,13 +594,15 @@ struct dns_aaaa {
 	struct in6_addr addr;
 }; /* struct dns_aaaa */
 
-int dns_aaaa_parse(struct dns_aaaa *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_aaaa_parse(struct dns_aaaa *, struct dns_rr *, struct dns_packet *);
 
-int dns_aaaa_push(struct dns_packet *, struct dns_aaaa *);
+DNS_PUBLIC int dns_aaaa_push(struct dns_packet *, struct dns_aaaa *);
 
-int dns_aaaa_cmp(const struct dns_aaaa *, const struct dns_aaaa *);
+DNS_PUBLIC int dns_aaaa_cmp(const struct dns_aaaa *, const struct dns_aaaa *);
 
-size_t dns_aaaa_print(void *, size_t, struct dns_aaaa *);
+DNS_PUBLIC size_t dns_aaaa_print(void *, size_t, struct dns_aaaa *);
+
+DNS_PUBLIC size_t dns_aaaa_arpa(void *, size_t, const struct dns_aaaa *);
 
 
 /*
@@ -594,15 +614,15 @@ struct dns_mx {
 	char host[DNS_D_MAXNAME + 1];
 }; /* struct dns_mx */
 
-int dns_mx_parse(struct dns_mx *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_mx_parse(struct dns_mx *, struct dns_rr *, struct dns_packet *);
 
-int dns_mx_push(struct dns_packet *, struct dns_mx *);
+DNS_PUBLIC int dns_mx_push(struct dns_packet *, struct dns_mx *);
 
-int dns_mx_cmp(const struct dns_mx *, const struct dns_mx *);
+DNS_PUBLIC int dns_mx_cmp(const struct dns_mx *, const struct dns_mx *);
 
-size_t dns_mx_print(void *, size_t, struct dns_mx *);
+DNS_PUBLIC size_t dns_mx_print(void *, size_t, struct dns_mx *);
 
-size_t dns_mx_cname(void *, size_t, struct dns_mx *);
+DNS_PUBLIC size_t dns_mx_cname(void *, size_t, struct dns_mx *);
 
 
 /*
@@ -613,15 +633,15 @@ struct dns_ns {
 	char host[DNS_D_MAXNAME + 1];
 }; /* struct dns_ns */
 
-int dns_ns_parse(struct dns_ns *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_ns_parse(struct dns_ns *, struct dns_rr *, struct dns_packet *);
 
-int dns_ns_push(struct dns_packet *, struct dns_ns *);
+DNS_PUBLIC int dns_ns_push(struct dns_packet *, struct dns_ns *);
 
-int dns_ns_cmp(const struct dns_ns *, const struct dns_ns *);
+DNS_PUBLIC int dns_ns_cmp(const struct dns_ns *, const struct dns_ns *);
 
-size_t dns_ns_print(void *, size_t, struct dns_ns *);
+DNS_PUBLIC size_t dns_ns_print(void *, size_t, struct dns_ns *);
 
-size_t dns_ns_cname(void *, size_t, struct dns_ns *);
+DNS_PUBLIC size_t dns_ns_cname(void *, size_t, struct dns_ns *);
 
 
 /*
@@ -632,15 +652,15 @@ struct dns_cname {
 	char host[DNS_D_MAXNAME + 1];
 }; /* struct dns_cname */
 
-int dns_cname_parse(struct dns_cname *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_cname_parse(struct dns_cname *, struct dns_rr *, struct dns_packet *);
 
-int dns_cname_push(struct dns_packet *, struct dns_cname *);
+DNS_PUBLIC int dns_cname_push(struct dns_packet *, struct dns_cname *);
 
-int dns_cname_cmp(const struct dns_cname *, const struct dns_cname *);
+DNS_PUBLIC int dns_cname_cmp(const struct dns_cname *, const struct dns_cname *);
 
-size_t dns_cname_print(void *, size_t, struct dns_cname *);
+DNS_PUBLIC size_t dns_cname_print(void *, size_t, struct dns_cname *);
 
-size_t dns_cname_cname(void *, size_t, struct dns_cname *);
+DNS_PUBLIC size_t dns_cname_cname(void *, size_t, struct dns_cname *);
 
 
 /*
@@ -653,13 +673,13 @@ struct dns_soa {
 	unsigned serial, refresh, retry, expire, minimum;
 }; /* struct dns_soa */
 
-int dns_soa_parse(struct dns_soa *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_soa_parse(struct dns_soa *, struct dns_rr *, struct dns_packet *);
 
-int dns_soa_push(struct dns_packet *, struct dns_soa *);
+DNS_PUBLIC int dns_soa_push(struct dns_packet *, struct dns_soa *);
 
-int dns_soa_cmp(const struct dns_soa *, const struct dns_soa *);
+DNS_PUBLIC int dns_soa_cmp(const struct dns_soa *, const struct dns_soa *);
 
-size_t dns_soa_print(void *, size_t, struct dns_soa *);
+DNS_PUBLIC size_t dns_soa_print(void *, size_t, struct dns_soa *);
 
 
 /*
@@ -670,15 +690,17 @@ struct dns_ptr {
 	char host[DNS_D_MAXNAME + 1];
 }; /* struct dns_ptr */
 
-int dns_ptr_parse(struct dns_ptr *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_ptr_parse(struct dns_ptr *, struct dns_rr *, struct dns_packet *);
 
-int dns_ptr_push(struct dns_packet *, struct dns_ptr *);
+DNS_PUBLIC int dns_ptr_push(struct dns_packet *, struct dns_ptr *);
 
-int dns_ptr_cmp(const struct dns_ptr *, const struct dns_ptr *);
+DNS_PUBLIC int dns_ptr_cmp(const struct dns_ptr *, const struct dns_ptr *);
 
-size_t dns_ptr_print(void *, size_t, struct dns_ptr *);
+DNS_PUBLIC size_t dns_ptr_print(void *, size_t, struct dns_ptr *);
 
-size_t dns_ptr_cname(void *, size_t, struct dns_ptr *);
+DNS_PUBLIC size_t dns_ptr_cname(void *, size_t, struct dns_ptr *);
+
+DNS_PUBLIC size_t dns_ptr_qname(void *, size_t, int, void *);
 
 
 /*
@@ -692,47 +714,58 @@ struct dns_srv {
 	char target[DNS_D_MAXNAME + 1];
 }; /* struct dns_srv */
 
-int dns_srv_parse(struct dns_srv *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_srv_parse(struct dns_srv *, struct dns_rr *, struct dns_packet *);
 
-int dns_srv_push(struct dns_packet *, struct dns_srv *);
+DNS_PUBLIC int dns_srv_push(struct dns_packet *, struct dns_srv *);
 
-int dns_srv_cmp(const struct dns_srv *, const struct dns_srv *);
+DNS_PUBLIC int dns_srv_cmp(const struct dns_srv *, const struct dns_srv *);
 
-size_t dns_srv_print(void *, size_t, struct dns_srv *);
+DNS_PUBLIC size_t dns_srv_print(void *, size_t, struct dns_srv *);
 
-size_t dns_srv_cname(void *, size_t, struct dns_srv *);
+DNS_PUBLIC size_t dns_srv_cname(void *, size_t, struct dns_srv *);
 
 
 /*
  * OPT  R E S O U R C E  R E C O R D
  */
 
-#define DNS_OPT_MINDATA 512
+#ifndef DNS_OPT_MINDATA
+#define DNS_OPT_MINDATA 256
+#endif
 
-#define DNS_OPT_BADVERS 16
+#define DNS_OPT_DNSSEC  0x8000
 
 struct dns_opt {
+	enum dns_rcode rcode;
+	unsigned char version;
+	unsigned short flags;
+
+	union {
+		unsigned short maxsize; /* deprecated as confusing */
+		unsigned short maxudp; /* maximum UDP payload size */
+	};
+
 	size_t size, len;
-
-	unsigned char rcode, version;
-	unsigned short maxsize;
-
 	unsigned char data[DNS_OPT_MINDATA];
 }; /* struct dns_opt */
 
-unsigned int dns_opt_ttl(const struct dns_opt *);
+#define DNS_OPT_INIT(opt) { .size = sizeof (*opt) - offsetof(struct dns_opt, data) }
 
-unsigned short dns_opt_class(const struct dns_opt *);
+DNS_PUBLIC struct dns_opt *dns_opt_init(struct dns_opt *, size_t);
 
-struct dns_opt *dns_opt_init(struct dns_opt *, size_t);
+DNS_PUBLIC int dns_opt_parse(struct dns_opt *, struct dns_rr *, struct dns_packet *);
 
-int dns_opt_parse(struct dns_opt *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_opt_push(struct dns_packet *, struct dns_opt *);
 
-int dns_opt_push(struct dns_packet *, struct dns_opt *);
+DNS_PUBLIC int dns_opt_cmp(const struct dns_opt *, const struct dns_opt *);
 
-int dns_opt_cmp(const struct dns_opt *, const struct dns_opt *);
+DNS_PUBLIC size_t dns_opt_print(void *, size_t, struct dns_opt *);
 
-size_t dns_opt_print(void *, size_t, struct dns_opt *);
+DNS_PUBLIC unsigned int dns_opt_ttl(const struct dns_opt *);
+
+DNS_PUBLIC unsigned short dns_opt_class(const struct dns_opt *);
+
+DNS_PUBLIC dns_error_t dns_opt_data_push(struct dns_opt *, unsigned char, unsigned short, const void *);
 
 
 /*
@@ -754,13 +787,13 @@ struct dns_sshfp {
 	} digest;
 }; /* struct dns_sshfp */
 
-int dns_sshfp_parse(struct dns_sshfp *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_sshfp_parse(struct dns_sshfp *, struct dns_rr *, struct dns_packet *);
 
-int dns_sshfp_push(struct dns_packet *, struct dns_sshfp *);
+DNS_PUBLIC int dns_sshfp_push(struct dns_packet *, struct dns_sshfp *);
 
-int dns_sshfp_cmp(const struct dns_sshfp *, const struct dns_sshfp *);
+DNS_PUBLIC int dns_sshfp_cmp(const struct dns_sshfp *, const struct dns_sshfp *);
 
-size_t dns_sshfp_print(void *, size_t, struct dns_sshfp *);
+DNS_PUBLIC size_t dns_sshfp_print(void *, size_t, struct dns_sshfp *);
 
 
 /*
@@ -776,15 +809,15 @@ struct dns_txt {
 	unsigned char data[DNS_TXT_MINDATA];
 }; /* struct dns_txt */
 
-struct dns_txt *dns_txt_init(struct dns_txt *, size_t);
+DNS_PUBLIC struct dns_txt *dns_txt_init(struct dns_txt *, size_t);
 
-int dns_txt_parse(struct dns_txt *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_txt_parse(struct dns_txt *, struct dns_rr *, struct dns_packet *);
 
-int dns_txt_push(struct dns_packet *, struct dns_txt *);
+DNS_PUBLIC int dns_txt_push(struct dns_packet *, struct dns_txt *);
 
-int dns_txt_cmp(const struct dns_txt *, const struct dns_txt *);
+DNS_PUBLIC int dns_txt_cmp(const struct dns_txt *, const struct dns_txt *);
 
-size_t dns_txt_print(void *, size_t, struct dns_txt *);
+DNS_PUBLIC size_t dns_txt_print(void *, size_t, struct dns_txt *);
 
 
 /*
@@ -805,19 +838,19 @@ union dns_any {
 	struct dns_txt txt, spf, rdata;
 }; /* union dns_any */
 
-#define DNS_ANY_INIT(any) { .rdata = { .size = sizeof *(any) } }
+#define DNS_ANY_INIT(any) { .rdata = { .size = sizeof *(any) - offsetof(struct dns_txt, data) } }
 
-union dns_any *dns_any_init(union dns_any *, size_t);
+DNS_PUBLIC union dns_any *dns_any_init(union dns_any *, size_t);
 
-int dns_any_parse(union dns_any *, struct dns_rr *, struct dns_packet *);
+DNS_PUBLIC int dns_any_parse(union dns_any *, struct dns_rr *, struct dns_packet *);
 
-int dns_any_push(struct dns_packet *, union dns_any *, enum dns_type);
+DNS_PUBLIC int dns_any_push(struct dns_packet *, union dns_any *, enum dns_type);
 
-int dns_any_cmp(const union dns_any *, enum dns_type, const union dns_any *, enum dns_type);
+DNS_PUBLIC int dns_any_cmp(const union dns_any *, enum dns_type, const union dns_any *, enum dns_type);
 
-size_t dns_any_print(void *, size_t, union dns_any *, enum dns_type);
+DNS_PUBLIC size_t dns_any_print(void *, size_t, union dns_any *, enum dns_type);
 
-size_t dns_any_cname(void *, size_t, union dns_any *, enum dns_type);
+DNS_PUBLIC size_t dns_any_cname(void *, size_t, union dns_any *, enum dns_type);
 
 
 /*
@@ -827,27 +860,27 @@ size_t dns_any_cname(void *, size_t, union dns_any *, enum dns_type);
 
 struct dns_hosts;
 
-struct dns_hosts *dns_hosts_open(int *);
+DNS_PUBLIC struct dns_hosts *dns_hosts_open(int *);
 
-void dns_hosts_close(struct dns_hosts *);
+DNS_PUBLIC void dns_hosts_close(struct dns_hosts *);
 
-unsigned dns_hosts_acquire(struct dns_hosts *);
+DNS_PUBLIC dns_refcount_t dns_hosts_acquire(struct dns_hosts *);
 
-unsigned dns_hosts_release(struct dns_hosts *);
+DNS_PUBLIC dns_refcount_t dns_hosts_release(struct dns_hosts *);
 
-struct dns_hosts *dns_hosts_mortal(struct dns_hosts *);
+DNS_PUBLIC struct dns_hosts *dns_hosts_mortal(struct dns_hosts *);
 
-struct dns_hosts *dns_hosts_local(int *);
+DNS_PUBLIC struct dns_hosts *dns_hosts_local(int *);
 
-int dns_hosts_loadfile(struct dns_hosts *, FILE *);
+DNS_PUBLIC int dns_hosts_loadfile(struct dns_hosts *, FILE *);
 
-int dns_hosts_loadpath(struct dns_hosts *, const char *);
+DNS_PUBLIC int dns_hosts_loadpath(struct dns_hosts *, const char *);
 
-int dns_hosts_dump(struct dns_hosts *, FILE *);
+DNS_PUBLIC int dns_hosts_dump(struct dns_hosts *, FILE *);
 
-int dns_hosts_insert(struct dns_hosts *, int, const void *, const void *, DNSBool);
+DNS_PUBLIC int dns_hosts_insert(struct dns_hosts *, int, const void *, const void *, _Bool);
 
-struct dns_packet *dns_hosts_query(struct dns_hosts *, struct dns_packet *, int *);
+DNS_PUBLIC struct dns_packet *dns_hosts_query(struct dns_hosts *, struct dns_packet *, int *);
 
 
 /*
@@ -863,8 +896,11 @@ struct dns_resolv_conf {
 	/* (f)ile, (b)ind, (c)ache */
 	char lookup[4 * (1 + (4 * 2))];
 
+	/* getaddrinfo family by preference order ("inet4", "inet6") */
+	int family[3];
+
 	struct {
-		DNSBool edns0;
+		_Bool edns0;
 
 		unsigned ndots;
 
@@ -872,11 +908,11 @@ struct dns_resolv_conf {
 
 		unsigned attempts;
 
-		DNSBool rotate;
+		_Bool rotate;
 
-		DNSBool recurse;
+		_Bool recurse;
 
-		DNSBool smart;
+		_Bool smart;
 
 		enum {
 			DNS_RESCONF_TCP_ENABLE,
@@ -892,52 +928,62 @@ struct dns_resolv_conf {
 	} _;
 }; /* struct dns_resolv_conf */
 
-struct dns_resolv_conf *dns_resconf_open(int *);
+DNS_PUBLIC struct dns_resolv_conf *dns_resconf_open(int *);
 
-void dns_resconf_close(struct dns_resolv_conf *);
+DNS_PUBLIC void dns_resconf_close(struct dns_resolv_conf *);
 
-unsigned dns_resconf_acquire(struct dns_resolv_conf *);
+DNS_PUBLIC dns_refcount_t dns_resconf_acquire(struct dns_resolv_conf *);
 
-unsigned dns_resconf_release(struct dns_resolv_conf *);
+DNS_PUBLIC dns_refcount_t dns_resconf_release(struct dns_resolv_conf *);
 
-struct dns_resolv_conf *dns_resconf_mortal(struct dns_resolv_conf *);
+DNS_PUBLIC struct dns_resolv_conf *dns_resconf_mortal(struct dns_resolv_conf *);
 
-struct dns_resolv_conf *dns_resconf_local(int *);
+DNS_PUBLIC struct dns_resolv_conf *dns_resconf_local(int *);
 
-struct dns_resolv_conf *dns_resconf_root(int *);
+DNS_PUBLIC struct dns_resolv_conf *dns_resconf_root(int *);
 
-int dns_resconf_loadfile(struct dns_resolv_conf *, FILE *);
+DNS_PUBLIC int dns_resconf_pton(struct sockaddr_storage *, const char *);
 
-int dns_resconf_loadpath(struct dns_resolv_conf *, const char *);
+DNS_PUBLIC int dns_resconf_loadfile(struct dns_resolv_conf *, FILE *);
+
+DNS_PUBLIC int dns_resconf_loadpath(struct dns_resolv_conf *, const char *);
 
 #ifdef USE_FIXED_NAMESERVERS
 int dns_resconf_load_fixed_nameservers(struct dns_resolv_conf *resconf);
 #endif /* USE_FIXED_NAMESERVERS */
+
 #ifdef USE_STRUCT_RES_STATE_NAMESERVERS
 int dns_resconf_load_struct_res_state_nameservers(struct dns_resolv_conf *resconf);
 #endif /* USE_STRUCT_RES_STATE_NAMESERVERS */
+
 #ifdef _WIN32
-int dns_resconf_loadwin(struct dns_resolv_conf *);
+DNS_PUBLIC int dns_resconf_loadwin(struct dns_resolv_conf *);
 #endif
+
 #ifdef ANDROID
-int dns_resconf_loadandroid(struct dns_resolv_conf *resconf);
+DNS_PUBLIC int dns_resconf_loadandroid(struct dns_resolv_conf *resconf);
 #endif
+
 #ifdef HAVE_RESINIT
-int dns_resconf_loadfromresolv(struct dns_resolv_conf *resconf);
+DNS_PUBLIC int dns_resconf_loadfromresolv(struct dns_resolv_conf *resconf);
 #endif /*HAVE_RESINIT*/
-int dns_nssconf_loadfile(struct dns_resolv_conf *, FILE *);
 
-int dns_nssconf_loadpath(struct dns_resolv_conf *, const char *);
+DNS_PUBLIC int dns_nssconf_loadfile(struct dns_resolv_conf *, FILE *);
 
-int dns_resconf_dump(struct dns_resolv_conf *, FILE *);
+DNS_PUBLIC int dns_nssconf_loadpath(struct dns_resolv_conf *, const char *);
 
-int dns_nssconf_dump(struct dns_resolv_conf *, FILE *);
+DNS_PUBLIC int dns_resconf_pton(struct sockaddr_storage *ss, const char *src);
+
+DNS_PUBLIC int dns_resconf_dump(struct dns_resolv_conf *, FILE *);
+
+DNS_PUBLIC int dns_nssconf_dump(struct dns_resolv_conf *, FILE *);
+
+DNS_PUBLIC int dns_resconf_setiface(struct dns_resolv_conf *, const char *, unsigned short);
 
-int dns_resconf_setiface(struct dns_resolv_conf *, const char *, unsigned short);
 
 typedef unsigned long dns_resconf_i_t;
 
-size_t dns_resconf_search(void *, size_t, const void *, size_t, struct dns_resolv_conf *, dns_resconf_i_t *);
+DNS_PUBLIC size_t dns_resconf_search(void *, size_t, const void *, size_t, struct dns_resolv_conf *, dns_resconf_i_t *);
 
 
 /*
@@ -947,23 +993,27 @@ size_t dns_resconf_search(void *, size_t, const void *, size_t, struct dns_resol
 
 struct dns_hints;
 
-struct dns_hints *dns_hints_open(struct dns_resolv_conf *, int *);
+DNS_PUBLIC struct dns_hints *dns_hints_open(struct dns_resolv_conf *, int *);
 
-void dns_hints_close(struct dns_hints *);
+DNS_PUBLIC void dns_hints_close(struct dns_hints *);
 
-unsigned dns_hints_acquire(struct dns_hints *);
+DNS_PUBLIC dns_refcount_t dns_hints_acquire(struct dns_hints *);
 
-unsigned dns_hints_release(struct dns_hints *);
+DNS_PUBLIC dns_refcount_t dns_hints_release(struct dns_hints *);
 
-struct dns_hints *dns_hints_mortal(struct dns_hints *);
+DNS_PUBLIC struct dns_hints *dns_hints_mortal(struct dns_hints *);
 
-int dns_hints_insert(struct dns_hints *, const char *, const struct sockaddr *, unsigned);
+DNS_PUBLIC int dns_hints_insert(struct dns_hints *, const char *, const struct sockaddr *, unsigned);
 
-unsigned dns_hints_insert_resconf(struct dns_hints *, const char *, const struct dns_resolv_conf *, int *);
+DNS_PUBLIC unsigned dns_hints_insert_resconf(struct dns_hints *, const char *, const struct dns_resolv_conf *, int *);
 
-struct dns_hints *dns_hints_local(struct dns_resolv_conf *, int *);
+DNS_PUBLIC struct dns_hints *dns_hints_local(struct dns_resolv_conf *, int *);
 
-struct dns_hints *dns_hints_root(struct dns_resolv_conf *, int *);
+DNS_PUBLIC struct dns_hints *dns_hints_root(struct dns_resolv_conf *, int *);
+
+DNS_PUBLIC struct dns_packet *dns_hints_query(struct dns_hints *, struct dns_packet *, int *);
+
+DNS_PUBLIC int dns_hints_dump(struct dns_hints *, FILE *);
 
 
 struct dns_hints_i {
@@ -977,7 +1027,7 @@ struct dns_hints_i {
 
 #define dns_hints_i_new(...)	(&(struct dns_hints_i){ __VA_ARGS__ })
 
-unsigned dns_hints_grep(struct sockaddr **, socklen_t *, unsigned, struct dns_hints_i *, struct dns_hints *);
+DNS_PUBLIC unsigned dns_hints_grep(struct sockaddr **, socklen_t *, unsigned, struct dns_hints_i *, struct dns_hints *);
 
 
 /*
@@ -988,8 +1038,8 @@ unsigned dns_hints_grep(struct sockaddr **, socklen_t *, unsigned, struct dns_hi
 struct dns_cache {
 	void *state;
 
-	dns_atomic_t (*acquire)(struct dns_cache *);
-	dns_atomic_t (*release)(struct dns_cache *);
+	dns_refcount_t (*acquire)(struct dns_cache *);
+	dns_refcount_t (*release)(struct dns_cache *);
 
 	struct dns_packet *(*query)(struct dns_packet *, struct dns_cache *, int *);
 
@@ -1005,12 +1055,16 @@ struct dns_cache {
 		long i;
 		void *p;
 	} arg[3];
+
+	struct { /* PRIVATE */
+		dns_atomic_t refcount;
+	} _;
 }; /* struct dns_cache */
 
 
-struct dns_cache *dns_cache_init(struct dns_cache *);
+DNS_PUBLIC struct dns_cache *dns_cache_init(struct dns_cache *);
 
-void dns_cache_close(struct dns_cache *);
+DNS_PUBLIC void dns_cache_close(struct dns_cache *);
 
 
 /*
@@ -1070,33 +1124,33 @@ struct dns_stat {
 
 struct dns_socket;
 
-struct dns_socket *dns_so_open(const struct sockaddr *, int, const struct dns_options *, int *error);
+DNS_PUBLIC struct dns_socket *dns_so_open(const struct sockaddr *, int, const struct dns_options *, int *error);
 
-void dns_so_close(struct dns_socket *);
+DNS_PUBLIC void dns_so_close(struct dns_socket *);
 
-void dns_so_reset(struct dns_socket *);
+DNS_PUBLIC void dns_so_reset(struct dns_socket *);
 
-unsigned short dns_so_mkqid(struct dns_socket *so);
+DNS_PUBLIC unsigned short dns_so_mkqid(struct dns_socket *so);
 
-struct dns_packet *dns_so_query(struct dns_socket *, struct dns_packet *, struct sockaddr *, int *);
+DNS_PUBLIC struct dns_packet *dns_so_query(struct dns_socket *, struct dns_packet *, struct sockaddr *, int *);
 
-int dns_so_submit(struct dns_socket *, struct dns_packet *, struct sockaddr *);
+DNS_PUBLIC int dns_so_submit(struct dns_socket *, struct dns_packet *, struct sockaddr *);
 
-int dns_so_check(struct dns_socket *);
+DNS_PUBLIC int dns_so_check(struct dns_socket *);
 
-struct dns_packet *dns_so_fetch(struct dns_socket *, int *);
+DNS_PUBLIC struct dns_packet *dns_so_fetch(struct dns_socket *, int *);
 
-time_t dns_so_elapsed(struct dns_socket *);
+DNS_PUBLIC time_t dns_so_elapsed(struct dns_socket *);
 
-void dns_so_clear(struct dns_socket *);
+DNS_PUBLIC void dns_so_clear(struct dns_socket *);
 
-int dns_so_events(struct dns_socket *);
+DNS_PUBLIC int dns_so_events(struct dns_socket *);
 
-int dns_so_pollfd(struct dns_socket *);
+DNS_PUBLIC int dns_so_pollfd(struct dns_socket *);
 
-int dns_so_poll(struct dns_socket *, int);
+DNS_PUBLIC int dns_so_poll(struct dns_socket *, int);
 
-const struct dns_stat *dns_so_stat(struct dns_socket *);
+DNS_PUBLIC const struct dns_stat *dns_so_stat(struct dns_socket *);
 
 
 /*
@@ -1106,39 +1160,47 @@ const struct dns_stat *dns_so_stat(struct dns_socket *);
 
 struct dns_resolver;
 
-struct dns_resolver *dns_res_open(struct dns_resolv_conf *, struct dns_hosts *hosts, struct dns_hints *, struct dns_cache *, const struct dns_options *, int *);
+DNS_PUBLIC struct dns_resolver *dns_res_open(struct dns_resolv_conf *, struct dns_hosts *hosts, struct dns_hints *, struct dns_cache *, const struct dns_options *, int *);
 
-struct dns_resolver *dns_res_stub(const struct dns_options *, int *);
+DNS_PUBLIC struct dns_resolver *dns_res_stub(const struct dns_options *, int *);
 
-void dns_res_reset(struct dns_resolver *);
+DNS_PUBLIC void dns_res_reset(struct dns_resolver *);
 
-void dns_res_close(struct dns_resolver *);
+DNS_PUBLIC void dns_res_close(struct dns_resolver *);
 
-unsigned dns_res_acquire(struct dns_resolver *);
+DNS_PUBLIC dns_refcount_t dns_res_acquire(struct dns_resolver *);
 
-unsigned dns_res_release(struct dns_resolver *);
+DNS_PUBLIC dns_refcount_t dns_res_release(struct dns_resolver *);
 
-struct dns_resolver *dns_res_mortal(struct dns_resolver *);
+DNS_PUBLIC struct dns_resolver *dns_res_mortal(struct dns_resolver *);
 
-int dns_res_submit(struct dns_resolver *, const char *, enum dns_type, enum dns_class);
+DNS_PUBLIC int dns_res_submit(struct dns_resolver *, const char *, enum dns_type, enum dns_class);
 
-int dns_res_check(struct dns_resolver *);
+DNS_PUBLIC int dns_res_submit2(struct dns_resolver *, const char *, size_t, enum dns_type, enum dns_class);
 
-struct dns_packet *dns_res_fetch(struct dns_resolver *, int *);
+DNS_PUBLIC int dns_res_check(struct dns_resolver *);
 
-time_t dns_res_elapsed(struct dns_resolver *);
+DNS_PUBLIC struct dns_packet *dns_res_fetch(struct dns_resolver *, int *);
 
-void dns_res_clear(struct dns_resolver *);
+DNS_PUBLIC time_t dns_res_elapsed(struct dns_resolver *);
 
-int dns_res_events(struct dns_resolver *);
+DNS_PUBLIC void dns_res_clear(struct dns_resolver *);
 
-int dns_res_pollfd(struct dns_resolver *);
+DNS_PUBLIC int dns_res_events(struct dns_resolver *);
 
-int dns_res_poll(struct dns_resolver *, int);
+DNS_PUBLIC int dns_res_pollfd(struct dns_resolver *);
 
-struct dns_packet *dns_res_query(struct dns_resolver *, const char *, enum dns_type, enum dns_class, int, int *);
+DNS_PUBLIC time_t dns_res_timeout(struct dns_resolver *);
 
-const struct dns_stat *dns_res_stat(struct dns_resolver *);
+DNS_PUBLIC int dns_res_poll(struct dns_resolver *, int);
+
+DNS_PUBLIC struct dns_packet *dns_res_query(struct dns_resolver *, const char *, enum dns_type, enum dns_class, int, int *);
+
+DNS_PUBLIC const struct dns_stat *dns_res_stat(struct dns_resolver *);
+
+DNS_PUBLIC void dns_res_sethints(struct dns_resolver *, struct dns_hints *);
+
+DNS_PUBLIC void dns_res_enable_search(struct dns_resolver *, unsigned char enable);
 
 
 /*
@@ -1148,25 +1210,27 @@ const struct dns_stat *dns_res_stat(struct dns_resolver *);
 
 struct dns_addrinfo;
 
-struct dns_addrinfo *dns_ai_open(const char *, const char *, enum dns_type, const struct addrinfo *, struct dns_resolver *, int *);
+DNS_PUBLIC struct dns_addrinfo *dns_ai_open(const char *, const char *, enum dns_type, const struct addrinfo *, struct dns_resolver *, int *);
 
-void dns_ai_close(struct dns_addrinfo *);
+DNS_PUBLIC void dns_ai_close(struct dns_addrinfo *);
 
-int dns_ai_nextent(struct addrinfo **, struct dns_addrinfo *);
+DNS_PUBLIC int dns_ai_nextent(struct addrinfo **, struct dns_addrinfo *);
 
-size_t dns_ai_print(void *, size_t, struct addrinfo *, struct dns_addrinfo *);
+DNS_PUBLIC size_t dns_ai_print(void *, size_t, struct addrinfo *, struct dns_addrinfo *);
 
-time_t dns_ai_elapsed(struct dns_addrinfo *);
+DNS_PUBLIC time_t dns_ai_elapsed(struct dns_addrinfo *);
 
-void dns_ai_clear(struct dns_addrinfo *);
+DNS_PUBLIC void dns_ai_clear(struct dns_addrinfo *);
 
-int dns_ai_events(struct dns_addrinfo *);
+DNS_PUBLIC int dns_ai_events(struct dns_addrinfo *);
 
-int dns_ai_pollfd(struct dns_addrinfo *);
+DNS_PUBLIC int dns_ai_pollfd(struct dns_addrinfo *);
 
-int dns_ai_poll(struct dns_addrinfo *, int);
+DNS_PUBLIC time_t dns_ai_timeout(struct dns_addrinfo *);
 
-const struct dns_stat *dns_ai_stat(struct dns_addrinfo *);
+DNS_PUBLIC int dns_ai_poll(struct dns_addrinfo *, int);
+
+DNS_PUBLIC const struct dns_stat *dns_ai_stat(struct dns_addrinfo *);
 
 
 /*
@@ -1174,9 +1238,9 @@ const struct dns_stat *dns_ai_stat(struct dns_addrinfo *);
  *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-size_t dns_strlcpy(char *, const char *, size_t);
+DNS_PUBLIC size_t dns_strlcpy(char *, const char *, size_t);
 
-size_t dns_strlcat(char *, const char *, size_t);
+DNS_PUBLIC size_t dns_strlcat(char *, const char *, size_t);
 
 
 /*
@@ -1184,6 +1248,7 @@ size_t dns_strlcat(char *, const char *, size_t);
  *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#define DNS_PP_MIN(a, b) (((a) < (b))? (a) : (b))
 #define DNS_PP_MAX(a, b) (((a) > (b))? (a) : (b))
 #define DNS_PP_NARG_(a, b, c, d, e, f, g, h, i, j, k, N,...) N
 #define DNS_PP_NARG(...)	DNS_PP_NARG_(__VA_ARGS__, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0)
