@@ -45,6 +45,11 @@
 #define BELLE_SIP_WINDOWS_UNIVERSAL 1
 #endif
 #endif
+#ifdef _MSC_VER
+#if (_MSC_VER >= 1900)
+#define BELLE_SIP_MSC_VER_GREATER_19
+#endif
+#endif
 
 #include <winsock2.h>
 #include <ws2tcpip.h>
@@ -79,7 +84,7 @@
 #	endif
 #endif
 
-#if defined(WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
+#if defined(_WIN32) || defined(_WIN32) || defined(__WIN32) || defined(__WIN32__)
 #ifdef BELLESIP_INTERNAL_EXPORTS
 #define BELLESIP_INTERNAL_EXPORT __declspec(dllexport)
 #else
@@ -172,9 +177,6 @@ static BELLESIP_INLINE int inet_aton(const char *ip, struct in_addr *p){
 	return 0;
 }
 
-#define BELLESIP_EWOULDBLOCK WSAEWOULDBLOCK
-#define BELLESIP_EINPROGRESS WSAEINPROGRESS
-
 #else
 
 typedef pthread_t belle_sip_thread_t;
@@ -197,11 +199,10 @@ static BELLESIP_INLINE int get_socket_error(void){
 #define belle_sip_get_socket_error_string() strerror(errno)
 #define belle_sip_get_socket_error_string_from_code(code) strerror(code)
 
-#define BELLESIP_EWOULDBLOCK EWOULDBLOCK
-#define BELLESIP_EINPROGRESS EINPROGRESS
-
 #endif
 
+#define BELLESIP_EWOULDBLOCK BCTBX_EWOULDBLOCK
+#define BELLESIP_EINPROGRESS BCTBX_EINPROGRESS
 #define belle_sip_error_code_is_would_block(err) ((err)==BELLESIP_EWOULDBLOCK || (err)==BELLESIP_EINPROGRESS)
 
 #endif
