@@ -101,6 +101,23 @@ static void channel_state_changed(belle_sip_channel_listener_t *obj, belle_sip_c
 	}
 }
 
+static int notify_client_transaction_match(const void *transaction, const void *notify){
+	belle_sip_client_transaction_t *tr=(belle_sip_client_transaction_t*)transaction;
+	belle_sip_request_t *notify_req=(belle_sip_request_t*)notify;
+	return !belle_sip_client_transaction_is_notify_matching_pending_subscribe(tr,notify_req);
+}
+
+belle_sip_client_transaction_t * belle_sip_provider_find_matching_pending_subscribe_client_transaction_from_notify_req(belle_sip_provider_t *prov, belle_sip_request_t *req) {
+	belle_sip_list_t* elem;
+	if (strcmp("NOTIFY",belle_sip_request_get_method(req)) != 0) {
+		belle_sip_error("belle_sip_provider_find_matching_pending_subscribe_client_transaction_from_notify_req requires a NOTIFY request, not a [%s], on prov [%p]"
+						,belle_sip_request_get_method(req)
+						,prov);
+	}
+	elem=belle_sip_list_find_custom(prov->client_transactions,notify_client_transaction_match,req);
+	return elem?BELLE_SIP_CLIENT_TRANSACTION(elem->data):NULL;
+}
+
 static void belle_sip_provider_dispatch_request(belle_sip_provider_t* prov, belle_sip_request_t *req){
 	belle_sip_server_transaction_t *t;
 	belle_sip_request_event_t ev;
@@ -128,11 +145,18 @@ static void belle_sip_provider_dispatch_request(belle_sip_provider_t* prov, bell
 					belle_sip_response_create_from_request(req,491));
 				return;
 			}
+		} else if (strcmp("NOTIFY",method) == 0) {
+			/*search for matching subscribe*/
+			belle_sip_client_transaction_t *sub = belle_sip_provider_find_matching_pending_subscribe_client_transaction_from_notify_req(prov,req);
+			if (sub) {
+				belle_sip_message("Found matching subscribe for NOTIFY [%p], creating dialog",req);
+				ev.dialog=belle_sip_provider_create_dialog_internal(prov,BELLE_SIP_TRANSACTION(sub),FALSE);
+			}
 		}
 	
-		if (prov->unconditional_answer_enabled && strcmp("ACK",method)!=0) { /*always answer 480 in this case*/
+		if (prov->unconditional_answer_enabled && strcmp("ACK",method)!=0) { /*always answer predefined value (I.E 480 by default)*/
 			belle_sip_server_transaction_t *tr=belle_sip_provider_create_server_transaction(prov,req);
-			belle_sip_server_transaction_send_response(tr,belle_sip_response_create_from_request(req,480));
+			belle_sip_server_transaction_send_response(tr,belle_sip_response_create_from_request(req,prov->unconditional_answer));
 			return;
 		} else {
 			ev.source=(belle_sip_object_t*)prov;
@@ -260,29 +284,29 @@ static void compute_hash_from_invariants(belle_sip_message_t *msg, char *branchi
 
 	belle_sip_md5_init(&ctx);
 	if (initial)
-		belle_sip_md5_append(&ctx,(uint8_t*)initial,strlen(initial));
+		belle_sip_md5_append(&ctx,(uint8_t*)initial,(int)strlen(initial));
 	if (requri){
 		size_t offset=0;
 		belle_sip_object_marshal((belle_sip_object_t*)requri,tmp,sizeof(tmp)-1,&offset);
-		belle_sip_md5_append(&ctx,(uint8_t*)tmp,strlen(tmp));
+		belle_sip_md5_append(&ctx,(uint8_t*)tmp,(int)strlen(tmp));
 	}
 	if (from_tag)
-		belle_sip_md5_append(&ctx,(uint8_t*)from_tag,strlen(from_tag));
+		belle_sip_md5_append(&ctx,(uint8_t*)from_tag,(int)strlen(from_tag));
 	if (to_tag)
-		belle_sip_md5_append(&ctx,(uint8_t*)to_tag,strlen(to_tag));
-	belle_sip_md5_append(&ctx,(uint8_t*)callid,strlen(callid));
+		belle_sip_md5_append(&ctx,(uint8_t*)to_tag,(int)strlen(to_tag));
+	belle_sip_md5_append(&ctx,(uint8_t*)callid,(int)strlen(callid));
 	belle_sip_md5_append(&ctx,(uint8_t*)&cseq,sizeof(cseq));
 	if (is_request){
 		if (prev_via){
 			size_t offset=0;
 			belle_sip_object_marshal((belle_sip_object_t*)prev_via,tmp,sizeof(tmp)-1,&offset);
-			belle_sip_md5_append(&ctx,(uint8_t*)tmp,offset);
+			belle_sip_md5_append(&ctx,(uint8_t*)tmp,(int)offset);
 		}
 	}else{
 		if (via){
 			size_t offset=0;
 			belle_sip_object_marshal((belle_sip_object_t*)via,tmp,sizeof(tmp)-1,&offset);
-			belle_sip_md5_append(&ctx,(uint8_t*)tmp,offset);
+			belle_sip_md5_append(&ctx,(uint8_t*)tmp,(int)offset);
 		}
 	}
 	belle_sip_md5_finish(&ctx,digest);
@@ -312,15 +336,15 @@ static char *compute_rfc2543_branch(belle_sip_request_t *req, char *branchid, si
 
 	belle_sip_md5_init(&ctx);
 
-	belle_sip_md5_append(&ctx,(uint8_t*)from_str,strlen(from_str));
-	belle_sip_md5_append(&ctx,(uint8_t*)to_str,strlen(to_str));
-	belle_sip_md5_append(&ctx,(uint8_t*)callid,strlen(callid));
+	belle_sip_md5_append(&ctx,(uint8_t*)from_str,(int)strlen(from_str));
+	belle_sip_md5_append(&ctx,(uint8_t*)to_str,(int)strlen(to_str));
+	belle_sip_md5_append(&ctx,(uint8_t*)callid,(int)strlen(callid));
 	belle_sip_md5_append(&ctx,(uint8_t*)&cseq,sizeof(cseq));
 	belle_sip_free(from_str);
 	belle_sip_free(to_str);
 
 	if (v_branch)
-		belle_sip_md5_append(&ctx,(uint8_t*)v_branch,strlen(v_branch));
+		belle_sip_md5_append(&ctx,(uint8_t*)v_branch,(int)strlen(v_branch));
 
 	belle_sip_md5_finish(&ctx,digest);
 	belle_sip_octets_to_text(digest,sizeof(digest),branchid,branchid_size);
@@ -422,12 +446,18 @@ static void channel_on_sending(belle_sip_channel_listener_t *obj, belle_sip_chan
 
 		belle_sip_uri_set_host(contact_uri,ip);
 		transport=belle_sip_channel_get_transport_name_lower_case(chan);
-		if (strcmp(transport,"udp")==0){
-			belle_sip_parameters_remove_parameter(BELLE_SIP_PARAMETERS(contact_uri),"transport");
-		}else{
-			if (!belle_sip_uri_is_secure(contact_uri))
-				belle_sip_uri_set_transport_param(contact_uri,transport);
-		}
+		
+		/* Enforce a transport name in "sip" scheme.
+		 * RFC3263 (locating SIP servers) says that UDP SHOULD be used in absence of transport parameter,
+		 * when port or numeric IP are provided. It is a SHOULD, not a must.
+		 * We need in this case that the automatic Contact exactly matches the socket that is going 
+		 * to be used for sending the messages.
+		 * TODO: we may need to do the same for sips, but dtls is currently not supported.
+		 **/
+		
+		if (!belle_sip_uri_is_secure(contact_uri))
+			belle_sip_uri_set_transport_param(contact_uri,transport);
+
 		if (port!=belle_sip_listening_point_get_well_known_port(transport)) {
 			belle_sip_uri_set_port(contact_uri,port);
 		}else{
@@ -440,6 +470,7 @@ static void channel_on_sending(belle_sip_channel_listener_t *obj, belle_sip_chan
  * However some servers (opensips) are confused when they receive a SIP/UDP packet without Content-Length (they shouldn't).
  */
 	if (!content_length
+		&& belle_sip_message_get_body_size(msg) == 0 /*if body present, content_length is automatically added at channel level*/
 #ifndef BELLE_SIP_FORCE_CONTENT_LENGTH
 		&& strcasecmp("udp",belle_sip_channel_get_transport_name(chan))!=0
 #endif
@@ -465,6 +496,7 @@ belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_lis
 	belle_sip_provider_t *p=belle_sip_object_new(belle_sip_provider_t);
 	p->stack=s;
 	p->rport_enabled=1;
+	p->unconditional_answer = 480;
 	if (lp) belle_sip_provider_add_listening_point(p,lp);
 	return p;
 }
@@ -679,7 +711,7 @@ void belle_sip_provider_add_dialog(belle_sip_provider_t *prov, belle_sip_dialog_
 }
 
 static void notify_dialog_terminated(belle_sip_dialog_terminated_event_t* ev) {
-	BELLE_SIP_PROVIDER_INVOKE_LISTENERS(((belle_sip_provider_t*)ev->source)->listeners,process_dialog_terminated,ev);
+	BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_DIALOG(ev->dialog,process_dialog_terminated,ev);
 	belle_sip_object_unref(ev->dialog);
 	belle_sip_free(ev);
 }
@@ -688,10 +720,11 @@ void belle_sip_provider_remove_dialog(belle_sip_provider_t *prov, belle_sip_dial
 	belle_sip_dialog_terminated_event_t* ev=belle_sip_malloc(sizeof(belle_sip_dialog_terminated_event_t));
 	ev->source=prov;
 	ev->dialog=dialog;
+	ev->is_expired=dialog->is_expired;
 	prov->dialogs=belle_sip_list_remove(prov->dialogs,dialog);
 	belle_sip_main_loop_do_later(belle_sip_stack_get_main_loop(prov->stack)
-													,(belle_sip_callback_t) notify_dialog_terminated
-													, ev);
+					,(belle_sip_callback_t) notify_dialog_terminated
+					, ev);
 
 }
 
@@ -1227,5 +1260,7 @@ int belle_sip_provider_nat_helper_enabled(const belle_sip_provider_t *prov){
 }
 void belle_sip_provider_enable_unconditional_answer(belle_sip_provider_t *prov, int enable) {
 	prov->unconditional_answer_enabled=enable;
-
+}
+void belle_sip_provider_set_unconditional_answer(belle_sip_provider_t *prov, unsigned short code) {
+	prov->unconditional_answer=code;
 }
