@@ -104,6 +104,7 @@ static void belle_sip_stack_destroy(belle_sip_stack_t *stack){
 	if (stack->http_proxy_host) belle_sip_free(stack->http_proxy_host);
 	if (stack->http_proxy_passwd) belle_sip_free(stack->http_proxy_passwd);
 	if (stack->http_proxy_username) belle_sip_free(stack->http_proxy_username);
+	belle_sip_list_free_with_data(stack->dns_servers, belle_sip_free);
 
 }
 
@@ -119,6 +120,7 @@ belle_sip_stack_t * belle_sip_stack_new(const char *properties){
 	stack->transport_timeout=63000;
 	stack->dns_timeout=15000;
 	stack->dns_srv_enabled=TRUE;
+	stack->dns_search_enabled=TRUE;
 	stack->inactive_transport_timeout=3600; /*one hour*/
 	return stack;
 }
@@ -160,6 +162,14 @@ void belle_sip_stack_enable_dns_srv(belle_sip_stack_t *stack, unsigned char enab
 	stack->dns_srv_enabled = enable;
 }
 
+unsigned char belle_sip_stack_dns_search_enabled(const belle_sip_stack_t *stack) {
+	return stack->dns_search_enabled;
+}
+
+void belle_sip_stack_enable_dns_search(belle_sip_stack_t *stack, unsigned char enable) {
+	stack->dns_search_enabled = enable;
+}
+
 belle_sip_listening_point_t *belle_sip_stack_create_listening_point(belle_sip_stack_t *s, const char *ipaddress, int port, const char *transport){
 	belle_sip_listening_point_t *lp=NULL;
 	if (strcasecmp(transport,"UDP")==0) {
@@ -251,6 +261,15 @@ void belle_sip_stack_set_dns_resolv_conf_file(belle_sip_stack_t *stack, const ch
 	stack->dns_resolv_conf = resolv_conf_file?belle_sip_strdup(resolv_conf_file):NULL;
 }
 
+void belle_sip_stack_set_dns_servers(belle_sip_stack_t *stack, const belle_sip_list_t *servers){
+	belle_sip_list_t *newservers = NULL;
+	if (servers) newservers = belle_sip_list_copy_with_data(servers, (void *(*)(void*))belle_sip_strdup);
+	if (stack->dns_servers){
+		belle_sip_list_free_with_data(stack->dns_servers, belle_sip_free);
+	}
+	stack->dns_servers = newservers;
+}
+
 const char* belle_sip_version_to_string() {
 #ifdef BELLESIP_VERSION
 	return BELLESIP_VERSION;
@@ -279,6 +298,13 @@ int belle_sip_stack_tls_available(belle_sip_stack_t *stack){
 	return belle_sip_tls_listening_point_available();
 }
 
+int belle_sip_stack_content_encoding_available(belle_sip_stack_t *stack, const char *content_encoding) {
+#ifdef HAVE_ZLIB
+	if (strcmp(content_encoding, "deflate") == 0) return TRUE;
+#endif
+	return FALSE;
+}
+
 GET_SET_STRING(belle_sip_stack,http_proxy_host)
 GET_SET_INT(belle_sip_stack,http_proxy_port, int)
 
