@@ -66,6 +66,7 @@ static void transaction_destroy(belle_sip_transaction_t *t){
 	if (t->channel) belle_sip_object_unref(t->channel);
 	if (t->branch_id) belle_sip_free(t->branch_id);
 	belle_sip_transaction_set_dialog(t,NULL);
+	belle_sip_message("Transaction [%p] deleted",t);
 
 }
 
@@ -94,6 +95,7 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 		break;
 		case BELLE_SIP_CHANNEL_DISCONNECTED:
 		case BELLE_SIP_CHANNEL_ERROR:
+			belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the listener*/
 			ev.transport=belle_sip_channel_get_transport_name(chan);
 			ev.source=BELLE_SIP_OBJECT(t);
 			ev.port=chan->peer_port;
@@ -108,6 +110,7 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 				notify_timeout((belle_sip_transaction_t*)t);
 
 			belle_sip_transaction_terminate(t);
+			belle_sip_object_unref(t);
 		break;
 		default:
 			/*ignored*/
@@ -133,10 +136,19 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_transaction_t)
 	NULL /*on_terminate*/
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
-void *belle_sip_transaction_get_application_data(const belle_sip_transaction_t *t){
+void *belle_sip_transaction_get_application_data_internal(const belle_sip_transaction_t *t){
 	return t->appdata;
 }
 
+void *belle_sip_transaction_get_application_data(const belle_sip_transaction_t *t){
+	if (t->is_internal) {
+		belle_sip_error("belle_sip_transaction_get_application_data should not be used on internal transaction [%p]",t);
+		return NULL;
+	} else {
+		return belle_sip_transaction_get_application_data_internal(t);
+	};
+}
+
 void belle_sip_transaction_set_application_data(belle_sip_transaction_t *t, void *data){
 	t->appdata=data;
 }
@@ -190,8 +202,7 @@ void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t){
 	 * Otherwise it will report the error.
 	 * We limit this dead channel reporting to REGISTER transactions, who are unlikely to be unresponded.
 	**/
-	
-
+	belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the timeout listener*/
 	if (strcmp(belle_sip_request_get_method(t->request),"REGISTER")==0){
 		if ( belle_sip_channel_notify_timeout(t->channel)==TRUE){
 			belle_sip_warning("Transaction [%p] reporting timeout, reporting to channel.",t);
@@ -201,6 +212,7 @@ void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t){
 		notify_timeout(t);
 		belle_sip_transaction_terminate(t);
 	}
+	belle_sip_object_unref(t);
 }
 
 belle_sip_dialog_t*  belle_sip_transaction_get_dialog(const belle_sip_transaction_t *t) {
@@ -408,10 +420,19 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 		belle_sip_dialog_update_request(dialog,req);
 	} else if (t->base.request->dialog_queued){
 		/*this request was created by belle_sip_dialog_create_queued_request().*/
+		
+		if (dialog == NULL){
+			belle_sip_error("belle_sip_client_transaction_send_request(): transaction [%p], cannot send"
+				" request because it was created in the context of a dialog that appears to be "
+				" no longer existing.", t);
+			belle_sip_transaction_terminate(BELLE_SIP_TRANSACTION(t));
+			return -1;
+		}
+		
 		if (belle_sip_dialog_request_pending(dialog) || dialog->queued_ct!=NULL){
 			/*it cannot be sent immediately, queue the transaction into dialog*/
-			belle_sip_message("belle_sip_client_transaction_send_request(): transaction [%p], cannot send request now because dialog is busy"
-			" or other transactions are queued, so queuing into dialog.",t);
+			belle_sip_message("belle_sip_client_transaction_send_request(): transaction [%p], cannot send request now because dialog [%p] is busy"
+			" or other transactions are queued, so queuing into dialog.",t, dialog);
 			belle_sip_dialog_queue_client_transaction(dialog,t);
 			return 0;
 		}
@@ -492,7 +513,7 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 
 	if (dialog && belle_sip_dialog_update(dialog,BELLE_SIP_TRANSACTION(t),FALSE)) {
 		/* retransmition, just return*/
-		belle_sip_message("[%p] is a 200 ok retransmition on dialog [%p], skiping",resp,dialog);
+		belle_sip_message("Response [%p] absorbed by dialog [%p], skipped from transaction layer.",resp,dialog);
 		return;
 	}
 
@@ -502,7 +523,7 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 	event.response=(belle_sip_response_t*)resp;
 	BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(((belle_sip_transaction_t*)t),process_response_event,&event);
 	/*check that 200Ok for INVITEs have been acknowledged by listener*/
-	if (dialog && strcmp(method,"INVITE")==0){
+	if (dialog && status_code>=200 && status_code<300 && strcmp(method,"INVITE")==0){
 		belle_sip_dialog_check_ack_sent(dialog);
 	}
 	/*report a server having internal errors for REGISTER to the channel, in order to go to a fallback IP*/
@@ -585,3 +606,54 @@ belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(b
 	return req;
 }
 
+/*
+ rfc 3265
+ 3.3.4. Dialog creation and termination
+ ...
+ NOTIFY requests are matched to such SUBSCRIBE requests if they
+ contain the same "Call-ID", a "To" header "tag" parameter which
+ matches the "From" header "tag" parameter of the SUBSCRIBE, and the
+ same "Event" header field.  Rules for comparisons of the "Event"
+ headers are described in section 7.2.1.  If a matching NOTIFY request
+ contains a "Subscription-State" of "active" or "pending", it creates
+ a new subscription and a new dialog (unless they have already been
+ created by a matching response, as described above).
+ 
+ 
+ */
+
+int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_client_transaction_t *trans
+																	  , belle_sip_request_t *notify) {
+	belle_sip_request_t *subscription;
+	belle_sip_header_event_t *sub_event, *notif_event;
+	belle_sip_header_call_id_t *sub_call_id, *notif_call_id;
+	const char* sub_from_tag, *notif_to_tag;
+	
+	if (!belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(trans)))
+		|| strcmp("SUBSCRIBE", belle_sip_transaction_get_method(BELLE_SIP_TRANSACTION(trans)))!=0) return 0;
+	
+	
+	if (strcmp("NOTIFY",belle_sip_request_get_method(notify)) != 0) {
+		belle_sip_error("belle_sip_client_transaction_is_notify_matching_pending_subscribe for dialog [%p], requires a notify request",notify);
+	}
+	
+	subscription = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(trans));
+	sub_event = belle_sip_message_get_header_by_type(subscription, belle_sip_header_event_t);
+	if (!sub_event || !belle_sip_header_event_get_package_name(sub_event))
+		return 0;
+	
+	notif_event = belle_sip_message_get_header_by_type(notify, belle_sip_header_event_t);
+	if (!notif_event || !belle_sip_header_event_get_package_name(notif_event))
+		return 0;
+	
+	sub_call_id = belle_sip_message_get_header_by_type(subscription, belle_sip_header_call_id_t);
+	notif_call_id = belle_sip_message_get_header_by_type(notify, belle_sip_header_call_id_t);
+	sub_from_tag = belle_sip_header_from_get_tag(belle_sip_message_get_header_by_type(subscription, belle_sip_header_from_t));
+	notif_to_tag = belle_sip_header_to_get_tag(belle_sip_message_get_header_by_type(notify, belle_sip_header_to_t));
+	
+	return strcmp(belle_sip_header_call_id_get_call_id(sub_call_id),belle_sip_header_call_id_get_call_id(notif_call_id))==0
+	&& sub_from_tag && notif_to_tag && strcmp(sub_from_tag,notif_to_tag)==0
+	&& strcasecmp(belle_sip_header_event_get_package_name(sub_event),belle_sip_header_event_get_package_name(notif_event))==0;
+	
+}
+
