@@ -40,8 +40,7 @@ static void stream_channel_uninit(belle_sip_stream_channel_t *obj){
 
 int stream_channel_send(belle_sip_stream_channel_t *obj, const void *buf, size_t buflen){
 	belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)obj);
-	int err;
-	err=send(sock,buf,buflen,0);
+	int err=bctbx_send(sock,buf,buflen,0);
 	if (err==(belle_sip_socket_t)-1){
 		int errnum=get_socket_error();
 		if (!belle_sip_error_code_is_would_block(errnum)){
@@ -54,8 +53,7 @@ int stream_channel_send(belle_sip_stream_channel_t *obj, const void *buf, size_t
 
 int stream_channel_recv(belle_sip_stream_channel_t *obj, void *buf, size_t buflen){
 	belle_sip_socket_t sock = belle_sip_source_get_socket((belle_sip_source_t*)obj);
-	int err;
-	err=recv(sock,buf,buflen,0);
+	int err=bctbx_recv(sock,buf,buflen,0);
 	if (err==(belle_sip_socket_t)-1){
 		int errnum=get_socket_error();
 		if (!belle_sip_error_code_is_would_block(errnum)){
@@ -135,7 +133,7 @@ int stream_channel_connect(belle_sip_stream_channel_t *obj, const struct addrinf
 		belle_sip_socket_enable_dual_stack(sock);
 	}
 	
-	err = connect(sock,ai->ai_addr,ai->ai_addrlen);
+	err = bctbx_connect(sock,ai->ai_addr,(socklen_t)ai->ai_addrlen);
 	if (err != 0 && get_socket_error()!=BELLESIP_EINPROGRESS && get_socket_error()!=BELLESIP_EWOULDBLOCK) {
 		belle_sip_error("stream connect failed %s",belle_sip_get_socket_error_string());
 		close_socket(sock);
