@@ -23,11 +23,7 @@
 
 #include "belle-sip/auth-helper.h"
 #include "belle_sip_tester.h"
-
-#ifdef HAVE_POLARSSL
-#include <polarssl/version.h>
-#endif
-
+#include <bctoolbox/crypto.h>
 
 static void test_authentication(void) {
 	const char* l_raw_header = "WWW-Authenticate: Digest "
@@ -37,8 +33,8 @@ static void test_authentication(void) {
 	belle_sip_header_www_authenticate_t* www_authenticate=belle_sip_header_www_authenticate_parse(l_raw_header);
 	belle_sip_header_authorization_t* authorization = belle_sip_auth_helper_create_authorization(www_authenticate);
 	belle_sip_header_authorization_set_uri(authorization,belle_sip_uri_parse("sip:sip.linphone.org"));
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_fill_authorization(authorization,"REGISTER",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_fill_authorization(authorization,"REGISTER",ha1), int, "%d");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(authorization),"77ebf3de72e41934d806175586086508");
 	belle_sip_object_unref(www_authenticate);
 	belle_sip_object_unref(authorization);
@@ -55,8 +51,8 @@ static void test_authentication_qop_auth(void) {
 	belle_sip_header_authorization_set_nonce_count(authorization,1);
 	belle_sip_header_authorization_set_qop(authorization,"auth");
 	belle_sip_header_authorization_set_cnonce(authorization,"8302210f"); /*for testing purpose*/
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_fill_authorization(authorization,"REGISTER",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_fill_authorization(authorization,"REGISTER",ha1), int, "%d");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_qop(authorization),"auth");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(authorization),"694dab8dfe7d50d28ba61e8c43e30666");
 	BC_ASSERT_EQUAL(belle_sip_header_authorization_get_nonce_count(authorization),1, int, "%d");
@@ -72,8 +68,8 @@ static void test_proxy_authentication(void) {
 	belle_sip_header_proxy_authenticate_t* proxy_authenticate=belle_sip_header_proxy_authenticate_parse(l_raw_header);
 	belle_sip_header_proxy_authorization_t* proxy_authorization = belle_sip_auth_helper_create_proxy_authorization(proxy_authenticate);
 	belle_sip_header_authorization_set_uri(BELLE_SIP_HEADER_AUTHORIZATION(proxy_authorization),belle_sip_uri_parse("sip:sip.linphone.org"));
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
-	BC_ASSERT_EQUAL_FATAL(0,belle_sip_auth_helper_fill_proxy_authorization(proxy_authorization,"REGISTER",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_compute_ha1("jehan-mac","sip.linphone.org","toto",ha1), int, "%d");
+	BC_ASSERT_EQUAL(0,belle_sip_auth_helper_fill_proxy_authorization(proxy_authorization,"REGISTER",ha1), int, "%d");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_response(BELLE_SIP_HEADER_AUTHORIZATION(proxy_authorization))
 							,"77ebf3de72e41934d806175586086508");
 	belle_sip_object_unref(proxy_authenticate);
@@ -84,8 +80,6 @@ static void test_proxy_authentication(void) {
 #define TEMPORARY_CERTIFICATE_DIR "/belle_sip_tester_crt"
 
 static void test_generate_and_parse_certificates(void) {
-#ifdef HAVE_POLARSSL
-#if POLARSSL_VERSION_NUMBER >= 0x01030000
 	belle_sip_certificates_chain_t *certificate, *parsed_certificate;
 	belle_sip_signing_key_t *key, *parsed_key;
 	char *pem_certificate, *pem_parsed_certificate, *pem_key, *pem_parsed_key;
@@ -94,28 +88,34 @@ static void test_generate_and_parse_certificates(void) {
 
 	/* create 2 certificates in the temporary certificate directory (TODO : set the directory in a absolute path?? where?)*/
 	ret = belle_sip_generate_self_signed_certificate(belle_sip_certificate_temporary_dir, "test_certificate1", &certificate, &key);
-	BC_ASSERT_EQUAL_FATAL(0, ret, int, "%d");
+	if (ret == BCTBX_ERROR_UNAVAILABLE_FUNCTION) {
+		belle_sip_warning("Test skipped, self signed certificate generation not available.");
+		return;
+	}
+	BC_ASSERT_EQUAL(0, ret, int, "%d");
+	belle_sip_object_unref(certificate);
+	belle_sip_object_unref(key);
 	ret = belle_sip_generate_self_signed_certificate(belle_sip_certificate_temporary_dir, "test_certificate2", &certificate, &key);
-	BC_ASSERT_EQUAL_FATAL(0, ret, int, "%d");
+	BC_ASSERT_EQUAL(0, ret, int, "%d");
 
 	/* parse directory to get the certificate2 */
 	ret = belle_sip_get_certificate_and_pkey_in_dir(belle_sip_certificate_temporary_dir, "test_certificate2", &parsed_certificate, &parsed_key, BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
 	free(belle_sip_certificate_temporary_dir);
-	BC_ASSERT_EQUAL_FATAL(0, ret, int, "%d");
+	BC_ASSERT_EQUAL(0, ret, int, "%d");
 
 	/* get pem version of generated and parsed certificate and compare them */
 	pem_certificate = belle_sip_certificates_chain_get_pem(certificate);
-	BC_ASSERT_TRUE_FATAL(pem_certificate!=NULL);
 	pem_parsed_certificate = belle_sip_certificates_chain_get_pem(parsed_certificate);
-	BC_ASSERT_TRUE_FATAL(pem_parsed_certificate!=NULL);
-	BC_ASSERT_STRING_EQUAL(pem_certificate, pem_parsed_certificate);
+	if (BC_ASSERT_PTR_NOT_NULL(pem_certificate) && BC_ASSERT_PTR_NOT_NULL(pem_parsed_certificate)) {
+		BC_ASSERT_STRING_EQUAL(pem_certificate, pem_parsed_certificate);
+	}
 
 	/* get pem version of generated and parsed key and compare them */
 	pem_key = belle_sip_signing_key_get_pem(key);
-	BC_ASSERT_TRUE_FATAL(pem_key!=NULL);
 	pem_parsed_key = belle_sip_signing_key_get_pem(parsed_key);
-	BC_ASSERT_TRUE_FATAL(pem_parsed_key!=NULL);
-	BC_ASSERT_STRING_EQUAL(pem_key, pem_parsed_key);
+	if (BC_ASSERT_PTR_NOT_NULL(pem_key) && BC_ASSERT_PTR_NOT_NULL(pem_parsed_key)) {
+		BC_ASSERT_STRING_EQUAL(pem_key, pem_parsed_key);
+	}
 
 	belle_sip_free(pem_certificate);
 	belle_sip_free(pem_parsed_certificate);
@@ -125,8 +125,6 @@ static void test_generate_and_parse_certificates(void) {
 	belle_sip_object_unref(parsed_certificate);
 	belle_sip_object_unref(key);
 	belle_sip_object_unref(parsed_key);
-#endif /* POLARSSL_VERSION_NUMBER >= 0x01030000 */
-#endif /* HAVE_POLARSSL */
 }
 
 
@@ -159,17 +157,23 @@ const char* belle_sip_tester_fingerprint256_cert_fingerprint =
 		"SHA-256 A0:98:2D:3E:68:F3:14:8D:ED:50:40:DB:ED:A4:28:BC:1E:1A:6A:05:59:9E:69:3F:02:E2:F8:22:BF:4C:92:14";
 
 static void test_certificate_fingerprint(void) {
-#ifdef HAVE_POLARSSL
-#if POLARSSL_VERSION_NUMBER >= 0x01030000
 	char *fingerprint;
+	belle_sip_certificates_chain_t *cert;
+
+	/* check underlying bctoolbox function availability */
+	if (bctbx_x509_certificate_get_fingerprint(NULL, NULL, 0, 0) == BCTBX_ERROR_UNAVAILABLE_FUNCTION) {
+		belle_sip_warning("Test skipped, certificate fingerprint generation not available.");
+		return;
+	}
 	/* parse certificate defined in belle_sip_register_tester.c */
-	belle_sip_certificates_chain_t* cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert,strlen(belle_sip_tester_client_cert),BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
+	cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert,strlen(belle_sip_tester_client_cert),BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
 	/* generate fingerprint */
 	fingerprint = belle_sip_certificates_chain_get_fingerprint(cert);
 
-	BC_ASSERT_TRUE_FATAL(fingerprint!=NULL);
-	BC_ASSERT_STRING_EQUAL(fingerprint, belle_sip_tester_client_cert_fingerprint);
-
+	BC_ASSERT_PTR_NOT_NULL(fingerprint);
+	if (fingerprint) {
+		BC_ASSERT_STRING_EQUAL(fingerprint, belle_sip_tester_client_cert_fingerprint);
+	}
 	belle_sip_free(fingerprint);
 	belle_sip_object_unref(cert);
 
@@ -178,14 +182,13 @@ static void test_certificate_fingerprint(void) {
 	/* generate fingerprint */
 	fingerprint = belle_sip_certificates_chain_get_fingerprint(cert);
 
-	BC_ASSERT_TRUE_FATAL(fingerprint!=NULL);
-	BC_ASSERT_STRING_EQUAL(fingerprint, belle_sip_tester_fingerprint256_cert_fingerprint);
+	BC_ASSERT_PTR_NOT_NULL(fingerprint);
+	if (fingerprint) {
+		BC_ASSERT_STRING_EQUAL(fingerprint, belle_sip_tester_fingerprint256_cert_fingerprint);
+	}
 
 	belle_sip_free(fingerprint);
 	belle_sip_object_unref(cert);
-
-#endif /* POLARSSL_VERSION_NUMBER >= 0x01030000 */
-#endif /* HAVE_POLARSSL */
 }
 
 test_t authentication_helper_tests[] = {
