@@ -18,11 +18,9 @@
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_tester.h"
-#include <stdio.h>
-#include "CUnit/Basic.h"
 
 
-static void test_basic_uri() {
+static void test_basic_uri(void) {
 	belle_generic_uri_t* source_uri = belle_generic_uri_parse("http://www.linphone.org/index.html");
 	char* source_uri_raw = belle_sip_object_to_string(source_uri);
 	belle_generic_uri_t* first_uri = belle_generic_uri_parse(source_uri_raw);
@@ -31,24 +29,24 @@ static void test_basic_uri() {
 	belle_sip_object_unref(source_uri);
 	belle_sip_object_unref(first_uri);
 
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"http");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"www.linphone.org");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/index.html");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"http");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"www.linphone.org");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/index.html");
 
 	belle_sip_object_unref(uri);
 
 	source_uri = belle_generic_uri_parse("http://www.linphone.org/");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/");
 	belle_sip_object_unref(source_uri);
 
 	source_uri = belle_generic_uri_parse("http://www.linphone.org/a/b/c");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/a/b/c");
-	CU_ASSERT_STRING_EQUAL("http://www.linphone.org/a/b/c",source_uri_raw = belle_sip_object_to_string(source_uri));
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/a/b/c");
+	BC_ASSERT_STRING_EQUAL("http://www.linphone.org/a/b/c",source_uri_raw = belle_sip_object_to_string(source_uri));
 	belle_sip_free(source_uri_raw);
 	belle_sip_object_unref(source_uri);
 }
 
-static void test_complex_uri() {
+static void test_complex_uri(void) {
 
 	belle_generic_uri_t* source_uri = belle_generic_uri_parse("ftp://toto:secret@ftp.linphone.fr:1234/url?sa=t&rct=j&url=http%3A%2F%2Ftranslate.google.fr");
 	char* source_uri_raw = belle_generic_uri_to_string(source_uri);
@@ -58,19 +56,19 @@ static void test_complex_uri() {
 	belle_sip_object_unref(source_uri);
 	belle_sip_object_unref(first_uri);
 
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"ftp");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"ftp.linphone.fr");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_user(uri),"toto");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_user_password(uri),"secret");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"ftp.linphone.fr");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/url");
-	CU_ASSERT_EQUAL(belle_generic_uri_get_port(uri),1234);
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_query(uri),"sa=t&rct=j&url=http://translate.google.fr");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"ftp");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"ftp.linphone.fr");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_user(uri),"toto");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_user_password(uri),"secret");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_host(uri),"ftp.linphone.fr");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/url");
+	BC_ASSERT_EQUAL(belle_generic_uri_get_port(uri),1234,int,"%d");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_query(uri),"sa=t&rct=j&url=http://translate.google.fr");
 
 	belle_sip_object_unref(uri);
 }
 
-static void test_file_path() {
+static void test_file_path(void) {
 	belle_generic_uri_t* source_uri = belle_generic_uri_parse("/index.html");
 	char* source_uri_raw = belle_sip_object_to_string(source_uri);
 	belle_generic_uri_t* first_uri = belle_generic_uri_parse(source_uri_raw);
@@ -79,33 +77,34 @@ static void test_file_path() {
 	belle_sip_object_unref(source_uri);
 	belle_sip_object_unref(first_uri);
 
-	CU_ASSERT_PTR_NULL(belle_generic_uri_get_scheme(uri));
-	CU_ASSERT_PTR_NULL(belle_generic_uri_get_host(uri));
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/index.html");
+	BC_ASSERT_PTR_NULL(belle_generic_uri_get_scheme(uri));
+	BC_ASSERT_PTR_NULL(belle_generic_uri_get_host(uri));
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/index.html");
 
 	belle_sip_object_unref(uri);
 
 	source_uri = belle_generic_uri_parse("file:///tmp/absolute-file");
-	CU_ASSERT_PTR_NOT_NULL(source_uri);
+	BC_ASSERT_PTR_NOT_NULL(source_uri);
 	if (source_uri!=NULL){
-		CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(source_uri),"file");
-		CU_ASSERT_PTR_NULL(belle_generic_uri_get_host(source_uri));
-		CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/tmp/absolute-file");
+		BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(source_uri),"file");
+		BC_ASSERT_PTR_NULL(belle_generic_uri_get_host(source_uri));
+		BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(source_uri),"/tmp/absolute-file");
 		belle_sip_object_unref(source_uri);
 	}
 
 	/*this is INVALID*/
 	source_uri = belle_generic_uri_parse("file://./relative-file");
-	CU_ASSERT_PTR_NOT_NULL(source_uri);
+	BC_ASSERT_PTR_NOT_NULL(source_uri);
+	if (source_uri) belle_sip_object_unref(source_uri);
 
 	/* PATH segment always start by / */
 	source_uri = belle_generic_uri_parse("./relative-file");
-	CU_ASSERT_PTR_NULL(source_uri);
+	BC_ASSERT_PTR_NULL(source_uri);
 	if (source_uri!=NULL){
 		belle_sip_object_unref(source_uri);
 	}
 }
-static void test_absolute_uri() {
+static void test_absolute_uri(void) {
 
 	belle_generic_uri_t* source_uri = belle_generic_uri_parse("tel:+33123457");
 	char* source_uri_raw = belle_generic_uri_to_string(source_uri);
@@ -115,8 +114,8 @@ static void test_absolute_uri() {
 	belle_sip_object_unref(source_uri);
 	belle_sip_object_unref(first_uri);
 
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"tel");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(uri),"+33123457");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"tel");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(uri),"+33123457");
 	belle_sip_object_unref(uri);
 
 	source_uri = belle_generic_uri_parse("tel:11234567888;phone-context=vzims.com");
@@ -127,8 +126,8 @@ static void test_absolute_uri() {
 	belle_sip_object_unref(source_uri);
 	belle_sip_object_unref(first_uri);
 
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"tel");
-	CU_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(uri),"11234567888;phone-context=vzims.com");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(uri),"tel");
+	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(uri),"11234567888;phone-context=vzims.com");
 	belle_sip_object_unref(uri);
 
 }
@@ -140,10 +139,5 @@ static test_t tests[] = {
 	{ "Absolute uri", test_absolute_uri }
 };
 
-test_suite_t generic_uri_test_suite = {
-	"Generic uri",
-	NULL,
-	NULL,
-	sizeof(tests) / sizeof(tests[0]),
-	tests
-};
+test_suite_t generic_uri_test_suite = {"Generic uri", NULL, NULL, belle_sip_tester_before_each,
+									   belle_sip_tester_after_each, sizeof(tests) / sizeof(tests[0]), tests};
