@@ -80,9 +80,11 @@ static int http_before_all(void) {
 	stack=belle_sip_stack_new(NULL);
 	prov=belle_sip_stack_create_http_provider(stack,"0.0.0.0");
 	if (belle_sip_tester_get_root_ca_path() != NULL) {
-		belle_tls_verify_policy_t *policy=belle_tls_verify_policy_new();
-		belle_tls_verify_policy_set_root_ca(policy,belle_sip_tester_get_root_ca_path());
-		belle_http_provider_set_tls_verify_policy(prov,policy);
+		belle_tls_crypto_config_t *crypto_config=belle_tls_crypto_config_new();
+		belle_tls_crypto_config_set_root_ca(crypto_config,belle_sip_tester_get_root_ca_path());
+		belle_http_provider_set_tls_crypto_config(prov,crypto_config);
+		belle_sip_object_unref(crypto_config);
+
 	}
 	return 0;
 }
@@ -130,8 +132,8 @@ static int one_get(const char *url,http_counters_t* counters, int *counter){
 static void one_http_get(void){
 	http_counters_t counters={0};
 	if (one_get("http://smtp.linphone.org",&counters,&counters.response_count) == 0) {
-		BC_ASSERT_TRUE(counters.response_count==1);
-		BC_ASSERT_TRUE(counters.io_error_count==0);
+		BC_ASSERT_EQUAL(counters.response_count,1,int,"%d");
+		BC_ASSERT_EQUAL(counters.io_error_count,0,int,"%d");
 		BC_ASSERT_EQUAL(counters.two_hundred,1,int,"%d");
 	}
 }
@@ -139,8 +141,8 @@ static void one_http_get(void){
 static void http_get_empty_body(void){
 	http_counters_t counters={0};
 	if (one_get("http://smtp.linphone.org/marie_invalid",&counters,&counters.response_count) == 0) {
-		BC_ASSERT_TRUE(counters.response_count==1);
-		BC_ASSERT_TRUE(counters.io_error_count==0);
+		BC_ASSERT_EQUAL(counters.response_count,1,int,"%d");
+		BC_ASSERT_EQUAL(counters.io_error_count,0,int,"%d");
 		BC_ASSERT_EQUAL(counters.two_hundred,1,int,"%d");
 	}
 }
@@ -200,7 +202,6 @@ static void on_progress(belle_sip_body_handler_t *bh, belle_sip_message_t *msg,
 	}else belle_sip_message("%i bytes transfered",(int)offset);
 }
 
-#define MULTIPART_BOUNDARY "---------------------------14737809831466499882746641449"
 #define MULTIPART_BEGIN "somehash.jpg\r\n" \
 			"--" MULTIPART_BOUNDARY "\r\n" \
 			"Content-Disposition: form-data; name=\"userfile\"; filename=\"belle_http_sip_tester.jpg\"\r\n" \
@@ -214,7 +215,7 @@ static int on_send_body(belle_sip_user_body_handler_t *bh, belle_sip_message_t *
 	size_t end_of_img=sizeof(MULTIPART_BEGIN)+image_size;
 	if (offset==0){
 		size_t partlen=sizeof(MULTIPART_BEGIN);
-		BC_ASSERT_TRUE_FATAL(partlen<*size);
+		BC_ASSERT_LOWER_STRICT((unsigned int)partlen,(unsigned int)*size,unsigned int,"%u");
 		memcpy(buffer,MULTIPART_BEGIN,partlen);
 		*size=partlen;
 	}else if (offset<end_of_img){
@@ -312,12 +313,12 @@ static void http_get_long_user_body(void){
 	belle_sip_object_data_set(BELLE_SIP_OBJECT(req),"file",outfile,NULL);
 	belle_http_provider_send_request(prov,req,l);
 	BC_ASSERT_TRUE(wait_for(stack,&counters.two_hundred,1,20000));
-	BC_ASSERT_TRUE(counters.response_headers_count==1);
+	BC_ASSERT_EQUAL(counters.response_headers_count,1,int,"%d");
 	resp=belle_http_request_get_response(req);
 	BC_ASSERT_PTR_NOT_NULL(resp);
 	if (resp){
 		bh=belle_sip_message_get_body_handler((belle_sip_message_t*)resp);
-		BC_ASSERT_TRUE(belle_sip_body_handler_get_size(bh)>0);
+		BC_ASSERT_GREATER_STRICT((unsigned int)belle_sip_body_handler_get_size(bh),0,unsigned int,"%u");
 	}
 	belle_sip_object_unref(req);
 	belle_sip_object_unref(l);
@@ -331,7 +332,7 @@ static void one_https_get_with_proxy(void){
 	http_counters_t counters={0};
 	belle_sip_stack_set_http_proxy_host(stack, test_http_proxy_addr);
 	belle_sip_stack_set_http_proxy_port(stack, test_http_proxy_port);
-	
+
 	if (one_get("https://smtp.linphone.org",&counters,&counters.response_count) == 0) {
 		BC_ASSERT_EQUAL(counters.response_count, 1, int, "%d");
 		BC_ASSERT_EQUAL(counters.io_error_count, 0, int, "%d");
@@ -356,5 +357,5 @@ test_t http_tests[] = {
 	{ "http GET with long user body", http_get_long_user_body}
 };
 
-test_suite_t http_test_suite = {"http", http_before_all, http_after_all, NULL,
+test_suite_t http_test_suite = {"HTTP stack", http_before_all, http_after_all, NULL,
 								NULL, sizeof(http_tests) / sizeof(http_tests[0]), http_tests};
