@@ -77,12 +77,12 @@ static void test_complex_contact_header(void) {
 	L_contact = belle_sip_header_contact_parse("Contact: super toto <sip:titi.com>;expires=3600; q=0.7");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_contact));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_contact));
-	
+
 	L_contact = belle_sip_header_contact_parse(l_raw_header);
 	belle_sip_free(l_raw_header);
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_address_get_displayname((belle_sip_header_address_t*)L_contact), "super toto");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_contact));
-	
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_contact_parse("m:sip:titi.com, nimportequoi"));
 }
 
@@ -123,16 +123,16 @@ static void test_from_header(void) {
 
 	/*test factory*/
 	L_from = belle_sip_header_from_create2("super <sip:titi.com:5060;lr;ttl=1;method=INVITE;maddr=192.168.0.1;transport=tcp?header=123>","12345-abc");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_from);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_from)) return;
 	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_uri);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_uri)) return;
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"lr"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"ttl"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"method"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"maddr"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"transport"));
 	BC_ASSERT_FALSE(belle_sip_uri_get_port(L_uri)>0);
-	BC_ASSERT_EQUAL(belle_sip_list_size(belle_sip_uri_get_header_names(L_uri)),0,int,"%d");
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_list_size(belle_sip_uri_get_header_names(L_uri)),0,unsigned int,"%u");
 
 	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "titi.com");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"12345-abc");
@@ -144,12 +144,12 @@ static void test_from_header(void) {
 static void test_from_header_with_paramless_address_spec(void) {
 	belle_generic_uri_t *L_absoluteUri;
 	belle_sip_header_from_t* L_from = belle_sip_header_from_parse("From: sip:bob@titi.com;tag=dlfjklcn6545614XX");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_from_get_tag(L_from));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_header_from_get_tag(L_from))) return;
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
 	belle_sip_object_unref(L_from);
 
 	L_from = belle_sip_header_from_parse("From: tel:1234567;tag=dlfjklcn6545614XX");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_from_get_tag(L_from));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_header_from_get_tag(L_from))) return;
 	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
 	BC_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from)));
 	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "1234567");
@@ -163,12 +163,12 @@ static void test_to_header_with_paramless_address_spec(void) {
 	belle_generic_uri_t *L_absoluteUri;
 	belle_sip_header_to_t* L_to = belle_sip_header_to_parse("To: sip:bob@titi.com;tag=dlfjklcn6545614XX");
 	belle_sip_uri_t *L_uri;
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_header_to_get_tag(L_to))) return;
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"dlfjklcn6545614XX");
 	belle_sip_object_unref(L_to);
 
 	L_to = belle_sip_header_to_parse("To:sip:1002@192.168.1.199;tag=as1f0a0817");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_header_to_get_tag(L_to))) return;
 	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
 	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_user(L_uri),"1002");
 	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "192.168.1.199");
@@ -177,7 +177,7 @@ static void test_to_header_with_paramless_address_spec(void) {
 	belle_sip_object_unref(L_to);
 
 	L_to = belle_sip_header_to_parse("To: tel:1234567;tag=dlfjklcn6545614XX");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_header_to_get_tag(L_to));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_header_to_get_tag(L_to))) return;
 	L_absoluteUri = belle_sip_header_address_get_absolute_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
 	BC_ASSERT_PTR_NULL(belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to)));
 	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_opaque_part(L_absoluteUri), "1234567");
@@ -210,7 +210,7 @@ static void test_to_header(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_to));
 	/*test factory*/
 	L_to = belle_sip_header_to_create2("\"super man\" <sip:titi.com:5060;lr;ttl=1;method=INVITE;maddr=192.168.0.1;transport=tcp?header=123>","12345-abc");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_to);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_to)) return;
 	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_to));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"lr"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"ttl"));
@@ -218,9 +218,9 @@ static void test_to_header(void) {
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"maddr"));
 	BC_ASSERT_FALSE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_uri),"transport"));
 	BC_ASSERT_FALSE(belle_sip_uri_get_port(L_uri)>0);
-	BC_ASSERT_EQUAL(belle_sip_list_size(belle_sip_uri_get_header_names(L_uri)),0,int,"%d");
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_list_size(belle_sip_uri_get_header_names(L_uri)),0,unsigned int,"%u");
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_uri);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_uri)) return;
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(L_to)), "super man");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_to_get_tag(L_to),"12345-abc");
 	belle_sip_object_unref(L_to);
@@ -346,6 +346,23 @@ static void test_content_type_header(void) {
 	L_content_type = belle_sip_header_content_type_parse(l_raw_header);
 	belle_sip_free(l_raw_header);
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_content_type));
+
+	L_content_type = belle_sip_header_content_type_parse("Content-Type: multipart/related; type=application/rlmi+xml");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_content_type));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_content_type));
+	L_tmp = belle_sip_header_content_type_parse(l_raw_header);
+	L_content_type = BELLE_SIP_HEADER_CONTENT_TYPE(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_content_type_get_type(L_content_type),"multipart");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_content_type_get_subtype(L_content_type),"related");
+	BC_ASSERT_STRING_EQUAL(belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_content_type),"type"),"application/rlmi+xml");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_content_type));
+
+
+
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_content_type_parse("nimportequoi"));
 }
 
@@ -389,10 +406,12 @@ static void test_route_header(void) {
 	belle_sip_header_route_t* L_next_route;
 	belle_sip_header_address_t* address = belle_sip_header_address_parse("<sip:212.27.52.5:5060;transport=udp;lr>");
 	char* l_raw_header;
-	BC_ASSERT_PTR_NOT_NULL_FATAL(address);
+	belle_sip_header_record_route_t* L_record_route;
+
+	if (!BC_ASSERT_PTR_NOT_NULL(address)) return;
 	L_route = belle_sip_header_route_create(address);
 	belle_sip_object_unref(address);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_route);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_route)) return;
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_route));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_route));
 	L_route = belle_sip_header_route_parse(l_raw_header);
@@ -419,6 +438,16 @@ static void test_route_header(void) {
 	L_route=belle_sip_header_route_parse(l_raw_header);
 	BC_ASSERT_PTR_NOT_NULL(L_route);
 	if (L_route) belle_sip_object_unref(L_route);
+
+	L_record_route = belle_sip_header_record_route_parse("Record-Route: <sip:212.27.52.5:5060;transport=udp;lr>;charset=ISO-8859-4, <sip:212.27.52.5:5060;transport=udp;lr>");
+	L_route = belle_sip_header_route_create(BELLE_SIP_HEADER_ADDRESS(L_record_route));
+	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_route));
+	BC_ASSERT_PTR_NULL(belle_sip_header_get_next(BELLE_SIP_HEADER(L_route)));
+	BC_ASSERT_PTR_NULL(belle_sip_uri_get_user(L_uri));
+	BC_ASSERT_EQUAL(belle_sip_uri_get_port(L_uri), 5060,int,"%d");
+	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "212.27.52.5");
+	belle_sip_object_unref(L_record_route);
+	belle_sip_object_unref(L_route);
 	belle_sip_free(l_raw_header);
 
 }
@@ -427,7 +456,7 @@ static void test_service_route_header(void) {
 	belle_sip_header_service_route_t* L_service_route = belle_sip_header_service_route_parse("Service-Route: <sip:orig@scscf.ims.linphone.com:6060;lr>");
 	belle_sip_uri_t* L_uri;
 	char* l_raw_header;
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_service_route);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_service_route)) return;
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_service_route));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_service_route));
 	L_service_route = belle_sip_header_service_route_parse(l_raw_header);
@@ -448,7 +477,7 @@ static void test_content_length_header(void) {
 	L_content_length = BELLE_SIP_HEADER_CONTENT_LENGTH(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
-	BC_ASSERT_EQUAL(belle_sip_header_content_length_get_content_length(L_content_length), 3495,int,"%d");
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_header_content_length_get_content_length(L_content_length), 3495, unsigned int, "%u");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_content_length));
 	BC_ASSERT_PTR_NULL(belle_sip_header_content_length_parse("nimportequoi"));
 }
@@ -496,7 +525,7 @@ static void test_authorization_header(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_authorization);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_authorization)) return;
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_scheme(L_authorization), "Digest");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_username(L_authorization), "0033482532176");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authorization_get_realm(L_authorization), "sip.ovh.net");
@@ -540,9 +569,9 @@ static void check_header_authenticate(belle_sip_header_www_authenticate_t* authe
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_www_authenticate_get_algorithm(authenticate), "MD5");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_www_authenticate_get_opaque(authenticate), "1bc7f9097684320");
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(qop=belle_sip_header_www_authenticate_get_qop(authenticate));
+	if (!BC_ASSERT_PTR_NOT_NULL(qop=belle_sip_header_www_authenticate_get_qop(authenticate))) return;
 	BC_ASSERT_STRING_EQUAL((const char*)qop->data, "auth");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(qop=qop->next);
+	if (!BC_ASSERT_PTR_NOT_NULL(qop=qop->next)) return;
 	BC_ASSERT_STRING_EQUAL((const char*)qop->data, "auth-int");
 
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_www_authenticate_get_qop(authenticate)->data, "auth");
@@ -775,14 +804,13 @@ static void test_very_long_address_header(void) {
 					";>"; /*not compliant but*/
 
 	belle_sip_header_address_t* laddress = belle_sip_header_address_parse(raw);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(laddress);
+	if (!BC_ASSERT_PTR_NOT_NULL(laddress)) return;
 	L_uri = belle_sip_header_address_get_uri(laddress);
 
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_uri_get_user(L_uri));
 	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "sip.linphone.org");
 	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_user(L_uri), "jehan");
-	printf("toto='%s'\n", belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_uri),"pn-tok"));
-		do {
+	do {
 		char format[512];
 		const char * cactual = (belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_uri),"pn-tok"));
 		const char * cexpected = ("APA91bHPVa4PuKOMnr6ppWb3XYUL06QO-ND4eeiw7dG49q4o_Ywzal7BxVRgH-wvqH9iB9V7h6kfb-DCiVdSpnl6CeWO25FAkM4eh6DJyWcbP7SzhKdku_-r9936kJW7-4drI6-Om4qp");
@@ -831,9 +859,9 @@ static void test_refer_to_header(void) {
 	/*test factory*/
 	L_refer_to = belle_sip_header_refer_to_create((ha=belle_sip_header_address_parse("\"super man\" <sip:titi.com>")));
 	belle_sip_object_unref(ha);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_refer_to);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_refer_to)) return;
 	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_refer_to));
-	BC_ASSERT_PTR_NOT_NULL_FATAL(L_uri);
+	if (!BC_ASSERT_PTR_NOT_NULL(L_uri)) return;
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_address_get_displayname(BELLE_SIP_HEADER_ADDRESS(L_refer_to)), "super man");
 	belle_sip_object_unref(L_refer_to);
 	BC_ASSERT_PTR_NULL(belle_sip_header_refer_to_parse("nimportequoi"));
@@ -913,7 +941,7 @@ static void _test_date_header(void){
 	date2=belle_sip_header_date_create_from_time(&utc);
 	BC_ASSERT_PTR_NOT_NULL(date2);
 	if (date2){
-		BC_ASSERT_TRUE(strcmp(belle_sip_header_date_get_date(date2),DATE_EXAMPLE)==0);
+		BC_ASSERT_STRING_EQUAL(belle_sip_header_date_get_date(date2),DATE_EXAMPLE);
 		BC_ASSERT_PTR_NULL(belle_sip_header_date_parse("nimportequoi"));
 		belle_sip_object_unref(date2);
 	}
@@ -1020,11 +1048,11 @@ static void test_supported(const char* header_name, const char * header_value, c
 	L_tmp = belle_sip_header_supported_parse(l_raw_header);
 	L_supported = BELLE_SIP_HEADER_SUPPORTED(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
-	
+
 	belle_sip_free(l_raw_header);
-	
+
 	list = belle_sip_header_supported_get_supported(L_supported);
-	
+
 	for(i=0;i<number_values;i++){
 		BC_ASSERT_PTR_NOT_NULL(list);
 		if (list) {
@@ -1051,7 +1079,7 @@ static void test_content_disposition_header(void) {
 	L_content_disposition = BELLE_SIP_HEADER_CONTENT_DISPOSITION(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_content_disposition_get_content_disposition(L_content_disposition), "form-data");
 	BC_ASSERT_STRING_EQUAL(belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_content_disposition), "name"), "\"File\"");
 	BC_ASSERT_STRING_EQUAL(belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_content_disposition), "filename"), "\"toto.jpeg\"");
@@ -1064,18 +1092,18 @@ static void test_accept_header(void) {
 	belle_sip_header_accept_t* L_tmp;
 	belle_sip_header_accept_t* L_accept = BELLE_SIP_HEADER_ACCEPT(belle_sip_header_create("Accept", "text/html; charset=ISO-8859-4"));
 	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_accept));
-	
+
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
 	L_tmp = belle_sip_header_accept_parse(l_raw_header);
 	L_accept = BELLE_SIP_HEADER_ACCEPT(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_type(L_accept),"text");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_subtype(L_accept),"html");
 	BC_ASSERT_STRING_EQUAL(belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(L_accept),"charset"),"ISO-8859-4");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
-	
+
 	L_accept = belle_sip_header_accept_parse("Accept: application/sdp");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_accept));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
@@ -1084,14 +1112,14 @@ static void test_accept_header(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_type(L_accept),"application");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_subtype(L_accept),"sdp");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
-	
+
 	L_accept = belle_sip_header_accept_parse("Accept: application/pkcs7-mime; smime-type=enveloped-data; \r\n name=smime.p7m");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_accept));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
 	L_accept = belle_sip_header_accept_parse(l_raw_header);
 	belle_sip_free(l_raw_header);
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
-	
+
 	L_accept = belle_sip_header_accept_parse("Accept: application/sdp, text/plain, application/vnd.gsma.rcs-ft-http+xml");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_type(L_accept),"application");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_subtype(L_accept),"sdp");
@@ -1102,7 +1130,7 @@ static void test_accept_header(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_type(L_tmp),"application");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_accept_get_subtype(L_tmp),"vnd.gsma.rcs-ft-http+xml");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_accept));
-	
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_accept_parse("nimportequoi"));
 }
 
