@@ -215,7 +215,7 @@ static void testRegisterRaw(void) {
 	size_t size=0;
 	size_t raw_message_size= strlen(raw_message);
 	belle_sip_message_t* message = belle_sip_message_parse_raw(raw_message,raw_message_size,&size);
-	BC_ASSERT_EQUAL(raw_message_size,size+9,int,"%d");
+	BC_ASSERT_EQUAL((unsigned int)raw_message_size,(unsigned int)size+9,unsigned int,"%u");
 	request = BELLE_SIP_REQUEST(message);
 	BC_ASSERT_STRING_EQUAL(belle_sip_request_get_method(request),"REGISTER");
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_request_get_uri(request));
@@ -427,10 +427,9 @@ void channel_parser_tester_recovery_from_error_base (const char* prelude,const c
 
 	check_uri_and_headers(message);
 	belle_sip_object_unref(channel);
-	belle_sip_object_unref(message);
 	belle_sip_object_unref(stack);
-
 }
+
 void channel_parser_tester_recovery_from_error(void) {
 	const char * raw_message=	"debut de stream tout pourri\r\n"
 			"INVITE je_suis_une_fausse _request_uri_hihihi SIP/2.0\r\n"
@@ -838,7 +837,7 @@ static void testHttpGet(void)  {
 	belle_sip_header_extension_t* host_header;
 	belle_sip_object_t* tmp;
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(msg);
+	if (!BC_ASSERT_PTR_NOT_NULL(msg)) return;
 
 	marshaled_msg=belle_sip_object_to_string(BELLE_SIP_OBJECT(msg));
 	belle_sip_object_unref(msg);
@@ -850,7 +849,7 @@ static void testHttpGet(void)  {
 
 	BC_ASSERT_TRUE(BELLE_SIP_IS_INSTANCE_OF(msg,belle_http_request_t));
 	http_request=BELLE_HTTP_REQUEST(msg);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(uri=belle_http_request_get_uri(http_request));
+	if (!BC_ASSERT_PTR_NOT_NULL(uri=belle_http_request_get_uri(http_request))) return;
 
 	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_path(uri),"/index.php");
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(msg,"User-Agent"));
@@ -883,7 +882,7 @@ static void testHttp200Ok(void)  {
 	belle_sip_header_extension_t* host_header;
 	belle_sip_object_t* tmp;
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(msg);
+	if (!BC_ASSERT_PTR_NOT_NULL(msg)) return;
 
 	marshaled_msg=belle_sip_object_to_string(BELLE_SIP_OBJECT(msg));
 	belle_sip_object_unref(msg);
@@ -945,7 +944,6 @@ void channel_parser_http_response(void) {
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(message,"Cache-Control"));
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(message,"Vary"));
 
-	belle_sip_object_unref(BELLE_SIP_OBJECT(message));
 	belle_sip_object_unref(channel);
 	belle_sip_object_unref(stack);
 }
@@ -1005,9 +1003,8 @@ void testGetBody(void) {
 
 	ctlt = belle_sip_message_get_header_by_type(message,belle_sip_header_content_length_t);
 	BC_ASSERT_PTR_NOT_NULL(ctlt);
-	BC_ASSERT_EQUAL(belle_sip_header_content_length_get_content_length(ctlt),strlen(belle_sip_message_get_body(message)),int,"%d");
-	BC_ASSERT_EQUAL(belle_sip_header_content_length_get_content_length(ctlt),belle_sip_message_get_body_size(message),int,"%d");
-	belle_sip_object_unref(message);
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_header_content_length_get_content_length(ctlt),(unsigned int)strlen(belle_sip_message_get_body(message)),unsigned int,"%u");
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_header_content_length_get_content_length(ctlt),(unsigned int)belle_sip_message_get_body_size(message),unsigned int,"%u");
 	belle_sip_object_unref(channel);
 	belle_sip_object_unref(stack);
 }
@@ -1032,8 +1029,8 @@ test_t message_tests[] = {
 	{ "Channel parser malformed start", channel_parser_malformed_start},
 	{ "Channel parser truncated start", channel_parser_truncated_start},
 	{ "Channel parser truncated start with garbage",channel_parser_truncated_start_with_garbage},
-	{ "RFC2543 compatibility", testRFC2543Compat},
-	{ "RFC2543 compatibility with branch id",testRFC2543CompatWithBranch},
+	TEST_ONE_TAG("RFC2543 compatibility", testRFC2543Compat, "LeaksMemory"),
+	TEST_ONE_TAG("RFC2543 compatibility with branch id",testRFC2543CompatWithBranch, "LeaksMemory"),
 	{ "Uri headers in sip INVITE",testUriHeadersInInvite},
 	{ "Uris components in request",testUrisComponentsForRequest},
 	{ "Generic message test",testGenericMessage},
