@@ -50,9 +50,11 @@ typedef enum auth_mode {
 typedef struct _status {
 	int twoHundredOk;
 	int fourHundredOne;
+	int fourHundredEightyOne;
 	int refreshOk;
 	int refreshKo;
 	int dialogTerminated;
+	int fiveHundredTree;
 }status_t;
 
 typedef struct endpoint {
@@ -74,12 +76,13 @@ typedef struct endpoint {
 	int transiant_network_failure;
 	belle_sip_refresher_t* refresher;
 	int early_refresher;
+	int number_of_body_found;
 } endpoint_t;
 
 
 static unsigned int  wait_for(belle_sip_stack_t*s1, belle_sip_stack_t*s2,int* counter,int value,int timeout) {
 	int retry=0;
-#define ITER 100
+#define ITER 20
 	while (*counter<value && retry++ <(timeout/ITER)) {
 		if (s1) belle_sip_stack_sleep(s1,ITER/2);
 		if (s2) belle_sip_stack_sleep(s2,ITER/2);
@@ -198,8 +201,8 @@ static void server_process_request_event(void *obj, const belle_sip_request_even
 				belle_sip_header_www_authenticate_add_qop(www_authenticate,"auth");
 				if (endpoint->nonce_count>=MAX_NC_COUNT) {
 					belle_sip_header_www_authenticate_set_stale(www_authenticate,1);
-					endpoint->nonce_count=1;
 				}
+				endpoint->nonce_count=1;
 			}
 		}
 	}
@@ -252,6 +255,9 @@ static void server_process_request_event(void *obj, const belle_sip_request_even
 		via=belle_sip_message_get_header_by_type(req,belle_sip_header_via_t);
 		belle_sip_header_via_set_received(via,endpoint->received);
 	}
+	if (belle_sip_message_get_body(BELLE_SIP_MESSAGE(req))) {
+		endpoint->number_of_body_found++;
+	}
 	belle_sip_server_transaction_send_response(server_transaction,resp);
 }
 
@@ -278,11 +284,12 @@ static void client_process_response_event(void *obj, const belle_sip_response_ev
 			int family_found;
 			belle_sip_header_contact_t *ct=belle_sip_message_get_header_by_type(
 				(belle_sip_message_t*)belle_sip_response_event_get_response(event),belle_sip_header_contact_t);
-			BC_ASSERT_PTR_NOT_NULL_FATAL(ct);
-			host=belle_sip_uri_get_host(belle_sip_header_address_get_uri((belle_sip_header_address_t*)ct));
-			if (strchr(host,':')) family_found=AF_INET6;
-			else family_found=AF_INET;
-			BC_ASSERT_TRUE(family_found==endpoint->connection_family);
+			if (BC_ASSERT_PTR_NOT_NULL(ct)) {
+				host=belle_sip_uri_get_host(belle_sip_header_address_get_uri((belle_sip_header_address_t*)ct));
+				if (strchr(host,':')) family_found=AF_INET6;
+				else family_found=AF_INET;
+				BC_ASSERT_EQUAL(family_found,endpoint->connection_family,int,"%d");
+			}
 		}
 		break;
 	case 401:endpoint->stat.fourHundredOne++; break;
@@ -314,10 +321,14 @@ static void belle_sip_refresher_listener (belle_sip_refresher_t* refresher
 	endpoint_t* endpoint = (endpoint_t*)user_pointer;
 	BELLESIP_UNUSED(refresher);
 	belle_sip_message("belle_sip_refresher_listener [%i] reason [%s]",status_code,reason_phrase);
+	if (status_code >=300)
+		endpoint->stat.refreshKo++;
 	switch (status_code) {
 		case 200:endpoint->stat.refreshOk++; break;
+		case 481:endpoint->stat.fourHundredEightyOne++;break;
+		case 503:endpoint->stat.fiveHundredTree++;break;
 		default:
-			endpoint->stat.refreshKo++;
+			/*nop*/
 			break;
 	}
 	if (endpoint->stat.refreshKo==1 && endpoint->transiant_network_failure) {
@@ -356,7 +367,7 @@ static void destroy_endpoint(endpoint_t* endpoint) {
 
 static endpoint_t* create_udp_endpoint(int port,belle_sip_listener_callbacks_t* listener_callbacks) {
 	endpoint_t *endpoint=create_endpoint("0.0.0.0",port,"udp",listener_callbacks);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(endpoint->lp);
+	BC_ASSERT_PTR_NOT_NULL(endpoint->lp);
 	return endpoint;
 }
 
@@ -423,26 +434,27 @@ static void refresher_base_with_body(endpoint_t* client
 			trans=belle_sip_provider_create_client_transaction(client->provider,req);
 			belle_sip_object_ref(trans);
 			belle_sip_client_transaction_send_request(trans);
-			BC_ASSERT_TRUE_FATAL(wait_for(server->stack,client->stack,&client->stat.twoHundredOk,1,1000));
+			BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.twoHundredOk,1,1000));
 		}
 		client->refresher= refresher = belle_sip_client_transaction_create_refresher(trans);
 	}
-	BC_ASSERT_TRUE_FATAL(refresher!=NULL);
-	belle_sip_object_unref(trans);
-	belle_sip_refresher_set_listener(refresher,belle_sip_refresher_listener,client);
+	if (BC_ASSERT_PTR_NOT_NULL(refresher)) {
+		belle_sip_object_unref(trans);
+		belle_sip_refresher_set_listener(refresher,belle_sip_refresher_listener,client);
 
-	begin = belle_sip_time_ms();
-	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+(client->early_refresher?1:0),client->register_count*1000 + 1000));
-	end = belle_sip_time_ms();
-	BC_ASSERT_TRUE(end-begin>=client->register_count*1000*.9); /*because refresh is at 90% of expire*/
-	BC_ASSERT_TRUE(end-begin<(client->register_count*1000 + 2000));
-	/*unregister twice to make sure refresh operation can be safely cascaded*/
-	belle_sip_refresher_refresh(refresher,0);
-	belle_sip_refresher_refresh(refresher,0);
-	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+1,1000));
-	BC_ASSERT_EQUAL(client->stat.refreshOk,client->register_count+1,int,"%d");
-	belle_sip_refresher_stop(refresher);
-	belle_sip_object_unref(refresher);
+		begin = belle_sip_time_ms();
+		BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+(client->early_refresher?1:0),client->register_count*1000 + 1000));
+		end = belle_sip_time_ms();
+		BC_ASSERT_GREATER((long double)(end-begin),client->register_count*1000*.9,long double,"%Lf"); /*because refresh is at 90% of expire*/
+		BC_ASSERT_LOWER_STRICT(end-begin,(client->register_count*1000 + 2000),unsigned long long,"%llu");
+		/*unregister twice to make sure refresh operation can be safely cascaded*/
+		belle_sip_refresher_refresh(refresher,0);
+		belle_sip_refresher_refresh(refresher,0);
+		BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,client->register_count+1,1000));
+		BC_ASSERT_EQUAL(client->stat.refreshOk,client->register_count+1,int,"%d");
+		belle_sip_refresher_stop(refresher);
+		belle_sip_object_unref(refresher);
+	}
 }
 static void refresher_base(endpoint_t* client,endpoint_t *server, const char* method) {
 	refresher_base_with_body(client,server,method,NULL,NULL);
@@ -480,8 +492,17 @@ static void refresher_base_with_param(const char* method, unsigned char expire_i
 static void register_test_with_param(unsigned char expire_in_contact,auth_mode_t auth_mode) {
 	refresher_base_with_param("REGISTER",expire_in_contact,auth_mode);
 }
+static char *list =	"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+						"<resource-lists xmlns=\"urn:ietf:params:xml:ns:resource-lists\"\n"
+						"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n"
+						"<list>\n"
+							"\t<entry uri=\"sip:marie@sexample.com\" />\n"
+							"\t<entry uri=\"sip:pauline@sexample.com\" />\n"
+						"</list>\n"
+					"</resource-lists>\n";
 
-static void subscribe_test(void) {
+
+static void subscribe_base(int with_resource_lists) {
 	belle_sip_listener_callbacks_t client_callbacks;
 	belle_sip_listener_callbacks_t server_callbacks;
 	belle_sip_request_t* req;
@@ -493,6 +514,8 @@ static void subscribe_test(void) {
 	belle_sip_uri_t *dest_uri;
 	belle_sip_refresher_t* refresher;
 	belle_sip_header_contact_t* contact=belle_sip_header_contact_new();
+	belle_sip_dialog_t * client_dialog;
+	int dummy = 0;
 	uint64_t begin;
 	uint64_t end;
 	memset(&client_callbacks,0,sizeof(belle_sip_listener_callbacks_t));
@@ -503,7 +526,7 @@ static void subscribe_test(void) {
 	client_callbacks.process_auth_requested=client_process_auth_requested;
 	server_callbacks.process_request_event=server_process_request_event;
 	server_callbacks.process_dialog_terminated=server_process_dialog_terminated;
-	
+
 	client = create_udp_endpoint(3452,&client_callbacks);
 	server = create_udp_endpoint(6788,&server_callbacks);
 	server->expire_in_contact=0;
@@ -528,6 +551,11 @@ static void subscribe_test(void) {
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_create("Event","Presence")));
 
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(destination_route));
+	if (with_resource_lists) {
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_content_type_create("application","resource-lists+xml")));
+		belle_sip_message_set_body(BELLE_SIP_MESSAGE(req), list, strlen(list));
+	}
+
 	trans=belle_sip_provider_create_client_transaction(client->provider,req);
 	belle_sip_object_ref(trans);/*to avoid trans from being deleted before refresher can use it*/
 	belle_sip_client_transaction_send_request(trans);
@@ -539,9 +567,9 @@ static void subscribe_test(void) {
 	trans=belle_sip_provider_create_client_transaction(client->provider,req);
 	belle_sip_object_ref(trans);
 	belle_sip_client_transaction_send_request(trans);
-	BC_ASSERT_TRUE_FATAL(wait_for(server->stack,client->stack,&client->stat.twoHundredOk,1,1000));
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.twoHundredOk,1,1000));
 	 /*maybe dialog should be automatically created*/
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(trans)));
+	BC_ASSERT_PTR_NOT_NULL(client_dialog = belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(trans)));
 
 	refresher = belle_sip_client_transaction_create_refresher(trans);
 	belle_sip_object_unref(trans);
@@ -550,20 +578,63 @@ static void subscribe_test(void) {
 	begin = belle_sip_time_ms();
 	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,3,4000));
 	end = belle_sip_time_ms();
-	BC_ASSERT_TRUE(end-begin>=3000);
-	BC_ASSERT_TRUE(end-begin<5000);
-	/*unsubscribe twice to make sure refresh operation can be safely cascaded*/
-	belle_sip_refresher_refresh(refresher,0);
-	belle_sip_refresher_refresh(refresher,0);
+	BC_ASSERT_GREATER((long double)(end-begin),3000*.9,long double,"%Lf");
+	BC_ASSERT_LOWER_STRICT(end-begin,5000,unsigned long long,"%llu");
+
+	belle_sip_message("simulating dialog error and recovery");
+	belle_sip_stack_set_send_error(client->stack, 1500);
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.fourHundredEightyOne,1,4000));
+
+	belle_sip_stack_set_send_error(client->stack, 0);
+	wait_for(server->stack,client->stack, &dummy, 1, 1000);
+
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,4,4000));
+	BC_ASSERT_EQUAL(client->stat.dialogTerminated, 0, int, "%i");
+
+	/*make sure dialog has changed*/
+	BC_ASSERT_PTR_NOT_EQUAL(client_dialog, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))));
+
+	
+	belle_sip_message("simulating dialog terminated server side and recovery");
+	
+	client_dialog = belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher)));
+	
+	belle_sip_provider_enable_unconditional_answer(server->provider,TRUE);
+	belle_sip_provider_set_unconditional_answer(server->provider,481);
+	belle_sip_refresher_refresh(refresher, 10);
+	
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.fourHundredEightyOne,2,4000));
+	belle_sip_provider_enable_unconditional_answer(server->provider,FALSE);
+
+	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.refreshOk,5,4000));
+	BC_ASSERT_EQUAL(client->stat.dialogTerminated, 0, int, "%i");
+	
+	/*make sure dialog has changed*/
+	BC_ASSERT_PTR_NOT_EQUAL(client_dialog, belle_sip_transaction_get_dialog(BELLE_SIP_TRANSACTION(belle_sip_refresher_get_transaction(refresher))));
+
+	belle_sip_refresher_refresh(refresher, 0);
+	belle_sip_refresher_refresh(refresher, 0);
 
 	belle_sip_refresher_stop(refresher);
-	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&client->stat.dialogTerminated,1,4000));
 	BC_ASSERT_TRUE(wait_for(server->stack,client->stack,&server->stat.dialogTerminated,1,4000));
 	belle_sip_object_unref(refresher);
+
+	if (with_resource_lists) {
+		BC_ASSERT_EQUAL(server->number_of_body_found, (server->auth == none ?1:2), int, "%i");
+	}
+
+	
 	destroy_endpoint(client);
 	destroy_endpoint(server);
 }
 
+static void subscribe_test(void) {
+	subscribe_base(FALSE);
+}
+static void subscribe_list_test(void) {
+	subscribe_base(TRUE);
+}
+
 static void register_expires_header(void) {
 	register_test_with_param(0,none);
 }
@@ -796,7 +867,8 @@ test_t refresher_tests[] = {
 	{ "REGISTER Expires in Contact digest auth", register_expires_in_contact_header_digest_auth },
 	{ "REGISTER with failure", register_with_failure },
 	{ "REGISTER with early refresher",register_early_refresher},
-	{ "SUBSCRIBE", subscribe_test },
+	TEST_ONE_TAG("SUBSCRIBE", subscribe_test, "LeaksMemory"),
+	TEST_ONE_TAG("SUBSCRIBE of list" , subscribe_list_test, "LeaksMemory"),
 	{ "PUBLISH", simple_publish },
 	{ "PUBLISH with early refresher", simple_publish_with_early_refresher },
 	{ "REGISTER with unrecognizable Contact", register_with_unrecognizable_contact },
