@@ -71,7 +71,9 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 	int status;
 	belle_sip_request_t* request;
 	BELLESIP_UNUSED(user_ctx);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_response_event_get_response(event));
+	if (!BC_ASSERT_PTR_NOT_NULL(belle_sip_response_event_get_response(event))) {
+		return;
+	}
 	belle_sip_message("process_response_event [%i] [%s]"
 					,status=belle_sip_response_get_status_code(belle_sip_response_event_get_response(event))
 					,belle_sip_response_get_reason_phrase(belle_sip_response_event_get_response(event)));
@@ -81,15 +83,15 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 		belle_sip_header_cseq_t* cseq;
 		belle_sip_client_transaction_t *t;
 		belle_sip_uri_t *dest;
-		// BC_ASSERT_NOT_EQUAL_FATAL(number_of_challenge,2);
-		BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_response_event_get_client_transaction(event)); /*require transaction mode*/
+		// BC_ASSERT_NOT_EQUAL(number_of_challenge,2);
+		BC_ASSERT_PTR_NOT_NULL(belle_sip_response_event_get_client_transaction(event)); /*require transaction mode*/
 		dest=belle_sip_client_transaction_get_route(belle_sip_response_event_get_client_transaction(event));
 		request=belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(belle_sip_response_event_get_client_transaction(event)));
 		cseq=(belle_sip_header_cseq_t*)belle_sip_message_get_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_CSEQ);
 		belle_sip_header_cseq_set_seq_number(cseq,belle_sip_header_cseq_get_seq_number(cseq)+1);
 		belle_sip_message_remove_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_AUTHORIZATION);
 		belle_sip_message_remove_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_PROXY_AUTHORIZATION);
-		BC_ASSERT_TRUE_FATAL(belle_sip_provider_add_authorization(prov,request,belle_sip_response_event_get_response(event),NULL,NULL,auth_domain));
+		BC_ASSERT_TRUE(belle_sip_provider_add_authorization(prov,request,belle_sip_response_event_get_response(event),NULL,NULL,auth_domain));
 
 		t=belle_sip_provider_create_client_transaction(prov,request);
 		belle_sip_client_transaction_send_request_to(t,dest);
@@ -198,12 +200,15 @@ int register_before_all(void) {
 	belle_sip_provider_add_listening_point(prov,lp);
 	lp=belle_sip_stack_create_listening_point(stack,"0.0.0.0",7061,"TLS");
 	if (lp) {
+		belle_tls_crypto_config_t *crypto_config=belle_tls_crypto_config_new();
 		/* since test.linphone.org does not have proper certificates, don't verify anything*/
-		belle_sip_tls_listening_point_set_verify_exceptions(BELLE_SIP_TLS_LISTENING_POINT(lp),BELLE_SIP_TLS_LISTENING_POINT_BADCERT_ANY_REASON);
+		belle_tls_crypto_config_set_verify_exceptions(crypto_config, BELLE_TLS_VERIFY_ANY_REASON);
 		if (belle_sip_tester_get_root_ca_path() != NULL) {
-			belle_sip_tls_listening_point_set_root_ca(BELLE_SIP_TLS_LISTENING_POINT(lp), belle_sip_tester_get_root_ca_path());
+			belle_tls_crypto_config_set_root_ca(crypto_config,belle_sip_tester_get_root_ca_path());
 		}
+		belle_sip_tls_listening_point_set_crypto_config(BELLE_SIP_TLS_LISTENING_POINT(lp), crypto_config);
 		belle_sip_provider_add_listening_point(prov,lp);
+		belle_sip_object_unref(crypto_config);
 	}
 
 	listener_callbacks.process_dialog_terminated=process_dialog_terminated;
@@ -433,7 +438,7 @@ static void stateful_register_tls_with_http_proxy(void) {
 }
 
 static void stateful_register_tls_with_wrong_http_proxy(void){
-	
+
 	belle_sip_tls_listening_point_t * lp = (belle_sip_tls_listening_point_t*)belle_sip_provider_get_listening_point(prov, "tls");
 	if (!lp) {
 		belle_sip_error("No TLS support, test skipped.");
@@ -445,7 +450,7 @@ static void stateful_register_tls_with_wrong_http_proxy(void){
 	try_register_user_at_domain(stack,prov,"tls",1,"tester",test_domain,NULL,0);
 	belle_sip_stack_set_http_proxy_host(stack, NULL);
 	belle_sip_stack_set_http_proxy_port(stack, 0);
-	
+
 }
 
 
@@ -459,7 +464,8 @@ static void bad_req_process_response_event(void *user_ctx, const belle_sip_respo
 	belle_sip_response_t *resp=belle_sip_response_event_get_response(event);
 	int *bad_request_response_received=(int*)user_ctx;
 	if (belle_sip_response_event_get_client_transaction(event) != NULL) {
-		BC_ASSERT_TRUE(resp && belle_sip_response_get_status_code(resp)==400);
+		BC_ASSERT_PTR_NOT_NULL(resp);
+		BC_ASSERT_EQUAL(belle_sip_response_get_status_code(resp),400,int,"%d");
 		*bad_request_response_received=1;
 		belle_sip_main_loop_quit(belle_sip_stack_get_main_loop(stack));
 	}
@@ -503,7 +509,7 @@ static void test_bad_request(void) {
 	t=belle_sip_provider_create_client_transaction(prov,req);
 	belle_sip_client_transaction_send_request(t);
 	belle_sip_stack_sleep(stack,3000);
-	BC_ASSERT_TRUE(bad_request_response_received==1);
+	BC_ASSERT_EQUAL(bad_request_response_received,1,int,"%d");
 	belle_sip_provider_remove_sip_listener(prov,bad_req_listener);
 	belle_sip_object_unref(bad_req_listener);
 
@@ -524,15 +530,17 @@ static void test_register_authenticate(void) {
 
 static void test_register_channel_inactive(void){
 	belle_sip_listening_point_t *lp=belle_sip_provider_get_listening_point(prov,"TCP");
-	BC_ASSERT_PTR_NOT_NULL_FATAL(lp);
-	belle_sip_stack_set_inactive_transport_timeout(stack,5);
-	belle_sip_listening_point_clean_channels(lp);
-	BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
-	register_test("tcp",1);
-	BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
-	belle_sip_stack_sleep(stack,5000);
-	BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
-	belle_sip_stack_set_inactive_transport_timeout(stack,3600);
+	BC_ASSERT_PTR_NOT_NULL(lp);
+	if (lp) {
+		belle_sip_stack_set_inactive_transport_timeout(stack,5);
+		belle_sip_listening_point_clean_channels(lp);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
+		register_test("tcp",1);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),1,int,"%d");
+		belle_sip_stack_sleep(stack,5000);
+		BC_ASSERT_EQUAL(belle_sip_listening_point_get_channel_count(lp),0,int,"%d");
+		belle_sip_stack_set_inactive_transport_timeout(stack,3600);
+	}
 }
 
 
@@ -540,9 +548,7 @@ static void test_register_channel_inactive(void){
 static void test_register_client_authenticated(void) {
 	belle_sip_request_t *reg;
 	authorized_request=NULL;
-	/*we don't care to check sercer cert*/
-	belle_sip_tls_listening_point_set_verify_exceptions(	(belle_sip_tls_listening_point_t*)belle_sip_provider_get_listening_point(prov,"tls")
-															,BELLE_SIP_TLS_LISTENING_POINT_BADCERT_ANY_REASON);
+
 	reg=register_user_at_domain(stack, prov, "tls",1,"tester",client_auth_domain,client_auth_outbound_proxy);
 	if (authorized_request) {
 		unregister_user(stack,prov,authorized_request,1);
@@ -555,7 +561,7 @@ static void test_connection_failure(void){
 	belle_sip_request_t *req;
 	io_error_count=0;
 	req=try_register_user_at_domain(stack, prov, "TCP",1,"tester","sip.linphone.org",no_server_running_here,0);
-	BC_ASSERT_TRUE(io_error_count>=1);
+	BC_ASSERT_GREATER(io_error_count,1,int,"%d");
 	if (req) belle_sip_object_unref(req);
 }
 
@@ -591,7 +597,7 @@ static void test_tls_to_tcp(void){
 	belle_sip_stack_set_transport_timeout(stack,2000);
 	req=try_register_user_at_domain(stack, prov, "TLS",1,"tester",test_domain,test_domain_tls_to_tcp,0);
 	if (req){
-		BC_ASSERT_TRUE(io_error_count>=1);
+		BC_ASSERT_GREATER(io_error_count,1,int,"%d");
 		belle_sip_object_unref(req);
 	}
 	belle_sip_stack_set_transport_timeout(stack,orig);
@@ -601,7 +607,7 @@ static void register_dns_srv_tcp(void){
 	belle_sip_request_t *req;
 	io_error_count=0;
 	req=try_register_user_at_domain(stack, prov, "TCP",1,"tester",client_auth_domain,"sip:linphone.net;transport=tcp",1);
-	BC_ASSERT_TRUE(io_error_count==0);
+	BC_ASSERT_EQUAL(io_error_count,0,int,"%d");
 	if (req) belle_sip_object_unref(req);
 }
 
@@ -642,14 +648,15 @@ static void register_dns_load_balancing(void) {
 static void process_message_response_event(void *user_ctx, const belle_sip_response_event_t *event){
 	int status;
 	BELLESIP_UNUSED(user_ctx);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_response_event_get_response(event));
-	belle_sip_message("process_response_event [%i] [%s]"
-					,status=belle_sip_response_get_status_code(belle_sip_response_event_get_response(event))
-					,belle_sip_response_get_reason_phrase(belle_sip_response_event_get_response(event)));
+	if (BC_ASSERT_PTR_NOT_NULL(belle_sip_response_event_get_response(event))) {
+		belle_sip_message("process_response_event [%i] [%s]"
+						,status=belle_sip_response_get_status_code(belle_sip_response_event_get_response(event))
+						,belle_sip_response_get_reason_phrase(belle_sip_response_event_get_response(event)));
 
-	if (status >= 200){
-		is_register_ok=status;
-		belle_sip_main_loop_quit(belle_sip_stack_get_main_loop(stack));
+		if (status >= 200){
+			is_register_ok=status;
+			belle_sip_main_loop_quit(belle_sip_stack_get_main_loop(stack));
+		}
 	}
 }
 static belle_sip_request_t* send_message(belle_sip_request_t *initial_request, const char* realm){
@@ -687,10 +694,9 @@ static belle_sip_request_t* send_message(belle_sip_request_t *initial_request, c
 }
 static void reuse_nonce(void) {
 	belle_sip_request_t *register_request;
-	int initial_auth_context_count=belle_sip_list_size(prov->auth_contexts);
+	size_t initial_auth_context_count=belle_sip_list_size(prov->auth_contexts);
 	register_request=register_user_at_domain(stack, prov, "tcp",1,"marie","sip.linphone.org",NULL);
 	if (register_request) {
-		char * first_nonce_used;
 		belle_sip_header_authorization_t * h = NULL;
 		belle_sip_request_t *message_request;
 		listener_callbacks.process_dialog_terminated=process_dialog_terminated;
@@ -706,18 +712,20 @@ static void reuse_nonce(void) {
 		belle_sip_provider_add_sip_listener(prov,BELLE_SIP_LISTENER(listener));
 
 		/*currently only one nonce should have been used (the one for the REGISTER)*/
-		BC_ASSERT_EQUAL(belle_sip_list_size(prov->auth_contexts), initial_auth_context_count+1,int,"%d");
+		BC_ASSERT_EQUAL((unsigned int)belle_sip_list_size(prov->auth_contexts), (unsigned int)initial_auth_context_count+1,unsigned int,"%u");
 
 		/*this should reuse previous nonce*/
 		message_request=send_message(register_request, auth_domain);
 		BC_ASSERT_EQUAL(is_register_ok, 404,int,"%d");
-		 h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
+		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 					BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 				));
-		BC_ASSERT_PTR_NOT_NULL_FATAL(h);
-		BC_ASSERT_EQUAL(2, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
-		first_nonce_used = belle_sip_strdup(belle_sip_header_authorization_get_nonce(h));
-
+		if (BC_ASSERT_PTR_NOT_NULL(h)) {
+			char * first_nonce_used;
+			BC_ASSERT_EQUAL(2, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
+			first_nonce_used = belle_sip_strdup(belle_sip_header_authorization_get_nonce(h));
+			belle_sip_free(first_nonce_used);
+		}
 		belle_sip_object_unref(message_request);
 
 
@@ -727,7 +735,7 @@ static void reuse_nonce(void) {
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 			));
-		BC_ASSERT_PTR_NULL_FATAL(h);
+		BC_ASSERT_PTR_NULL(h);
 		belle_sip_object_unref(message_request);
 
 
@@ -737,7 +745,7 @@ static void reuse_nonce(void) {
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 			));
-		BC_ASSERT_PTR_NULL_FATAL(h);
+		BC_ASSERT_PTR_NULL(h);
 		belle_sip_object_unref(message_request);
 
 
@@ -747,14 +755,14 @@ static void reuse_nonce(void) {
 		h = BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_message_get_header_by_type(
 				BELLE_SIP_MESSAGE(message_request), belle_sip_header_proxy_authorization_t
 			));
-		BC_ASSERT_PTR_NOT_NULL_FATAL(h);
-		BC_ASSERT_EQUAL(3, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
+		if (BC_ASSERT_PTR_NOT_NULL(h)) {
+			BC_ASSERT_EQUAL(3, belle_sip_header_authorization_get_nonce_count(h),int,"%d");
+		}
 		belle_sip_object_unref(message_request);
 
 		belle_sip_provider_remove_sip_listener(prov,BELLE_SIP_LISTENER(listener));
 		unregister_user(stack,prov,register_request,1);
 		belle_sip_object_unref(register_request);
-		belle_sip_free(first_nonce_used);
 	}
 }
 
@@ -787,6 +795,6 @@ test_t register_tests[] = {
 	{ "Nonce reutilization", reuse_nonce }
 };
 
-test_suite_t register_test_suite = {"REGISTER", register_before_all, register_after_all, NULL,
+test_suite_t register_test_suite = {"Register", register_before_all, register_after_all, NULL,
 									NULL, sizeof(register_tests) / sizeof(register_tests[0]),
 									register_tests};
