@@ -57,7 +57,7 @@ static unsigned int  wait_for(belle_sip_stack_t *stack, int *current_value, int
 	else return TRUE;
 }
 
-static endpoint_t* create_endpoint() {
+static endpoint_t* create_endpoint(void) {
 	endpoint_t* endpoint;
 	endpoint = belle_sip_new0(endpoint_t);
 	endpoint->stack = belle_sip_stack_new(NULL);
@@ -69,7 +69,7 @@ static void reset_endpoint(endpoint_t *endpoint) {
 	endpoint->resolve_done = 0;
 	endpoint->resolve_ko = 0;
 	if (endpoint->ai_list != NULL) {
-		belle_sip_freeaddrinfo(endpoint->ai_list);
+		bctbx_freeaddrinfo(endpoint->ai_list);
 		endpoint->ai_list = NULL;
 	}
 	if (endpoint->srv_list != NULL) {
@@ -113,7 +113,7 @@ static void ipv4_a_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -123,10 +123,10 @@ static void ipv4_a_query(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -140,7 +140,7 @@ static void ipv4_cname_a_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_CNAME, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -150,10 +150,10 @@ static void ipv4_cname_a_query(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_CNAME_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_CNAME_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -164,7 +164,7 @@ static void local_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, "localhost", SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
@@ -183,7 +183,7 @@ static void ipv4_a_query_no_result(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_BAD_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -197,7 +197,7 @@ static void ipv4_a_query_no_result(void) {
 static void ipv4_a_query_send_failure(void) {
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	belle_sip_stack_set_resolver_send_error(client->stack, -1);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NULL(client->resolver_ctx);
@@ -211,7 +211,7 @@ static void ipv4_a_query_timeout(void) {
 
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	belle_sip_stack_set_dns_timeout(client->stack, 0);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, "toto.com", SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -226,7 +226,7 @@ static void ipv4_a_query_multiple_results(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_MULTIRES_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -250,7 +250,7 @@ static void ipv4_a_query_with_v4mapped_results(void) {
 
 	client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET6, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -273,7 +273,7 @@ static void ipv6_aaaa_query(void) {
 	}
 
 	client = create_endpoint();
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV6_SIP_DOMAIN, SIP_PORT, AF_INET6, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
@@ -285,9 +285,9 @@ static void ipv6_aaaa_query(void) {
 		int ntohsi = ntohs(sock_in6->sin6_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
 		/*the IPv6 address shall return first, and must be a real ipv6 address*/
-		BC_ASSERT_TRUE(client->ai_list->ai_family==AF_INET6);
+		BC_ASSERT_EQUAL(client->ai_list->ai_family,AF_INET6,int,"%d");
 		BC_ASSERT_FALSE(IN6_IS_ADDR_V4MAPPED(&sock_in6->sin6_addr));
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET6, IPV6_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET6, SOCK_STREAM, IPV6_SIP_IP, SIP_PORT);
 		BC_ASSERT_PTR_NOT_NULL(ai);
 		if (ai) {
 			struct in6_addr *ipv6_address = &((struct sockaddr_in6 *)ai->ai_addr)->sin6_addr;
@@ -295,17 +295,17 @@ static void ipv6_aaaa_query(void) {
 			for (i = 0; i < 8; i++) {
 				BC_ASSERT_EQUAL(sock_in6->sin6_addr.s6_addr[i], ipv6_address->s6_addr[i], int, "%d");
 			}
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 		next=client->ai_list->ai_next;
 		BC_ASSERT_PTR_NOT_NULL(next);
 		if (next){
 			int ntohsi = ntohs(sock_in6->sin6_port);
 			sock_in6 = (struct sockaddr_in6 *)next->ai_addr;
-			BC_ASSERT_TRUE(next->ai_family==AF_INET6);
+			BC_ASSERT_EQUAL(next->ai_family,AF_INET6,int,"%d");
 			BC_ASSERT_TRUE(IN6_IS_ADDR_V4MAPPED(&sock_in6->sin6_addr));
 			BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-			ai = belle_sip_ip_address_to_addrinfo(AF_INET6, IPV6_SIP_IPV4, SIP_PORT);
+			ai = bctbx_ip_address_to_addrinfo(AF_INET6, SOCK_STREAM, IPV6_SIP_IPV4, SIP_PORT);
 			BC_ASSERT_PTR_NOT_NULL(ai);
 			if (ai) {
 				struct in6_addr *ipv6_address = &((struct sockaddr_in6 *)ai->ai_addr)->sin6_addr;
@@ -313,7 +313,7 @@ static void ipv6_aaaa_query(void) {
 				for (i = 0; i < 8; i++) {
 					BC_ASSERT_EQUAL(sock_in6->sin6_addr.s6_addr[i], ipv6_address->s6_addr[i], int, "%d");
 				}
-				belle_sip_freeaddrinfo(ai);
+				bctbx_freeaddrinfo(ai);
 			}
 		}
 	}
@@ -325,13 +325,13 @@ static void srv_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
-	client->resolver_ctx = belle_sip_stack_resolve_srv(client->stack, "udp", SRV_DOMAIN, srv_resolve_done, client);
+	client->resolver_ctx = belle_sip_stack_resolve_srv(client->stack, "sip", "udp", SRV_DOMAIN, srv_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
 	BC_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
 	BC_ASSERT_PTR_NOT_NULL(client->srv_list);
-	BC_ASSERT_NOT_EQUAL(belle_sip_list_size(client->srv_list), 0,int,"%d");
+	BC_ASSERT_NOT_EQUAL((unsigned int)belle_sip_list_size(client->srv_list), 0,unsigned int,"%u");
 	if (client->srv_list && (belle_sip_list_size(client->srv_list) > 0)) {
 		belle_sip_dns_srv_t *result_srv = belle_sip_list_nth_data(client->srv_list, 0);
 		BC_ASSERT_EQUAL(belle_sip_dns_srv_get_port(result_srv), SIP_PORT, int, "%d");
@@ -345,9 +345,9 @@ static void srv_a_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
-	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "udp", SRV_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
+	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "sip", "udp", SRV_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
 	BC_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
 	BC_ASSERT_PTR_NOT_EQUAL(client->ai_list, NULL);
@@ -361,9 +361,9 @@ static void srv_a_query_no_srv_result(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
-	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "udp", IPV4_CNAME, SIP_PORT, AF_INET, a_resolve_done, client);
+	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "sip", "udp", IPV4_CNAME, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
 	BC_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
 	BC_ASSERT_PTR_NOT_NULL(client->ai_list);
@@ -371,10 +371,10 @@ static void srv_a_query_no_srv_result(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_CNAME_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_CNAME_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -385,9 +385,9 @@ static void local_full_query(void) {
 	int timeout;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
-	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "tcp", "localhost", SIP_PORT, AF_INET, a_resolve_done, client);
+	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "sip", "tcp", "localhost", SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NOT_NULL(client->resolver_ctx);
 	BC_ASSERT_TRUE(wait_for(client->stack, &client->resolve_done, 1, timeout));
 	BC_ASSERT_PTR_NOT_EQUAL(client->ai_list, NULL);
@@ -404,8 +404,8 @@ static void no_query_needed(void) {
 	struct addrinfo *ai;
 	endpoint_t *client = create_endpoint();
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
-	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "udp", IPV4_SIP_IP, SIP_PORT, AF_INET, a_resolve_done, client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
+	client->resolver_ctx = belle_sip_stack_resolve(client->stack, "sip", "udp", IPV4_SIP_IP, SIP_PORT, AF_INET, a_resolve_done, client);
 	BC_ASSERT_PTR_NULL(client->resolver_ctx);
 	BC_ASSERT_TRUE(client->resolve_done);
 	BC_ASSERT_PTR_NOT_EQUAL(client->ai_list, NULL);
@@ -413,10 +413,10 @@ static void no_query_needed(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -450,7 +450,7 @@ static void dns_fallback(void) {
 		NULL
 	};
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	set_custom_resolv_conf(client->stack,nameservers);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
@@ -461,10 +461,10 @@ static void dns_fallback(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -486,7 +486,7 @@ static void ipv6_dns_server(void) {
 	}
 
 	client = create_endpoint();
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	set_custom_resolv_conf(client->stack,nameservers);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
@@ -497,10 +497,10 @@ static void ipv6_dns_server(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
@@ -522,7 +522,7 @@ static void ipv4_and_ipv6_dns_server(void) {
 		return;
 	}
 	client = create_endpoint();
-	BC_ASSERT_PTR_NOT_NULL_FATAL(client);
+	if (!BC_ASSERT_PTR_NOT_NULL(client)) return;
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	set_custom_resolv_conf(client->stack,nameservers);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV4_SIP_DOMAIN, SIP_PORT, AF_INET, a_resolve_done, client);
@@ -533,10 +533,10 @@ static void ipv4_and_ipv6_dns_server(void) {
 		struct sockaddr_in *sock_in = (struct sockaddr_in *)client->ai_list->ai_addr;
 		int ntohsi = (int)ntohs(sock_in->sin_port);
 		BC_ASSERT_EQUAL(ntohsi, SIP_PORT, int, "%d");
-		ai = belle_sip_ip_address_to_addrinfo(AF_INET, IPV4_SIP_IP, SIP_PORT);
+		ai = bctbx_ip_address_to_addrinfo(AF_INET, SOCK_STREAM, IPV4_SIP_IP, SIP_PORT);
 		if (ai) {
 			BC_ASSERT_EQUAL(sock_in->sin_addr.s_addr, ((struct sockaddr_in *)ai->ai_addr)->sin_addr.s_addr, int, "%d");
-			belle_sip_freeaddrinfo(ai);
+			bctbx_freeaddrinfo(ai);
 		}
 	}
 
