@@ -19,7 +19,11 @@
 #ifndef _BELLE_SIP_TESTER_H
 #define _BELLE_SIP_TESTER_H
 
-#include "bc_tester_utils.h"
+#include <bctoolbox/tester.h>
+
+
+#define MULTIPART_BOUNDARY "---------------------------14737809831466499882746641449"
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -49,8 +53,8 @@ const char * belle_sip_tester_get_root_ca_path(void);
 void belle_sip_tester_set_root_ca_path(const char *root_ca_path);
 void belle_sip_tester_init(void(*ftester_printf)(int level, const char *fmt, va_list args));
 void belle_sip_tester_uninit(void);
-void belle_sip_tester_before_each();
-void belle_sip_tester_after_each();
+void belle_sip_tester_before_each(void);
+int belle_sip_tester_after_each(void);
 int belle_sip_tester_set_log_file(const char *filename);
 
 #ifdef __cplusplus
