@@ -28,12 +28,12 @@ static void cast_test(void){
 	belle_sip_message_t *msg;
 	int tmp;
 
-	BC_ASSERT_PTR_NOT_NULL_FATAL(stack);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(lp);
+	BC_ASSERT_PTR_NOT_NULL(stack);
+	BC_ASSERT_PTR_NOT_NULL(lp);
 	provider=belle_sip_stack_create_provider(stack,lp);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(provider);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(req);
-	BC_ASSERT_PTR_NOT_NULL_FATAL(resp);
+	BC_ASSERT_PTR_NOT_NULL(provider);
+	BC_ASSERT_PTR_NOT_NULL(req);
+	BC_ASSERT_PTR_NOT_NULL(resp);
 
 	belle_sip_message("Casting belle_sip_request_t to belle_sip_message_t");
 	msg=BELLE_SIP_MESSAGE(req);
